@@ -1,4 +1,6 @@
-;	$B2x$7$$$1$I%7%9%F%`JQ?t$N=i4|2=!#(B
+;; $B$3$N(B init.el $B$O(B emacs-20.7 $B$K4^$^$l$k(B emacs-lisp $B725Z$S(B skk8.6 $B$N(B
+;; skk.el $B$+$iH4?h!"MxMQ$7$F$$$kItJ,$,$"$j$^$9!#(B
+;; $B$3$l$i$N(B copyright $B$O(B emacs, emacs-lisp $B:n<T5Z$S(B skk $B:n<T$K$"$j$^$9!#(B
 
 (setq global-map '(keymap
  [set-mark-command		beginning-of-line	backward-char
@@ -82,13 +84,13 @@
 (defmacro unless (cond &rest body)
   (cons 'if (cons cond (cons nil body))))
 
-(defun backward-word (arg)
-  (interactive "vSymbol: ")
-  (forward-char))
-
 ;(defun backward-word (arg)
-;  (interactive "p")
-;  (forward-word (- arg)))
+;  (interactive "vSymbol: ")
+;  (forward-char))
+
+(defun backward-word (arg)
+  (interactive "p")
+  (forward-word (- arg)))
 
 (defun kill-word (arg)
   (interactive "p")
