@@ -154,7 +154,7 @@ void
 TLispClient_ClassFinalize (
 	register TLispMachine*	pLM)
 {
-	static const char		sstrEND []	= "(mutual-eval skk-jisyo-mutex '(skk-save-jisyo))" ;
+	static const char		sstrEND []	= "(mutual-eval skk-jisyo-mutex '(progn (setq this-command 'save-buffers-kill-emacs) (skk-save-jisyo)))" ;
 	register TLispManager*	pLispMgr ;
 	register TLispEntity*	pEntTarget ;
 
