@@ -106,7 +106,8 @@ lispMgr_CreateBuffer (
 	/*	󥰥ꥹȤˤʤäƤ롣*/
 	pBuffer->m_pNextBuffer		= pEntity ;
 	pBuffer->m_pPrevBuffer		= pEntity ;
-	pBuffer->m_pEntKeymap		= NULL ;
+	pBuffer->m_pEntKeymap		= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_NIL) ;
+	lispEntity_AddRef (pLispMgr, pBuffer->m_pEntKeymap) ;
 	pBuffer->m_pEntName			= NULL ;
 	pBuffer->m_fModified		= False ;
 	pBuffer->m_nTick			= 0 ;
@@ -1070,6 +1071,21 @@ lispBuffer_MoveMarkerWithDeletion (
 	return	True ;
 }
 
+void
+lispBuffer_Desync (
+	register TLispManager*	pLispMgr,
+	register TLispEntity*	pEntBuffer)
+{
+	register TLispBuffer*	pBuffer ;
+	
+	assert (pLispMgr   != NULL) ;
+	assert (pEntBuffer != NULL) ;
+
+	pBuffer		= lispEntity_GetBufferPtr (pEntBuffer) ;
+	pBuffer->m_nTick	++ ;
+	return ;
+}
+
 /*
  *	ǥХåѤδؿ
  *	ХåեȤɽ롣
