@@ -361,17 +361,41 @@ lispEntity_Sequencep (
 			 lispEntity_Arrayp (pLispMgr, pEntity)) ;
 }
 
+extern inline	TLispEntity*
+lispEntity_XEvent2Integer (
+	register TLispEntity*		pEntity)
+{
+	assert (pEntity != NULL) ;
+	assert (pEntity->m_iType == LISPENTITY_XEVENT) ;
+	return	(lispEntity_GetXEventPtr (pEntity))->m_pEntValue ;
+}
+
 extern inline	Boolean
 lispEntity_Eq (
 	register TLispManager*	pLispMgr,
-	register TLispEntity*	pLeftEntity,
-	register TLispEntity*	pRightEntity)
+	register TLispEntity*	pEntLeft,
+	register TLispEntity*	pEntRight)
 {
-	assert (pLispMgr     != NULL) ;
-	assert (pLeftEntity  != NULL) ;
-	assert (pRightEntity != NULL) ;
-	return	(pLeftEntity == pRightEntity)? True : False ;
+#if 0
+	assert (pLispMgr  != NULL) ;
+	assert (pEntLeft  != NULL) ;
+	assert (pEntRight != NULL) ;
+	return	(pEntLeft == pEntRight)? True : False ;
+#else
+	register TLispEntity*	pEntLeftCmp ;
+	register TLispEntity*	pEntRightCmp ;
 
+	assert (pLispMgr  != NULL) ;
+	assert (pEntLeft  != NULL) ;
+	assert (pEntRight != NULL) ;
+
+	/*	ʤκXEVENT ENTITY  Integer Ȥơeq 褦ˤ롣
+	 *	ޤɬפˤʤΤ im-send ۤˤλ顣
+	 */
+	pEntLeftCmp		= (pEntLeft->m_iType != LISPENTITY_XEVENT)? pEntLeft : lispEntity_XEvent2Integer (pEntLeft) ;
+	pEntRightCmp	= (pEntRight->m_iType != LISPENTITY_XEVENT)? pEntRight : lispEntity_XEvent2Integer (pEntRight) ;
+	return	pEntLeftCmp == pEntRightCmp ;
+#endif
 	UNREFERENCED_PARAMETER (pLispMgr) ;
 }
 
