@@ -1305,6 +1305,21 @@ lispEntity_printXEvent (
 	register TLispManager*	pLispMgr,
 	register TLispEntity*	pEntity)
 {
+	XEvent*	pEv ;
+
+	if (TFAILED (lispEntity_GetXEvent (pLispMgr, pEntity, &pEv)))
+		return	False ;
+
+	switch (pEv->type) {
+	case	KeyPress:
+		fprintf (stderr, "#<xkeyevent: type(%d), window(%lx), keycode(%d)>",
+				 pEv->type, pEv->xkey.window, pEv->xkey.keycode) ;
+		break ;
+	default:
+		fprintf (stderr, "#<xevent: type(%d), window(%lx)>",
+				 pEv->type, pEv->xany.window) ;
+		break ;
+	}
 	return	True ;
 }
 
@@ -1336,15 +1351,23 @@ lispEntity_formatString (
 	TVarbuffer				vbuf ;
 	register Boolean			fRetval	= False ;
 	register const Char*	pString ;
-	register int			nUsage ;
+	register int			nUsage, nOffset ;
 
 	if (TFAILED (TVarbuffer_Initialize (&vbuf, sizeof (Char))))
 		return	False ;
 	if (TFAILED (lispEntity_PrincStr (pLispMgr, pEntity, &vbuf))) 
 		goto	error ;
 
-	nUsage	= TVarbuffer_GetUsage (&vbuf) - 2 ;
-	if (0 < nCount && nUsage < nCount) {
+	/*	string äˤϡ double quote ĤΤ
+	 *	򳰤Τ -2 롣*/
+	nUsage	= TVarbuffer_GetUsage (&vbuf) ;
+	nOffset	= 0 ;
+	if (TSUCCEEDED (lispEntity_Stringp (pLispMgr, pEntity))) {
+		assert (nUsage >= 2) ;
+		nUsage	-= 2 ;
+		nOffset	++ ;
+	}
+	if (0 < nCount && 0 <= nUsage && nUsage < nCount) {
 		register int		nSpace	= nUsage - nCount ;
 		static const Char	cc		= ' ' ;
 		while (nSpace > 0) {
@@ -1353,13 +1376,15 @@ lispEntity_formatString (
 			nSpace	-- ;
 		}
 	}
-	pString	= (const Char *)TVarbuffer_GetBuffer (&vbuf) + 1 ;
-	fRetval	= TVarbuffer_Add (pvbuf, pString, nUsage) ;
+	if (nUsage > 0) {
+		pString	= (const Char *)TVarbuffer_GetBuffer (&vbuf) + nOffset ;
+		fRetval	= TVarbuffer_Add (pvbuf, pString, nUsage) ;
+	} else {
+		fRetval	= True ;
+	}
   error:
 	TVarbuffer_Uninitialize (&vbuf) ;
 	return	fRetval ;
-
-
 }
 
 Boolean
@@ -1390,7 +1415,7 @@ lispEntity_formatNumber (
 	assert (pEntity  != NULL) ;
 	assert (pvbuf    != NULL) ;
 
-	if (nFormat	>= sizeof (achFormat))
+	if (nFormat	>= NELEMENTS (achFormat))
 		return	False ;
 	cstrtostr (achFormat, pFormat, nFormat) ;
 	achFormat [nFormat]	= '\0' ;
@@ -1412,7 +1437,7 @@ lispEntity_formatNumberA (
 	assert (pEntity  != NULL) ;
 	assert (pvbuf    != NULL) ;
 
-	if (nFormat	>= sizeof (achFormat))
+	if (nFormat	>= NELEMENTS (achFormat))
 		return	False ;
 	memcpy (achFormat, pFormat, nFormat) ;
 	achFormat [nFormat]	= '\0' ;
