@@ -212,50 +212,32 @@ lispMgr_initialize (
 {
 	/*	졹ľʤƺѤ褦ϿƤ*/
 	static const char*	apReservedSymbols [LISPMGR_SIZE_RESERVED]	= {
-		"nil",
-		"t",
-		"-",
-		"lambda",
-		"macro",
-		"&optional",
-		"&rest",
-		"keymap",
-		"error",
-		"exit",
-		"quit",
-		"quit-flag",
-		"inhibit-quit",
-		"features",
-		"interactive",
-		"this-command",
-		"last-command",
-		"unread-command-events",
-		"pre-command-hook",
-		"post-command-hook",
-		"last-command-event",
-		"unread-command-char",
-		"global-map",
-		"minibuffer-local-map",
-		"minor-mode-map-alist",
-		"wrong-type-argument",
-		"buffer-file-name",
-		"mode-line-format",
-		"load-path",
-		"kill-ring",
-		"kill-region",
-		"kill-ring-yank-pointer",
-		"yank",
+		"nil",					"t",					"-",
+		"lambda",				"macro",				"&optional",
+		"&rest",				"keymap",				"error",
+		"exit",					"quit",					"quit-flag",
+		"inhibit-quit",			"features",				"interactive",
+		"this-command",			"last-command",			"unread-command-events",
+		"pre-command-hook",		"post-command-hook",	"last-command-char",
+		"last-command-event",	"unread-command-char",	"global-map",
+		"minibuffer-local-map",	"minor-mode-map-alist",	"wrong-type-argument",
+		"buffer-file-name",		"mode-line-format",		"load-path",
+		"kill-ring",			"kill-region",			"kill-ring-yank-pointer",
+		"yank",				
+
+		"coding-system-for-read",	"coding-system-for-write",	"euc-japan",
+		"euc-jp",				"shift-jis",			"japanese-shift-jis",
+		"sjis",					"junet",				"iso-2022-jp",
+		"iso-2022-jp-2",		"ctext",				"x-ctext",
+		"compound-text",	
 
 		/*	for skk */
-		"skk-server-host",
-		"skk-server-list",
-		"skk-portnum",
-		"skk-process-okuri-early",
-		"j-henkan-key",
-		"j-search-key",
-		"j-okuri-ari",
-		"j-henkan-okurigana",
-		"j-henkan-okuri-strictly",
+		"skk-server-host",		"skk-server-list",		"skk-portnum",
+		"skk-process-okuri-early",	"j-henkan-key",		"j-search-key",
+		"j-okuri-ari",			"j-henkan-okurigana",	"j-henkan-okuri-strictly",
+
+		/* for skk10 */
+		"skk-servers-list",
 	} ;
 	/*	default  buffer-local-symbol ȤѰդ륷ܥ롣
 	 */
