@@ -62,6 +62,7 @@ enum {
 	LISPMGR_INDEX_UNREAD_COMMAND_EVENTS,
 	LISPMGR_INDEX_PRE_COMMAND_HOOK,
 	LISPMGR_INDEX_POST_COMMAND_HOOK,
+ 	LISPMGR_INDEX_LAST_COMMAND_CHAR,
 	LISPMGR_INDEX_LAST_COMMAND_EVENT,
 	LISPMGR_INDEX_UNREAD_COMMAND_CHAR,
 	LISPMGR_INDEX_GLOBAL_MAP,
@@ -76,6 +77,20 @@ enum {
 	LISPMGR_INDEX_KILL_RING_YANK_POINTER,
 	LISPMGR_INDEX_YANK,
 
+	LISPMGR_INDEX_CODING_SYSTEM_FOR_READ,
+	LISPMGR_INDEX_CODING_SYSTEM_FOR_WRITE,
+	LISPMGR_INDEX_EUC_JAPAN,
+	LISPMGR_INDEX_EUC_JP,
+	LISPMGR_INDEX_SHIFT_JIS,
+	LISPMGR_INDEX_JAPANESE_SHIFT_JIS,
+	LISPMGR_INDEX_SJIS,
+	LISPMGR_INDEX_JUNET,
+	LISPMGR_INDEX_ISO2022JP,
+	LISPMGR_INDEX_ISO2022JP2,
+	LISPMGR_INDEX_CTEXT,
+	LISPMGR_INDEX_X_CTEXT,
+	LISPMGR_INDEX_COMPOUND_TEXT,
+
 	LISPMGR_INDEX_SKK_SERVER_HOST,
 	LISPMGR_INDEX_SKK_SERVER_LIST,
 	LISPMGR_INDEX_SKK_PORTNUM,
@@ -86,6 +101,8 @@ enum {
 	LISPMGR_INDEX_J_HENKAN_OKURIGANA,
 	LISPMGR_INDEX_J_HENKAN_OKURI_STRICTLY,
 
+	LISPMGR_INDEX_SKK_SERVERS_LIST,
+
 	LISPMGR_SIZE_RESERVED,
 } ;
 
@@ -166,6 +183,7 @@ Boolean	lispMgr_CollectSymbolGarbage	(TLispManager*) ;
 Boolean	lispMgr_SearchSymbol			(TLispManager*, const Char*, int, TLispEntity** const) ;
 Boolean	lispMgr_SearchSymbolA			(TLispManager*, const char*, int, TLispEntity** const) ;
 Boolean	lispMgr_UnregisterSymbol		(TLispManager*, TLispEntity*) ;
+Boolean	lispMgr_SymbolRequireEvalp		(TLispManager*, TLispEntity*) ;
 
 /*	lentity.c */
 Boolean	lispEntity_Memq					(TLispManager*, TLispEntity*, TLispEntity*, TLispEntity**) ;
@@ -192,9 +210,11 @@ Boolean	lispEntity_Downcase				(TLispManager*, TLispEntity*, TLispEntity** const
 /*	lisplist.c	*/
 Boolean	lispMgr_CreateConscell			(TLispManager*, TLispEntity*, TLispEntity*, TLispEntity** const) ;
 Boolean	lispMgr_CreateList				(TLispManager*, TLispEntity** const, int, TLispEntity** const) ;
+Boolean	lispMgr_CreateTime				(TLispManager*, time_t, TLispEntity** const) ;
 Boolean	lispEntity_PlistGet				(TLispManager*, TLispEntity*, TLispEntity*, TLispEntity** const) ;
 Boolean	lispEntity_PlistPut				(TLispManager*, TLispEntity*, TLispEntity*, TLispEntity*, TLispEntity** const) ;
 Boolean	lispEntity_CopyList				(TLispManager*, TLispEntity*, TLispEntity** const) ;
+Boolean	lispEntity_Push2List			(TLispManager*, TLispConscell*, TLispEntity*) ;
 
 /*	lnum.c	*/
 Boolean	lispMgr_CreateInteger			(TLispManager*, long, TLispEntity** const) ;
@@ -285,6 +305,7 @@ Boolean	lispBuffer_MoveMarkerWithDeletion (TLispManager*, TLispEntity*, int, int
 Boolean	lispBuffer_Print				(TLispManager*, TLispEntity*) ;
 Boolean	lispBuffer_GetModifiedp			(TLispManager*, TLispEntity*) ;
 void	lispBuffer_SetModifiedp			(TLispManager*, TLispEntity*, Boolean) ;
+void	lispBuffer_Desync				(TLispManager*, TLispEntity*) ;
 
 /*	lispwindow.c */
 Boolean	lispMgr_CreateWindow			(TLispManager*, Boolean, TLispEntity** const) ;
