@@ -94,8 +94,10 @@ lispMgr_CreateSparseKeymap (
 	apEntNode [0]	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_KEYMAP) ;
 	lispEntity_AddRef (pLispMgr, apEntNode [nNode ++]) ;
 	if (pEntString != NULL &&
-		TFAILED (lispEntity_Nullp (pLispMgr, pEntString)))
+		TFAILED (lispEntity_Nullp (pLispMgr, pEntString))) {
+		lispEntity_AddRef (pLispMgr, pEntString) ;
 		apEntNode [nNode ++]	= pEntString ;
+	}
 	fRetval	= lispMgr_CreateList (pLispMgr, apEntNode, nNode, ppEntReturn) ;
 	while (nNode > 0)
 		lispEntity_Release (pLispMgr, apEntNode [-- nNode]) ;
