@@ -103,6 +103,35 @@ lispMgr_CreateList (
 	return	True ;
 }
 
+Boolean
+lispMgr_CreateTime (
+	register TLispManager*			pLispMgr,
+	register time_t					tmValue,
+	register TLispEntity** const	ppEntReturn)
+{
+	TLispEntity*			arEntity [2] ;
+	register Boolean		fRetval	= False ;
+
+	assert (pLispMgr != NULL) ;
+	assert (ppEntReturn != NULL) ;
+
+	arEntity [0]	= NULL ;
+	arEntity [1]	= NULL ;
+	if (TFAILED (lispMgr_CreateInteger (pLispMgr, (tmValue >> 16) & 0xFFFF, &arEntity [0])))
+		goto	exit_func ;
+	lispEntity_AddRef (pLispMgr, arEntity [0]) ;
+	if (TFAILED (lispMgr_CreateInteger (pLispMgr, (tmValue >>  0) & 0xFFFF, &arEntity [1])))
+		goto	exit_func ;
+	lispEntity_AddRef (pLispMgr, arEntity [1]) ;
+	fRetval	= lispMgr_CreateList (pLispMgr, arEntity, 2, ppEntReturn) ;
+  exit_func:
+	if (arEntity [1] != NULL)
+		lispEntity_Release (pLispMgr, arEntity [1]) ;
+	if (arEntity [0] != NULL)
+		lispEntity_Release (pLispMgr, arEntity [0]) ;
+	return	fRetval ;
+}
+
 Boolean
 lispEntity_PlistGet (
 	register TLispManager*			pLispMgr,
@@ -235,3 +264,26 @@ lispEntity_CopyList (
 	return	False ;
 }
 
+Boolean
+lispEntity_Push2List (
+	register TLispManager*	pLispMgr,
+	register TLispConscell*	pList,
+	register TLispEntity*	pEntElement)
+{
+	register TLispEntity*	pEntNil ;
+	TLispEntity*			pEntNode ;
+
+	pEntNil	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_NIL) ;
+	if (TFAILED (lispMgr_CreateConscell (pLispMgr, pEntElement, pEntNil, &pEntNode)))
+		return	False ;
+	if (pList->m_pCar == NULL) {
+		pList->m_pCar	= pEntNode ;
+		pList->m_pCdr	= pEntNode ;
+		lispEntity_AddRef (pLispMgr, pEntNode) ;
+	} else {
+		lispEntity_SetCdr (pLispMgr, pList->m_pCdr, pEntNode) ;
+		pList->m_pCdr	= pEntNode ;
+	}
+	return	True ;
+}
+
