@@ -26,6 +26,7 @@
 #include "lmachinep.h"
 #include "cstring.h"
 
+static	Boolean		lispMachine_initialize		(TLispMachine*) ;
 static	Boolean		lispMachine_onKeyPress		(TLispMachine*, void*, void*) ;
 static	Boolean		lispMachine_onFrameResize	(TLispMachine*, void*, void*) ;
 static	Boolean		lispMachine_keyEvent2lispEntity	(TLispMachine*, void*, TLispEntity**) ;
@@ -105,6 +106,8 @@ TLispMachine_Create (
 		pLM->m_alVREGS [i].m_lValue	= 0 ;
 	for (i = 0 ; i < NELEMENTS (pLM->m_apVariableTable) ; i ++) 
 		pLM->m_apVariableTable [i]	= NULL ;
+ 	for (i = 0 ; i < NELEMENTS (pLM->m_apSymbol2SymbolTable) ; i ++) 
+ 		pLM->m_apSymbol2SymbolTable [i]	= NULL ;
 	for (i = 0 ; i < NELEMENTS (pLM->m_apFunctionTable) ; i ++) 
 		pLM->m_apFunctionTable [i]	= NULL ;
 	for (i = 0 ; i < NELEMENTS (pLM->m_apPropertyTable) ; i ++) 
@@ -122,6 +125,7 @@ TLispMachine_Create (
 	lispMachineCode_ResetSignal    (pLM) ;
 	lispMachineCode_ResetException (pLM) ;
 	lispMachineCode_ClearQuitFlag  (pLM) ;
+ 	lispMachine_initialize (pLM) ;
 	return	True ;
 }
 
@@ -132,6 +136,13 @@ TLispMachine_Destroy (
 	register TLispManager*	pLispMgr ;
 	register TLispEntity*	pEntity ;
 	register int			i ;
+	static int				srEntityOffsets []	= {
+		offsetof (TLispMachine, m_pEntRegMatch),
+		offsetof (TLispMachine, m_pEntSignal),
+		offsetof (TLispMachine, m_pEntSignalValue),
+		offsetof (TLispMachine, m_pEntException),
+		offsetof (TLispMachine, m_pEntExceptionValue),
+	} ;
 
 	assert (pLM != NULL) ;
 
@@ -143,6 +154,14 @@ TLispMachine_Destroy (
 			pLM->m_apLREGS [i]	= NULL ;
 		}
 	}
+	for (i = 0 ; i < NELEMENTS (srEntityOffsets) ; i ++) {
+		register TLispEntity**	ppEntity ;
+		ppEntity	= (TLispEntity **)((unsigned char*)pLM + srEntityOffsets [i]) ;
+		if (*ppEntity != NULL) {
+			lispEntity_Release (pLispMgr, *ppEntity) ;
+			*ppEntity	= NULL ;
+		}
+	}
 	if (pLM->m_lstBuffer != NULL) {
 		register Boolean	fExit	= False ;
 		TLispEntity*		pNextBuffer	;
@@ -178,9 +197,10 @@ TLispMachine_Destroy (
 	 *	Ǥʤʤ롣*/
 	lispMgr_AbondonMutex (pLispMgr, pLM) ;
 
-	lispBindTable_Destroy (pLispMgr, pLM->m_apVariableTable, SIZE_LISP_BIND_TABLE) ;
-	lispBindTable_Destroy (pLispMgr, pLM->m_apFunctionTable, SIZE_LISP_BIND_TABLE) ;
-	lispBindTable_Destroy (pLispMgr, pLM->m_apPropertyTable, SIZE_LISP_BIND_TABLE) ;
+	lispBindTable_Destroy (pLispMgr, pLM->m_apVariableTable,      SIZE_LISP_BIND_TABLE) ;
+	lispBindTable_Destroy (pLispMgr, pLM->m_apSymbol2SymbolTable, SIZE_LISP_BIND_TABLE) ;
+	lispBindTable_Destroy (pLispMgr, pLM->m_apFunctionTable,      SIZE_LISP_BIND_TABLE) ;
+	lispBindTable_Destroy (pLispMgr, pLM->m_apPropertyTable,      SIZE_LISP_BIND_TABLE) ;
 	Vstack_Uninitialize (&pLM->m_vstLispObj) ;
 	Vstack_Uninitialize (&pLM->m_vstNonLispObj) ;
 	TVarbuffer_Uninitialize (&pLM->m_vbufCmdEvents) ;
@@ -202,7 +222,7 @@ TLispMachine_Test (
 	do {
 		res	= lispMachine_ExecuteLoop (pLM) ; 
 	}	while (res == LMR_TICK) ;
-#if defined (DEBUG)
+#if defined (DEBUG) || defined (DEBUG_CONSOLE)
 	if (res == LMR_DONE) {
 		TLispEntity*	pValue ;
 
@@ -699,6 +719,72 @@ lispMachine_GenerateNewBufferName (
 	return	fRetval ;
 }
 
+Boolean
+lispMachine_EnumBuffer (
+	register TLispMachine*	pLM,
+	register Boolean		fStartCurrentBufferp,
+	register Boolean		(*pEnumProc)(TLispMachine*, TLispEntity*, void*, Boolean*),
+	register void*			pCaller)
+{
+	register TLispEntity*	pEntBuffer ;
+	register TLispEntity*	pEntTop ;
+
+	assert (pLM       != NULL) ;
+	assert (pEnumProc != NULL) ;
+
+	pEntBuffer	= (fStartCurrentBufferp)? pLM->m_pCurBuffer : pLM->m_lstBuffer ;
+	pEntTop		= pEntBuffer ;
+	if (pEntBuffer != NULL) {
+		TLispEntity*	pEntNext ;
+		Boolean			fContinue	= True ;
+
+		do {
+			if (TFAILED ((pEnumProc)(pLM, pEntBuffer, pCaller, &fContinue)))
+				return	False ;
+			if (TFAILED (fContinue))
+				break ;
+			lispBuffer_GetNext (pEntBuffer, &pEntNext) ;
+			pEntBuffer	= pEntNext ;
+		}	while (pEntBuffer != pEntTop) ;
+	}
+	return	True ;
+}
+
+/*========================================================================*
+ *	private functions
+ */
+Boolean
+lispMachine_initialize (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr ;
+	static struct {
+		int		m_nSrc ;
+		int		m_nDest ;
+	}	srEquivalentSymbols []	= {
+		{	LISPMGR_INDEX_LAST_COMMAND_EVENT,	LISPMGR_INDEX_LAST_COMMAND_CHAR, },
+	} ;
+	register TLispEntity*	pEntSrc ;
+	register TLispEntity*	pEntDest ;
+	register int			i ;
+	register TLispBind**	pBindTable ;
+	TLispBind*				pBind ;
+
+	/*	̾ƱưܥϿ롣
+	 */
+	pLispMgr	= pLM->m_pLispMgr ;
+	pBindTable	= pLM->m_apSymbol2SymbolTable ;
+	for (i = 0 ; i < NELEMENTS (srEquivalentSymbols) ; i ++) {
+		pEntSrc		= lispMgr_GetReservedEntity (pLispMgr, srEquivalentSymbols [i].m_nSrc) ;
+		pEntDest	= lispMgr_GetReservedEntity (pLispMgr, srEquivalentSymbols [i].m_nDest) ;
+		if (TFAILED (lispBindTable_MakeEntry (pLispMgr, pBindTable, NELEMENTS (pLM->m_apSymbol2SymbolTable), pEntSrc, &pBind)))
+			return	False ;
+		if (TFAILED (lispBind_SetValue (pLispMgr, pBind, pEntDest)))
+			return	False ;
+	}
+	return	True ;
+}
+
 /*	ʸɤ꿶ʤäʸɤϳ꿶줿ɤ⡢Ʊ
  *	˲Ƥ륭(status)ʸɤ˱ƶʤäˤϡüʥ
  *	ˤɬפ롣
@@ -849,10 +935,6 @@ lispMachine_keyEvent2lispEntity (
 		register char			chOrig		= aBuf [0] ;
 		register int			i ;
 
-#if defined (DEBUG)
-		fprintf (stderr, "event state:(%x), keysym(%x), char(%x)\n",
-				 pEvent->state, keysymOrig, chOrig) ;
-#endif
 		/*	signal äΤʤ顢Ϥǽλ*/
 		if (aBuf [0] == 0x07 && nKey == 1 &&
 			TSUCCEEDED (lispMachine_keyboardSignal (pLM)))
@@ -863,10 +945,7 @@ lispMachine_keyEvent2lispEntity (
 			if (pEvent->state & uMask) {
 				pEvent->state	&= ~uMask ;
 				nKey	= XLookupString (pEvent, aBuf, NELEMENTS (aBuf), &keysym, NULL) ;
-#if defined (DEBUG)
-				fprintf (stderr, "event state:(%x), nkey(%d), keysym(%x), char(%x)\n",
-						 pEvent->state, nKey, keysym, aBuf [0]) ;
-#endif
+
 				/*	Ƥޤ State ȴФkeysym ξȤƤ
 				 *	ޤΤǡȽˤϻȤʤ
 				 */
