@@ -117,6 +117,7 @@ struct tagTLispMachine {
 
 	/*	Lisp Machine  Local ѿ Lisp 
 	 *	ǤΤǤϤʤ*/
+	struct tagTLispBind*	m_apSymbol2SymbolTable [SIZE_LISP_BIND_TABLE] ;
 	struct tagTLispBind*	m_apVariableTable [SIZE_LISP_BIND_TABLE] ;
 	struct tagTLispBind*	m_apFunctionTable [SIZE_LISP_BIND_TABLE] ;
 	struct tagTLispBind*	m_apPropertyTable [SIZE_LISP_BIND_TABLE] ;
@@ -160,17 +161,24 @@ Boolean		lispMachine_QueueInEvent	(TLispMachine*, TLispEntity*, TLispEntity*) ;
 Boolean		lispMachine_GetBuffer		(TLispMachine*, const Char*, int, TLispEntity**) ;
 Boolean		lispMachine_GetFileBuffer	(TLispMachine*, const Char*, int, TLispEntity**) ;
 Boolean		lispMachine_GenerateNewBufferName (TLispMachine*, const Char*, int, TLispEntity**) ;
+Boolean		lispMachine_EnumBuffer		(TLispMachine*, Boolean, Boolean (*)(TLispMachine*, TLispEntity*, void*, Boolean*), void*) ;
 
 TLMRESULT	lispMachine_ReadStringStart	(TLispMachine*, const Char*, int, TLispEntity*) ;
 
+Boolean		lispMachine_EnumFrame			(TLispMachine*, Boolean (*)(TLispMachine*, TLispEntity*, void*, Boolean*), void*) ;
 Boolean		lispMachine_UpdateCurrentFrame	(TLispMachine*) ;
 Boolean		lispMachine_UpdateAllFrame		(TLispMachine*) ;
 void		lispMachine_ScheduleUpdateAllFrame	(TLispMachine*) ;
 Boolean		lispMachine_ResizeFrame			(TLispMachine*, TLispEntity*, const XRectangle*) ;
+Boolean		lispMachine_EnumWindow			(TLispMachine*, TLispEntity*, Boolean (*)(TLispMachine*, TLispEntity*, void*, Boolean*), void*) ;
 Boolean		lispMachine_UpdateWindow		(TLispMachine*, TLispEntity*, TLispEntity*) ;
 Boolean		lispMachine_UpdateOverTheSpotFrameWindow(TLispMachine*, TLispEntity*, TLispEntity*) ;
 Char		lispMachine_lispKeyEventSymbol2Char	(TLispManager* pLispMgr, TLispEntity*) ;
 
+int			lispMachine_GetCodingSystemForRead	(TLispMachine*) ;
+int			lispMachine_GetCodingSystemForWrite	(TLispMachine*) ;
+Boolean		lispMachine_CodingSystemp			(TLispManager*, TLispEntity*) ;
+
 #include	"lmachine-inl.h"
 
 #endif
