@@ -28,7 +28,9 @@
 #include "regex.h"
 
 static	Boolean	lispMachine_addTail (TLispManager*, TLispEntity*, TLispEntity**, TLispEntity**) ;
-/*static	void	cputstr (FILE*, const Char*, int) ;*/
+#if defined (DEBUG)
+static	void	cputstr (FILE*, const Char*, int) ;
+#endif
 
 Boolean
 lispMachineCode_PushVReg (
@@ -531,7 +533,6 @@ lispMachineCode_SetRegmatchTarget (
 	register TLispManager*	pLispMgr ;
 
 	assert (pLM        != NULL) ;
-	assert (pEntTarget != NULL) ;
 
 	pLispMgr	= pLM->m_pLispMgr ;
 	assert (pLispMgr   != NULL) ;
@@ -539,7 +540,8 @@ lispMachineCode_SetRegmatchTarget (
 	if (pLM->m_pEntRegMatch != NULL)
 		lispEntity_Release (pLispMgr, pLM->m_pEntRegMatch) ;
 	pLM->m_pEntRegMatch	= pEntTarget ;
-	lispEntity_AddRef (pLispMgr, pEntTarget) ;
+	if (pEntTarget != NULL)
+		lispEntity_AddRef (pLispMgr, pEntTarget) ;
 	return	True ;
 }
 
@@ -549,11 +551,14 @@ lispMachineCode_StringMatch (
 	register const Char*	pStrREGEXP,
 	register int			nStrREGEXP,
 	register const Char*	pStrSTRING,
-	register int			nStrSTRING)
+	register int			nStrSTRING,
+	register int			nStart)
 {
-	regex_t		re ;
-	Boolean		fRetval ;
-	rm_detail_t	rd ;
+	regex_t					re ;
+	Boolean					fRetval ;
+	rm_detail_t				rd ;
+	register regmatch_t*	pMatch ;
+	register int			i ;
 
 	assert (pLM != NULL) ;
 #if defined (DEBUG)
@@ -568,8 +573,14 @@ lispMachineCode_StringMatch (
 		return	False ;
 	pLM->m_aRegMatch [0].rm_so	= -1 ;
 	pLM->m_aRegMatch [0].rm_eo	= -1 ;
-	fRetval	= !ReExec (&re, pStrSTRING, nStrSTRING, &rd, MAX_REGEXP_MATCHES, pLM->m_aRegMatch, 0) ;
+	fRetval	= !ReExec (&re, pStrSTRING + nStart, nStrSTRING - nStart, &rd, MAX_REGEXP_MATCHES, pLM->m_aRegMatch, 0) ;
 	ReFree (&re) ;
+
+	pMatch	= pLM->m_aRegMatch ;
+	for (i = 0 ; i < MAX_REGEXP_MATCHES && pMatch->rm_eo >= 0 ; i ++, pMatch ++) {
+		pMatch->rm_so	+= nStart ;
+		pMatch->rm_eo	+= nStart ;
+	}
 	return	fRetval ;
 }
 
@@ -636,7 +647,7 @@ lispMachineCode_SearchForward (
 				pLM->m_aRegMatch [0].rm_eo	= nOffset + nPoint + nStrPATTERN ;
 #if defined (DEBUG)
 				fprintf (stderr, "(search-forward) [%d ... %d]\n", 
-						 pLM->m_aRegMatch [0].rm_so, pLM->m_aRegMatch [0].rm_eo) ;
+						 (int)pLM->m_aRegMatch [0].rm_so, (int)pLM->m_aRegMatch [0].rm_eo) ;
 #endif
 				fFound	= True ;
 			}
@@ -721,7 +732,7 @@ lispMachineCode_SearchBackward (
 				fFound	= True ;
 #if defined (DEBUG)
 				fprintf (stderr, "(search-backward) [%d ... %d]\n", 
-						 pLM->m_aRegMatch [0].rm_so, pLM->m_aRegMatch [0].rm_eo) ;
+						 (int) pLM->m_aRegMatch [0].rm_so, (int) pLM->m_aRegMatch [0].rm_eo) ;
 #endif
 				break ;
 			}
@@ -1111,6 +1122,7 @@ lispMachine_addTail (
 	return	True ;
 }
 
+#if defined (DEBUG)
 void
 cputstr (
 	register FILE*			pFile,
@@ -1130,4 +1142,6 @@ cputstr (
 	}
 	return ;
 }
+#endif
+
 
