@@ -52,7 +52,7 @@ Boolean		lispMachineCode_GetThisCommandKeys (TLispMachine*, Char const **, int*
 Boolean		lispMachineCode_ClearThisCommandKeys (TLispMachine*) ;
 
 Boolean		lispMachineCode_SetRegmatchTarget (TLispMachine*, TLispEntity*) ;
-Boolean		lispMachineCode_StringMatch		(TLispMachine*, const Char*, int, const Char*, int) ;
+Boolean		lispMachineCode_StringMatch		(TLispMachine*, const Char*, int, const Char*, int, int) ;
 Boolean		lispMachineCode_SearchForward	(TLispMachine*, TLispEntity*, const Char*, int, int, int) ;
 Boolean		lispMachineCode_SearchBackward	(TLispMachine*, TLispEntity*, const Char*, int, int, int) ;
 Boolean		lispMachineCode_ReSearchForward	(TLispMachine*, TLispEntity*, const Char*, int, int, int) ;
