@@ -103,6 +103,38 @@ lispMachine_ResizeFrame (
 	}
 }	
 
+/*	Ƥ frame ֤󤹤롣
+ */
+Boolean
+lispMachine_EnumFrame (
+	register TLispMachine*	pLM,
+	register Boolean		(*pProc)(TLispMachine*, TLispEntity*, void*, Boolean*),
+	register void*			pCaller)
+{
+	register TLispEntity*	pEntFrame ;
+	Boolean					fContinue ;
+
+	assert (pLM   != NULL) ;
+	assert (pProc != NULL) ;
+
+	if (pLM->m_lstFrame == NULL) 
+		return	True ;
+
+	fContinue	= True ;
+	pEntFrame	= pLM->m_lstFrame ;
+	do {
+		TLispEntity*	pEntNextFrame ;
+		lispFrame_GetNext (pEntFrame, &pEntNextFrame) ;
+		if (TFAILED ((pProc)(pLM, pEntFrame, pCaller, &fContinue)))
+			return	False ;
+		if (TFAILED (fContinue))
+			break ;
+		pEntFrame	= pEntNextFrame ;
+	}	while (pEntFrame != pLM->m_lstFrame) ;
+	
+	return	True ;
+}
+
 /*	Frame ˤɽʸʤˤʤơեʤо
  *	ĤΤ롣(XUnmapWindow ưŪ˹Ԥ)
  *
