@@ -29,6 +29,7 @@ LMSTATEPROTO (lispMachineState_Cdr) ;
 LMSTATEPROTO (lispMachineState_Cons) ;
 LMSTATEPROTO (lispMachineState_CarSafe) ;
 LMSTATEPROTO (lispMachineState_Set) ;
+LMSTATEPROTO (lispMachineState_SetDefault) ;
 LMSTATEPROTO (lispMachineState_Fset) ;
 LMSTATEPROTO (lispMachineState_Eq) ;
 LMSTATEPROTO (lispMachineState_Equal) ;
@@ -63,6 +64,7 @@ LMSTATEPROTO (lispMachineState_Signal) ;
 LMSTATEPROTO (lispMachineState_Setcar) ;
 LMSTATEPROTO (lispMachineState_Setcdr) ;
 LMSTATEPROTO (lispMachineState_Boundp) ;
+LMSTATEPROTO (lispMachineState_DefaultBoundp) ;
 LMSTATEPROTO (lispMachineState_DefaultValue) ;
 LMSTATEPROTO (lispMachineState_SymbolValue) ;
 LMSTATEPROTO (lispMachineState_Makunbound) ;
@@ -74,6 +76,8 @@ LMSTATEPROTO (lispMachineState_Princ) ;
 LMSTATEPROTO (lispMachineState_Natnump) ;
 LMSTATEPROTO (lispMachineState_Featurep) ;
 LMSTATEPROTO (lispMachineState_Provide) ;
+LMSTATEPROTO (lispMachineState_Reverse) ;
+LMSTATEPROTO (lispMachineState_CopySequence) ;
 
 LMSTATEPROTO (lispMachineState_Eval) ;
 LMSTATEPROTO (lispMachineState_Funcall) ;
@@ -155,6 +159,9 @@ LMSTATEPROTO (lispMachineState_ReadFromString) ;
 LMSTATEPROTO (lispMachineState_StringToNumber) ;
 LMSTATEPROTO (lispMachineState_CharCharset) ;
 LMSTATEPROTO (lispMachineState_StringLessp) ;
+LMSTATEPROTO (lispMachineState_StringWidth) ;
+LMSTATEPROTO (lispMachineState_MakeChar) ;
+LMSTATEPROTO (lispMachineState_CharValidp) ;
 
 /*	file.kc */
 LMSTATEPROTO (lispMachineState_LoadFile) ;
@@ -168,6 +175,11 @@ LMSTATEPROTO (lispMachineState_FileModes) ;
 LMSTATEPROTO (lispMachineState_SetFileModes) ;
 LMSTATEPROTO (lispMachineState_WriteRegion) ;
 LMSTATEPROTO (lispMachineState_FileExistsp) ;
+LMSTATEPROTO (lispMachineState_FileWritablep) ;
+LMSTATEPROTO (lispMachineState_FileReadablep) ;
+LMSTATEPROTO (lispMachineState_FileAttributes) ;
+LMSTATEPROTO (lispMachineState_MakeDirectoryInternal) ;
+LMSTATEPROTO (lispMachineState_MakeTempName) ;
 
 /*	subr.kc	*/
 LMSTATEPROTO (lispMachineState_Nth) ;
@@ -176,11 +188,15 @@ LMSTATEPROTO (lispMachineState_Functionp) ;
 LMSTATEPROTO (lispMachineState_MakeLocalHook) ;
 LMSTATEPROTO (lispMachineState_AddHook) ;
 LMSTATEPROTO (lispMachineState_Documentation) ;
+LMSTATEPROTO (lispMachineState_ForceModeLineUpdate) ;
+
 
 LMSTATEPROTO (lispMachineState_Error) ;
 
 /*	poem.c */
 LMSTATEPROTO (lispMachineState_StringToCharList) ;
+LMSTATEPROTO (lispMachineState_EventToCharacter) ;
+LMSTATEPROTO (lispMachineState_CharacterToEvent) ;
 
 /*	queue-m.kc	*/
 LMSTATEPROTO (lispMachineState_QueueCreate) ;
@@ -202,6 +218,7 @@ LMSTATEPROTO (lispMachineState_CurrentMinorModeMaps) ;
 LMSTATEPROTO (lispMachineState_UseLocalMap) ;
 LMSTATEPROTO (lispMachineState_CurrentLocalMap) ;
 LMSTATEPROTO (lispMachineState_CopyKeymap) ;
+LMSTATEPROTO (lispMachineState_WhereIsInternal) ;
 LMSTATEPROTO (lispMachineState_KeyBinding) ;
 LMSTATEPROTO (lispMachineState_JKeyBinding) ;
 
@@ -226,7 +243,7 @@ LMSTATEPROTO (lispMachineState_MakeLocalVariable) ;
 LMSTATEPROTO (lispMachineState_MakeVariableBufferLocal) ;
 LMSTATEPROTO (lispMachineState_LookingAt) ;
 LMSTATEPROTO (lispMachineState_CharAfter) ;
-LMSTATEPROTO (lispMachineState_CharAfter) ;
+LMSTATEPROTO (lispMachineState_CharBefore) ;
 LMSTATEPROTO (lispMachineState_NarrowToRegion) ;
 LMSTATEPROTO (lispMachineState_Widen) ;
 LMSTATEPROTO (lispMachineState_GetBufferCreate) ;
@@ -237,6 +254,11 @@ LMSTATEPROTO (lispMachineState_GenerateNewBuffer) ;
 LMSTATEPROTO (lispMachineState_GetFileBuffer) ;
 LMSTATEPROTO (lispMachineState_SetBufferModifiedp) ;
 LMSTATEPROTO (lispMachineState_BufferModifiedp) ;
+LMSTATEPROTO (lispMachineState_BufferSize) ;
+LMSTATEPROTO (lispMachineState_LocalVariablep) ;
+LMSTATEPROTO (lispMachineState_KillBuffer) ;
+LMSTATEPROTO (lispMachineState_BufferName) ;
+LMSTATEPROTO (lispMachineState_BufferList) ;
 
 /*	edit.c	*/
 LMSTATEPROTO (lispMachineState_SelfInsertCommand) ;
@@ -259,6 +281,7 @@ LMSTATEPROTO (lispMachineState_ThisCommandKeys) ;
 /*	stmarker.c */
 LMSTATEPROTO (lispMachineState_MakeMarker) ;
 LMSTATEPROTO (lispMachineState_SetMarker) ;
+LMSTATEPROTO (lispMachineState_SetMarkerInsertionType) ;
 LMSTATEPROTO (lispMachineState_MarkerPosition) ;
 LMSTATEPROTO (lispMachineState_MarkerBuffer) ;
 LMSTATEPROTO (lispMachineState_MarkerInsertionType) ;
@@ -272,6 +295,8 @@ LMSTATEPROTO (lispMachineState_Interactive) ;
 LMSTATEPROTO (lispMachineState_WindowFrame) ;
 LMSTATEPROTO (lispMachineState_WindowBuffer) ;
 LMSTATEPROTO (lispMachineState_SelectedWindow) ;
+LMSTATEPROTO (lispMachineState_WindowWidth) ;
+LMSTATEPROTO (lispMachineState_WindowMinibufferp) ;
 
 /*	frame.c */
 LMSTATEPROTO (lispMachineState_SelectedFrame) ;
@@ -296,6 +321,7 @@ LMSTATEPROTO (lispMachineState_ReadFromMinibuffer) ;
 /*	macro.c */
 LMSTATEPROTO (lispMachineState_Defmacro) ;
 LMSTATEPROTO (lispMachineState_EvalMacro) ;
+LMSTATEPROTO (lispMachineState_Macroexpand) ;
 LMSTATEPROTO (lispMachineState_Backquote) ;
 
 /*	symbol.c */
@@ -331,6 +357,7 @@ LMSTATEPROTO (lispMachineState_ReSearchBackward) ;
 LMSTATEPROTO (lispMachineState_MatchBeginning) ;
 LMSTATEPROTO (lispMachineState_MatchEnd) ;
 LMSTATEPROTO (lispMachineState_MatchData) ;
+LMSTATEPROTO (lispMachineState_SetMatchData) ;
 LMSTATEPROTO (lispMachineState_SearchForward) ;
 LMSTATEPROTO (lispMachineState_SearchBackward) ;
 LMSTATEPROTO (lispMachineState_RegexpQuote) ;
@@ -339,6 +366,8 @@ LMSTATEPROTO (lispMachineState_RegexpQuote) ;
 LMSTATEPROTO (lispMachineState_JSearchServer) ;
 LMSTATEPROTO (lispMachineState_JOkuriSearchSubr) ;
 LMSTATEPROTO (lispMachineState_JSearchJisyoFile) ;
+LMSTATEPROTO (lispMachineState_SkkinputSearchServerSubr) ;
+LMSTATEPROTO (lispMachineState_SkkinputComputeHenkanLists) ;
 
 /* misc.c */
 LMSTATEPROTO (lispMachineState_Ding) ;
