@@ -23,6 +23,119 @@
 #include <assert.h>
 #include "lmachinep.h"
 
+/*========================================================================*
+ *	private functions
+ */
+
+/*	symbol-to-symbol  wrapper
+ *--
+ *	last-command-char, last-command-event Τ褦Ʊư褦ʤΤ
+ *	б뤿 wrapper Ǥ롣lisp machine ̤Ǥ롣
+ */
+static inline TLispEntity*
+lispMachine_getRealSymbol (
+	register TLispMachine*	pLM,
+	register TLispEntity*	pEntSymbol)
+{
+	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	register TLispBind**	pSym2SymTbl	= pLM->m_apSymbol2SymbolTable ;
+	register Boolean		f ;
+	TLispBind*				pBind ;
+	TLispEntity*			pEntNewSymbol ;
+
+	f	= lispBindTable_SearchEntry (pLispMgr, pSym2SymTbl, SIZE_LISP_BIND_TABLE, pEntSymbol, &pBind) ;
+	if (TSUCCEEDED (f) && pBind != NULL) {
+		if (TSUCCEEDED (lispBind_GetValue (pBind, &pEntNewSymbol)) &&
+			pEntNewSymbol != NULL)
+			return	pEntNewSymbol ;
+	}
+	return	pEntSymbol ;
+}
+
+/*
+ *[ǽ]
+ *	buffer-local-variable ͤꤹ롣
+ *[]
+ *	Lisp ̿ buffer-local-variable ؤơͤ򥻥åȤ
+ *	Τ뤫ɤʬʤĴ­⤷ʤ
+ */
+static inline Boolean
+lispMachine_setCurrentBufferLocalSymbolValue (
+	register TLispMachine*	pLM,
+	register TLispEntity*	pEntSymbol,
+	register TLispEntity*	pEntValue)
+{
+	register TLispManager*	pLispMgr ;
+	register TLispEntity*	pEntBuffer ;
+
+	assert (pLM        != NULL) ;
+	assert (pEntSymbol != NULL) ;
+	assert (pEntValue  != NULL) ;
+
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr   != NULL) ;
+	pEntBuffer	= pLM->m_pCurBuffer ;
+
+	/*	ιԤϰŪʤΤ*/
+	if (pEntBuffer == NULL)
+		return	False ;
+
+	return	lispBuffer_SetSymbolValue (pLispMgr, pEntBuffer, pEntSymbol, pEntValue) ;
+}
+
+/*
+ *[ǽ]
+ *	Lisp Machine  Local ʥܥͤ򥻥åȤ롣
+ */
+static inline Boolean
+lispMachine_setGlobalSymbolValue (
+	register TLispMachine*	pLM,
+	register TLispEntity*	pEntSymbol,
+	register TLispEntity*	pEntValue)
+{
+	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	register TLispBind**	pVarTbl ;
+	TLispBind*	pBind ;
+
+	assert (pLM             != NULL) ;
+	assert (pLM->m_pLispMgr != NULL) ;
+	assert (pEntSymbol      != NULL) ;
+
+	pVarTbl		= pLM->m_apVariableTable ;
+	if (TFAILED (lispBindTable_SearchEntry (pLispMgr, pVarTbl, SIZE_LISP_BIND_TABLE, pEntSymbol, &pBind)) ||
+		pBind == NULL) {
+		if (TFAILED (lispBindTable_MakeEntry (pLispMgr, pVarTbl, SIZE_LISP_BIND_TABLE, pEntSymbol, &pBind)))
+			return	False ;
+	}
+	lispBind_SetValue (pLispMgr, pBind, pEntValue) ;
+	return	True ;
+}
+
+static inline Boolean
+lispMachine_getBufferLocalSymbolValue (
+	register TLispMachine*			pLM,
+	register TLispEntity*			pEntBuffer,
+	register TLispEntity*			pEntTarget,
+	register TLispEntity** const	ppEntReturn)
+{
+	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	register Boolean			fRetval ;
+	TLispEntity*	pEntRetval ;
+
+	assert (pLM         != NULL) ;
+	assert (pEntTarget  != NULL) ;
+	assert (pEntBuffer  != NULL) ;
+	assert (ppEntReturn != NULL) ;
+
+	fRetval	= lispBuffer_GetSymbolValue (pLispMgr, pEntBuffer, pEntTarget, &pEntRetval) ;
+	if (TFAILED (fRetval) ||
+		pEntRetval == NULL ||
+		TSUCCEEDED (lispEntity_Emptyp (pLispMgr, pEntRetval)))
+		return	False ;
+	*ppEntReturn	= pEntRetval ;
+	return	True ;
+}
+
 /*
  *[ǽ]
  *	Lisp Machine  local ʥܥ(ѿ)롣ܥ̾
@@ -79,8 +192,9 @@ lispMachine_SetCurrentSymbolValue (
 	assert (pEntSymbol != NULL) ;
 	assert (pEntValue  != NULL) ;
 
-	return	(lispMachine_SetCurrentBufferLocalSymbolValue (pLM, pEntSymbol, pEntValue) ||
-			 lispMachine_SetGlobalSymbolValue      (pLM, pEntSymbol, pEntValue)) ;
+	pEntSymbol	= lispMachine_getRealSymbol (pLM, pEntSymbol) ;
+	return	(lispMachine_setCurrentBufferLocalSymbolValue (pLM, pEntSymbol, pEntValue) ||
+			 lispMachine_setGlobalSymbolValue      (pLM, pEntSymbol, pEntValue)) ;
 }
 
 /*
@@ -111,6 +225,7 @@ lispMachine_SetCurrentBufferLocalSymbolValue (
 	if (pEntBuffer == NULL)
 		return	False ;
 
+	pEntSymbol	= lispMachine_getRealSymbol (pLM, pEntSymbol) ;
 	return	lispBuffer_SetSymbolValue (pLispMgr, pEntBuffer, pEntSymbol, pEntValue) ;
 }
 
@@ -124,20 +239,11 @@ lispMachine_SetGlobalSymbolValue (
 	register TLispEntity*	pEntSymbol,
 	register TLispEntity*	pEntValue)
 {
-	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
-	TLispBind*	pBind ;
-
 	assert (pLM             != NULL) ;
-	assert (pLM->m_pLispMgr != NULL) ;
 	assert (pEntSymbol      != NULL) ;
 
-	if (TFAILED (lispBindTable_SearchEntry (pLispMgr, pLM->m_apVariableTable, NELEMENTS (pLM->m_apVariableTable), pEntSymbol, &pBind)) ||
-		pBind == NULL) {
-		if (TFAILED (lispBindTable_MakeEntry (pLispMgr, pLM->m_apVariableTable, NELEMENTS (pLM->m_apVariableTable), pEntSymbol, &pBind)))
-			return	False ;
-	}
-	lispBind_SetValue (pLispMgr, pBind, pEntValue) ;
-	return	True ;
+	pEntSymbol	= lispMachine_getRealSymbol (pLM, pEntSymbol) ;
+	return	lispMachine_setGlobalSymbolValue (pLM, pEntSymbol, pEntValue) ;
 }
 
 /*
@@ -176,7 +282,6 @@ lispMachine_SetGlobalSymbolValueWithName (
 {
 	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
 	TLispEntity*	pEntSymbol ;
-	TLispBind*		pBind ;
 
 	assert (pLM             != NULL) ;
 	assert (pLM->m_pLispMgr != NULL) ;
@@ -186,13 +291,8 @@ lispMachine_SetGlobalSymbolValueWithName (
 	if (TFAILED (lispMgr_InternSymbol (pLispMgr, pName, nName, &pEntSymbol)))
 		return	False ;
 
-	if (TFAILED (lispBindTable_SearchEntry (pLispMgr, pLM->m_apVariableTable, NELEMENTS (pLM->m_apVariableTable), pEntSymbol, &pBind)) ||
-		pBind == NULL) {
-		if (TFAILED (lispBindTable_MakeEntry (pLispMgr, pLM->m_apVariableTable, NELEMENTS (pLM->m_apVariableTable), pEntSymbol, &pBind)))
-			return	False ;
-	}
-	lispBind_SetValue (pLispMgr, pBind, pEntValue) ;
-	return	True ;
+	pEntSymbol	= lispMachine_getRealSymbol (pLM, pEntSymbol) ;
+	return	lispMachine_setGlobalSymbolValue (pLM, pEntSymbol, pEntValue) ;
 }
 
 /*
@@ -221,8 +321,9 @@ lispMachine_SetCurrentSymbolValueWithName (
 	if (TFAILED (lispMgr_InternSymbol (pLispMgr, pName, nName, &pEntSymbol)))
 		return	False ;
 
-	return	(lispMachine_SetCurrentBufferLocalSymbolValue (pLM, pEntSymbol, pEntValue) ||
-			 lispMachine_SetGlobalSymbolValue      (pLM, pEntSymbol, pEntValue)) ;
+	pEntSymbol	= lispMachine_getRealSymbol (pLM, pEntSymbol) ;
+	return	(lispMachine_setCurrentBufferLocalSymbolValue (pLM, pEntSymbol, pEntValue) ||
+			 lispMachine_setGlobalSymbolValue      (pLM, pEntSymbol, pEntValue)) ;
 }
 
 Boolean
@@ -242,8 +343,9 @@ lispMachine_SetCurrentSymbolValueWithNameA (
 	if (TFAILED (lispMgr_InternSymbolA (pLispMgr, pName, nName, &pEntSymbol)))
 		return	False ;
 
-	return	(lispMachine_SetCurrentBufferLocalSymbolValue (pLM, pEntSymbol, pEntValue) ||
-			 lispMachine_SetGlobalSymbolValue      (pLM, pEntSymbol, pEntValue)) ;
+	pEntSymbol	= lispMachine_getRealSymbol (pLM, pEntSymbol) ;
+	return	(lispMachine_setCurrentBufferLocalSymbolValue (pLM, pEntSymbol, pEntValue) ||
+			 lispMachine_setGlobalSymbolValue      (pLM, pEntSymbol, pEntValue)) ;
 }
 
 /*
@@ -274,7 +376,8 @@ lispMachine_GetCurrentBufferLocalSymbolValue (
 	register TLispEntity*			pEntTarget,
 	register TLispEntity** const	ppEntReturn)
 {
-	return	lispMachine_GetBufferLocalSymbolValue (pLM, pLM->m_pCurBuffer, pEntTarget, ppEntReturn) ;
+	pEntTarget	= lispMachine_getRealSymbol (pLM, pEntTarget) ;
+	return	lispMachine_getBufferLocalSymbolValue (pLM, pLM->m_pCurBuffer, pEntTarget, ppEntReturn) ;
 }
 
 Boolean
@@ -284,6 +387,7 @@ lispMachine_GetGlobalSymbolValue (
 	register TLispEntity** const	ppEntReturn)
 {
 	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	register TLispEntity*	pEntTarget ;
 
 	assert (pLM             != NULL) ;
 	assert (pLispMgr        != NULL) ;
@@ -291,7 +395,8 @@ lispMachine_GetGlobalSymbolValue (
 	assert (ppEntReturn     != NULL) ;
 
 	while (pLM != NULL) {
-		if (TSUCCEEDED (lispBindTable_GetEntryValue (pLispMgr, pLM->m_apVariableTable, NELEMENTS (pLM->m_apVariableTable), pEntSymbol, ppEntReturn)))
+		pEntTarget	= lispMachine_getRealSymbol (pLM, pEntSymbol) ;
+		if (TSUCCEEDED (lispBindTable_GetEntryValue (pLispMgr, pLM->m_apVariableTable, NELEMENTS (pLM->m_apVariableTable), pEntTarget, ppEntReturn)))
 			return	True ;
 		pLM	= pLM->m_pMacParent ;
 	}
@@ -316,22 +421,13 @@ lispMachine_GetBufferLocalSymbolValue (
 	register TLispEntity*			pEntTarget,
 	register TLispEntity** const	ppEntReturn)
 {
-	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
-	register Boolean			fRetval ;
-	TLispEntity*	pEntRetval ;
-
 	assert (pLM         != NULL) ;
 	assert (pEntTarget  != NULL) ;
 	assert (pEntBuffer  != NULL) ;
 	assert (ppEntReturn != NULL) ;
 
-	fRetval	= lispBuffer_GetSymbolValue (pLispMgr, pEntBuffer, pEntTarget, &pEntRetval) ;
-	if (TFAILED (fRetval) ||
-		pEntRetval == NULL ||
-		TSUCCEEDED (lispEntity_Emptyp (pLispMgr, pEntRetval)))
-		return	False ;
-	*ppEntReturn	= pEntRetval ;
-	return	True ;
+	pEntTarget	= lispMachine_getRealSymbol (pLM, pEntTarget) ;
+	return	lispMachine_getBufferLocalSymbolValue (pLM, pEntBuffer, pEntTarget, ppEntReturn) ;
 }
 
 Boolean
@@ -363,25 +459,20 @@ lispMachine_GetCurrentBufferLocalSymbolValueWithName (
 {
 	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
 	register TLispEntity*	pEntBuffer	= pLM->m_pCurBuffer ;
-	register Boolean			fRetval ;
-	TLispEntity*	pEntRetval ;
+	TLispEntity*			pEntSymbol ;
+	register TLispEntity*	pEntTarget ;
 
 	assert (pLM      != NULL) ;
 	assert (pName    != NULL && nName > 0) ;
 	assert (ppEntReturn != NULL) ;
 
-	/*	ιԤϰŪʤΤ*/
-	if (pLM->m_pCurBuffer == NULL)
+	if (pEntBuffer == NULL)
 		return	False ;
 
-	fRetval	= lispBuffer_GetSymbolValueWithName (pLispMgr, pEntBuffer, pName, nName, &pEntRetval) ;
-	if (TFAILED (fRetval) ||
-		pEntRetval == NULL ||
-		TSUCCEEDED (lispEntity_Emptyp (pLispMgr, pEntRetval)))
+	if (TFAILED (lispMgr_InternSymbol (pLispMgr, pName, nName, &pEntSymbol)))
 		return	False ;
-	*ppEntReturn	= pEntRetval ;
-	return	True ;
-
+	pEntTarget	= lispMachine_getRealSymbol (pLM, pEntSymbol) ;
+	return	lispMachine_getBufferLocalSymbolValue (pLM, pEntBuffer, pEntTarget, ppEntReturn) ;
 }
 
 Boolean
@@ -392,7 +483,8 @@ lispMachine_GetGlobalSymbolValueWithName (
 	register TLispEntity** const	ppEntReturn)
 {
 	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
-	TLispEntity*	pEntSymbol ;
+	TLispEntity*			pEntSymbol ;
+	register TLispEntity*	pEntTarget ;
 
 	assert (pLM             != NULL) ;
 	assert (pLM->m_pLispMgr != NULL) ;
@@ -401,13 +493,16 @@ lispMachine_GetGlobalSymbolValueWithName (
 		return	False ;
 
 	while (pLM != NULL) {
-		if (TSUCCEEDED (lispBindTable_GetEntryValue (pLispMgr, pLM->m_apVariableTable, NELEMENTS (pLM->m_apVariableTable), pEntSymbol, ppEntReturn)))
+		pEntTarget	= lispMachine_getRealSymbol (pLM, pEntSymbol) ;
+		if (TSUCCEEDED (lispBindTable_GetEntryValue (pLispMgr, pLM->m_apVariableTable, NELEMENTS (pLM->m_apVariableTable), pEntTarget, ppEntReturn)))
 			return	True ;
 		pLM	= pLM->m_pMacParent ;
 	}
 	return	False ;
 }
 
+/*	function, property ˤ symbol -> symbol  map ϱƶʤȤ롣
+ */
 Boolean
 lispMachine_SetSymbolFunctionValue (
 	register TLispMachine*			pLM,
@@ -415,15 +510,17 @@ lispMachine_SetSymbolFunctionValue (
 	register TLispEntity*			pEntValue)
 {
 	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	register TLispBind**	ppBindTbl ;
 	TLispBind*	pBind ;
 
 	assert (pLM         != NULL) ;
 	assert (pEntSymbol  != NULL) ;
 	assert (pEntValue   != NULL) ;
 
-	if (TFAILED (lispBindTable_SearchEntry (pLispMgr, pLM->m_apFunctionTable, NELEMENTS (pLM->m_apFunctionTable), pEntSymbol, &pBind)) ||
+	ppBindTbl	= pLM->m_apFunctionTable ;
+	if (TFAILED (lispBindTable_SearchEntry (pLispMgr, ppBindTbl, NELEMENTS (pLM->m_apFunctionTable), pEntSymbol, &pBind)) ||
 		pBind == NULL) {
-		if (TFAILED (lispBindTable_MakeEntry (pLispMgr, pLM->m_apFunctionTable, NELEMENTS (pLM->m_apFunctionTable), pEntSymbol, &pBind)))
+		if (TFAILED (lispBindTable_MakeEntry (pLispMgr, ppBindTbl, NELEMENTS (pLM->m_apFunctionTable), pEntSymbol, &pBind)))
 			return	False ;
 	}
 	lispBind_SetValue (pLispMgr, pBind, pEntValue) ;
@@ -487,6 +584,8 @@ lispMachine_GetFinalSymbolFunctionValue (
 	return	True ;
 }
 
+/*
+ */
 Boolean
 lispMachine_SetSymbolProperty (
 	register TLispMachine*			pLM,
@@ -494,15 +593,17 @@ lispMachine_SetSymbolProperty (
 	register TLispEntity* 			pEntValue)
 {
 	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	register TLispBind**	pBindTbl ;
 	TLispBind*	pBind ;
 
 	assert (pLM         != NULL) ;
 	assert (pEntSymbol  != NULL) ;
 	assert (pEntValue   != NULL) ;
 
-	if (TFAILED (lispBindTable_SearchEntry (pLispMgr, pLM->m_apPropertyTable, NELEMENTS (pLM->m_apPropertyTable), pEntSymbol, &pBind)) ||
+	pBindTbl	= pLM->m_apPropertyTable ;
+	if (TFAILED (lispBindTable_SearchEntry (pLispMgr, pBindTbl, NELEMENTS (pLM->m_apPropertyTable), pEntSymbol, &pBind)) ||
 		pBind == NULL) {
-		if (TFAILED (lispBindTable_MakeEntry (pLispMgr, pLM->m_apPropertyTable, NELEMENTS (pLM->m_apPropertyTable), pEntSymbol, &pBind)))
+		if (TFAILED (lispBindTable_MakeEntry (pLispMgr, pBindTbl, NELEMENTS (pLM->m_apPropertyTable), pEntSymbol, &pBind)))
 			return	False ;
 	}
 	lispBind_SetValue (pLispMgr, pBind, pEntValue) ;
