@@ -304,6 +304,46 @@ lispMachine_updateWindowBufferWithoutFrame (
 	return ;
 }
 
+Boolean
+lispMachine_EnumWindow (
+	register TLispMachine*	pLM,
+	register TLispEntity*	pEntFrame,
+	register Boolean		(*pProc)(TLispMachine*, TLispEntity*, void*, Boolean*),
+	register void*			pCaller)
+{
+	TLispEntity*	pEntTopWindow ;
+	TLispEntity*	pEntWindow ;
+	Boolean			fContinue ;
+
+	assert (pLM       != NULL) ;
+	assert (pEntFrame != NULL) ;
+	assert (pProc     != NULL) ;
+
+	if (TFAILED (lispFrame_GetTopWindow (pEntFrame, &pEntTopWindow)))
+		return	False ;
+
+	if (pEntTopWindow == NULL)
+		return	True ;
+
+	fContinue	= True ;
+	pEntWindow	= pEntTopWindow ;
+	do {
+		TLispEntity*	pEntNextWindow ;
+
+		lispWindow_GetNext (pEntWindow, &pEntNextWindow) ;
+		if (TFAILED ((pProc)(pLM, pEntWindow, pCaller, &fContinue)))
+			return	False ;
+		if (TFAILED (fContinue))
+			break ;
+		/*	top window ݤľʤ... procedure call 
+		 *	top window ѹǽ롣*/
+		pEntWindow	= pEntNextWindow ;
+		(void) lispFrame_GetTopWindow (pEntFrame, &pEntTopWindow) ;
+	}	while (pEntWindow != pEntTopWindow) ;
+	
+	return	True ;
+}
+
 Boolean
 lispMachine_UpdateWindow (
 	register TLispMachine*	pLM,
