@@ -202,20 +202,28 @@ lispMgr_ParseFile (
 	TLispEntity*	pRetvalue ;
 	LISPPARSEFUNC	pNextStateFunc ;
 	TVarbuffer		vbEntity ;
+	TVarbuffer		vbufTmpFileName ;
 	int				nEntity ;
 
 	assert (pLispMgr  != NULL) ;
 	assert (pFileName != NULL) ;
 
-	if (TFAILED (TVarbuffer_Initialize (&vbEntity, sizeof (TLispEntity *))))
+	if (TFAILED (TVarbuffer_Initialize (&vbEntity, sizeof (TLispEntity *))) ||
+		TFAILED (TVarbuffer_Initialize (&vbufTmpFileName, sizeof (char))))
 		return	NULL ;
 
-	if (!KFile_Open (&kfile, pFileName, KCODING_SYSTEM_UNKNOWN/*KCODING_SYSTEM_EUCJP*/))
-		return	NULL ;
+	pRetvalue	= NULL ;
+	if (!KFile_Open (&kfile, pFileName, KCODING_SYSTEM_UNKNOWN/*KCODING_SYSTEM_EUCJP*/)) {
+		register int	nFileName	= strlen (pFileName) ;
+		if (TFAILED (TVarbuffer_Add (&vbufTmpFileName, pFileName, nFileName)) ||
+			TFAILED (TVarbuffer_Add (&vbufTmpFileName, ".el", 4))) /* include NUL */
+			goto	exit_func ;
+		pFileName	= TVarbuffer_GetBuffer (&vbufTmpFileName) ;
+		if (!KFile_Open (&kfile, pFileName, KCODING_SYSTEM_UNKNOWN/*KCODING_SYSTEM_EUCJP*/)) 
+			goto	exit_func ;
+	}
 	KFile_Rewind (&kfile) ;
 
-	pRetvalue				= NULL ;
-
 	parser.m_src.m_file.m_pFile		= &kfile ;
 	parser.m_src.m_file.m_iBufUsage	= 0 ;
 	parser.m_pGetChar		= lispParser_GetFileChar ;
@@ -283,8 +291,9 @@ lispMgr_ParseFile (
 	} else {
 		pRetvalue	= NULL ;
 	}
+  exit_func:
 	TVarbuffer_Uninitialize (&vbEntity) ;
-
+	TVarbuffer_Uninitialize (&vbufTmpFileName) ;
 	return	pRetvalue ;
 }
 
@@ -1128,6 +1137,7 @@ lispParser_SkipSpace (
 		if (Char_DifferenceAscii (cc, ' ')	&&
 			Char_DifferenceAscii (cc, '\t')	&&
 			Char_DifferenceAscii (cc, '\n')	&&
+			Char_DifferenceAscii (cc, '\f')	&&
 			Char_DifferenceAscii (cc, '\r'))
 			break ;
 	}
