@@ -28,16 +28,16 @@
 
 #define	lispEntity_GetStringPtr(ptr)	((TLispString *)((TLispEntity *)(ptr) + 1))
 
-static	Boolean	stringToCharListString	(TLispManager* pLispMgr, TLispEntity* pSequence, TLispEntity** ppRetval) ;
-static	Boolean	stringToCharListVector	(TLispManager* pLispMgr, TLispEntity* pSequence, TLispEntity** ppRetval) ;
-static	Boolean	stringToCharListList	(TLispManager* pLispMgr, TLispEntity* pSequence, TLispEntity** ppRetval) ;
-static	Boolean	concatEntity			(TLispManager* pLispMgr, TLispEntity* pArg, TVarbuffer* pvbufStr) ;
-static	Boolean	concatString			(TLispManager* pLispMgr, TLispEntity* pArg, TVarbuffer* pvbufStr) ;
-static	Boolean	concatVector			(TLispManager* pLispMgr, TLispEntity* pArg, TVarbuffer* pvbufStr) ;
-static	Boolean	concatList				(TLispManager* pLispMgr, TLispEntity* pArg, TVarbuffer* pvbufStr) ;
-static	Boolean	concatInteger			(TLispManager* pLispMgr, TLispEntity* pArg, TVarbuffer* pvbufStr) ;
-static	Boolean	substringString			(TLispManager* pLispMgr, TLispEntity* pEntity, long lFrom, long lTo, TLispEntity** ppReturn) ;
-static	Boolean	substringVector			(TLispManager* pLispMgr, TLispEntity* pEntity, long lFrom, long lTo, TLispEntity** ppReturn) ;
+static	Boolean	stringToCharListString	(TLispManager*, TLispEntity*, TLispEntity**) ;
+static	Boolean	stringToCharListVector	(TLispManager*, TLispEntity*, TLispEntity**) ;
+static	Boolean	stringToCharListList	(TLispManager*, TLispEntity*, TLispEntity**) ;
+static	Boolean	concatEntity			(TLispManager*, TLispEntity*, TVarbuffer*) ;
+static	Boolean	concatString			(TLispManager*, TLispEntity*, TVarbuffer*) ;
+static	Boolean	concatVector			(TLispManager*, TLispEntity*, TVarbuffer*) ;
+static	Boolean	concatList				(TLispManager*, TLispEntity*, TVarbuffer*) ;
+static	Boolean	concatInteger			(TLispManager*, TLispEntity*, TVarbuffer*) ;
+static	Boolean	substringString			(TLispManager*, TLispEntity*, long, long, TLispEntity**) ;
+static	Boolean	substringVector			(TLispManager*, TLispEntity*, long, long, TLispEntity**) ;
 
 
 /*	LISP ʸ롣ʸξ硢ȤƱʸϿƤƤ
@@ -137,8 +137,9 @@ lispMgr_Concat (
 		for ( ; ; ) {
 			if (TFAILED (lispEntity_GetCar (pLispMgr, pArglist, &pArg)))
 				break ;
-			if (TFAILED (concatEntity (pLispMgr, pArg, &vbufStr)))
-				break ;
+			if (TFAILED (lispEntity_Nullp (pLispMgr, pArg)))
+				if (TFAILED (concatEntity (pLispMgr, pArg, &vbufStr)))
+					break ;
 			
 			if (TFAILED (lispEntity_GetCdr (pLispMgr, pArglist, &pNextArglist)))
 				break ;
@@ -146,6 +147,7 @@ lispMgr_Concat (
 			if (TSUCCEEDED (lispEntity_Nullp (pLispMgr, pArglist)))
 				break ;
 			if (pSeparator != NULL &&
+				TFAILED (lispEntity_Nullp (pLispMgr, pSeparator)) &&
 				TFAILED (concatEntity (pLispMgr, pSeparator, &vbufStr)))
 				break ;
 		}
@@ -254,10 +256,17 @@ lispEntity_Downcase (
 	assert (pObj      != NULL) ;
 	assert (ppReturn  != NULL) ;
 
-	if (TFAILED (lispEntity_GetType (pLispMgr, pObj, &iType)) ||
-		(iType != LISPENTITY_INTEGER && iType != LISPENTITY_STRING))
+	if (TFAILED (lispEntity_GetType (pLispMgr, pObj, &iType)))
 		return	False ;
-
+	if (iType != LISPENTITY_INTEGER && iType != LISPENTITY_STRING) {
+		if (iType != LISPENTITY_XEVENT)
+			return	False ;
+		pObj	= lispEntity_XEvent2Integer (pObj) ;
+		if (pObj == NULL ||
+			TFAILED (lispEntity_GetType (pLispMgr, pObj, &iType)) ||
+			iType != LISPENTITY_INTEGER)
+			return	False ;
+	}
 	if (iType == LISPENTITY_INTEGER) {
 		long	lValue ;
 
@@ -457,6 +466,7 @@ concatEntity (
 		fRetval	= concatInteger (pLispMgr, pArg, pvbufStr) ;
 		break ;
 	default:
+		fRetval = lispEntity_Nullp (pLispMgr, pArg) ;
 		break ;
 	}
 	return	fRetval ;
