@@ -346,6 +346,25 @@ lispMgr_UnregisterSymbol (
 	return	True ;
 }
 
+Boolean
+lispMgr_SymbolRequireEvalp (
+	register TLispManager*	pLispMgr,
+	register TLispEntity*	pEntSymbol)
+{
+	const Char*	pSymName ;
+	int			nSymName ;
+
+	assert (pLispMgr   != NULL) ;
+	assert (pEntSymbol != NULL) ;
+
+	if (TSUCCEEDED (lispEntity_Nullp (pLispMgr, pEntSymbol)) ||
+		TSUCCEEDED (lispEntity_Tp    (pLispMgr, pEntSymbol))) 
+		return	False ;
+	if (TFAILED (lispEntity_GetSymbolName (pLispMgr, pEntSymbol, &pSymName, &nSymName)))
+		return	True ;
+	return	(nSymName <= 0 || Char_DifferenceAscii (*pSymName, ':'))? True : False ;
+}
+
 Boolean
 lispMgr_registerSymbol (
 	register TLispManager*	pLispMgr,
