@@ -154,7 +154,9 @@ lispMgr_Vconcat (
 				goto	exit_loop ;
 			break ;
 		default:
-			goto	exit_loop ;
+			if (TFAILED (lispEntity_Nullp (pLispMgr, pCar)))
+				goto	exit_loop ;
+			break ;
 		}
 		if (TFAILED (lispEntity_GetCdr (pLispMgr, pArglist, &pNextArglist)) ||
 			pNextArglist == NULL)
