@@ -150,11 +150,41 @@ lispMachineState_Set (
 	return	LMR_RETURN ;
 }
 
+/*	(set-default SYMBOL VALUE)
+ *
+ *	SYMBOL  default value  VALUE ꤹ롣SYMBOL  VALUE ɾ롣
+ */
+TLMRESULT
+lispMachineState_SetDefault (
+	register TLispMachine* pLM)
+{
+	register TLispManager*	pLispMgr ;
+	TLispEntity*	pEntArglist ;
+	TLispEntity*	pEntSymbol ;
+	TLispEntity*	pEntValue ;
+
+	assert (pLM != NULL) ;
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr != NULL) ;
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	if (TFAILED (lispEntity_GetCar  (pLispMgr, pEntArglist, &pEntSymbol)) ||
+		TFAILED (lispEntity_Symbolp (pLispMgr, pEntSymbol))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	if (TFAILED (lispEntity_GetCadr (pLispMgr, pEntArglist, &pEntValue)) ||
+		TFAILED (lispMachine_SetGlobalSymbolValue (pLM, pEntSymbol, pEntValue))) 
+		lispMachineCode_SetError (pLM) ;
+	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntValue) ;
+	return	LMR_RETURN ;
+}
+
 /*
  *	(fset SYMBOL DEFINITION)
  */
 TLMRESULT
-lispMachineState_Fset (register TLispMachine* pLM)
+lispMachineState_Fset (
+	register TLispMachine* pLM)
 {
 	register TLispManager*	pLispMgr ;
 	TLispEntity*	pEntArglist ;
@@ -819,7 +849,8 @@ lispMachineState_Vconcat (register TLispMachine* pLM)
 }
 
 TLMRESULT
-lispMachineState_Nreverse (register TLispMachine* pLM)
+lispMachineState_Nreverse (
+	register TLispMachine*	pLM)
 {
 	TLispEntity*	pResult ;
 	TLispEntity*	pList ;
@@ -992,7 +1023,8 @@ lispMachineState_Setcdr (TLispMachine* pLM)
  *	SYMBOL ͤ void ǤʤСt ֤
  */
 TLMRESULT
-lispMachineState_Boundp (register TLispMachine* pLM)
+lispMachineState_Boundp (
+	register TLispMachine* pLM)
 {
 	register TLispManager*	pLispMgr ;
 	TLispEntity*	pEntArglist ;
@@ -1021,6 +1053,43 @@ lispMachineState_Boundp (register TLispMachine* pLM)
 	return	LMR_RETURN ;
 }
 
+/*	(default-boundp SYMBOL)
+ *
+ *	SYMBOL  void Ǥʤ default value äƤСt ֤
+ *	 buffer ǤͤʤΤʬΤ˻Ȥ롣
+ *
+ */
+TLMRESULT
+lispMachineState_DefaultBoundp (
+	register TLispMachine* pLM)
+{
+	register TLispManager*	pLispMgr ;
+	TLispEntity*	pEntArglist ;
+	TLispEntity*	pEntSymbol ;
+
+	assert (pLM != NULL) ;
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr != NULL) ;
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	assert (pEntArglist != NULL) ;
+	if (TFAILED (lispEntity_GetCar (pLispMgr, pEntArglist, &pEntSymbol)) ||
+		TFAILED (lispEntity_Symbolp (pLispMgr, pEntSymbol))) {
+		lispMachineCode_SetError (pLM) ;
+	} else {
+		TLispEntity*	pEntValue ;
+		TLispEntity*	pEntReturn ;
+
+		if (TFAILED (lispMachine_GetGlobalSymbolValue (pLM, pEntSymbol, &pEntValue)) ||
+			TSUCCEEDED (lispEntity_Voidp (pLispMgr, pEntValue))) {
+			lispMgr_CreateNil (pLispMgr, &pEntReturn) ;
+		} else {
+			lispMgr_CreateT (pLispMgr, &pEntReturn) ;
+		}
+		lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntReturn) ;
+	}
+	return	LMR_RETURN ;
+}
+
 /*
  *	(default-value SYMBOL)
  *
@@ -1029,7 +1098,8 @@ lispMachineState_Boundp (register TLispMachine* pLM)
  *	Ǥ롣
  */
 TLMRESULT
-lispMachineState_DefaultValue (register TLispMachine* pLM)
+lispMachineState_DefaultValue (
+	register TLispMachine* pLM)
 {
 	register TLispManager*	pLispMgr ;
 	TLispEntity*	pEntArglist ;
@@ -1337,7 +1407,7 @@ lispMachineState_Natnump (
 		lispMachineCode_SetError (pLM) ;
 		return	LMR_RETURN ;
 	}
-	if (TFAILED (lispEntity_GetIntegerValue (pLispMgr, pEntObject, &lValue)) ||
+	if (TSUCCEEDED (lispEntity_GetIntegerValue (pLispMgr, pEntObject, &lValue)) &&
 		lValue >= 0) {
 		lispMgr_CreateT (pLispMgr, &pEntRetval) ;
 	} else {
@@ -1391,3 +1461,130 @@ lispMachineState_Provide (
 	return	LMR_RETURN ;
 }
 
+/*
+ */
+TLMRESULT
+lispMachineState_Reverse (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	TLispEntity*	pEntArglist ;
+	TLispEntity*	pEntSrc ;
+	TLispEntity*	pEntHead ;
+
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	if (TFAILED (lispEntity_GetCar (pLispMgr, pEntArglist, &pEntSrc)) ||
+		TFAILED (lispEntity_Listp  (pLispMgr, pEntSrc))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	if (TFAILED (lispMgr_CreateNil (pLispMgr, &pEntHead)))
+		return	LMR_ERROR ;
+	lispEntity_AddRef (pLispMgr, pEntHead) ;
+	while (TFAILED (lispEntity_Nullp (pLispMgr, pEntSrc))) {
+		TLispEntity*	pEntTarget ;
+		TLispEntity*	pEntNext ;
+		TLispEntity*	pEntNewHead ;
+
+		if (TFAILED (lispEntity_GetCar (pLispMgr, pEntSrc, &pEntTarget)) ||
+			TFAILED (lispEntity_GetCdr (pLispMgr, pEntSrc, &pEntNext))) {
+			lispMachineCode_SetError (pLM) ;
+			goto	exit_func ;
+		}
+		if (TFAILED (lispMgr_CreateConscell (pLispMgr, pEntTarget, pEntHead, &pEntNewHead)))
+			return	LMR_ERROR ;
+		lispEntity_AddRef  (pLispMgr, pEntNewHead) ;
+		lispEntity_Release (pLispMgr, pEntHead) ;
+		pEntHead	= pEntNewHead ;
+		pEntSrc		= pEntNext ;
+	}
+	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntHead) ;
+  exit_func:
+	lispEntity_Release (pLispMgr, pEntHead) ;
+	return	LMR_RETURN ;
+}
+
+/*	(copy-sequence ARG)
+ */
+TLMRESULT
+lispMachineState_CopySequence (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	TLispEntity*	pEntArglist ;
+	TLispEntity*	pEntARG ;
+	int				nType ;
+
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	if (TFAILED (lispEntity_GetCar (pLispMgr, pEntArglist, &pEntARG))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	if (TFAILED (lispEntity_GetType (pLispMgr, pEntARG, &nType)))
+		return	LMR_ERROR ;
+	switch (nType) {
+	case	LISPENTITY_CONSCELL:
+	{
+		TLispConscell	lstCopied ;
+		TLispEntity*	pEntCAR ;
+		TLispEntity*	pEntCDR ;
+
+		lstCopied.m_pCar = lstCopied.m_pCdr	= NULL ;
+		while (TFAILED (lispEntity_Nullp (pLispMgr, pEntARG))) {
+			if (TFAILED (lispEntity_GetCar (pLispMgr, pEntARG, &pEntCAR)) ||
+				TFAILED (lispEntity_GetCdr (pLispMgr, pEntARG, &pEntCDR))) {
+				lispMachineCode_SetError (pLM) ;
+				break ;
+			}
+			if (TFAILED (lispEntity_Push2List (pLispMgr, &lstCopied, pEntCAR)))
+				return	LMR_ERROR ;
+			pEntARG	= pEntCDR ;
+		}
+		if (lstCopied.m_pCar == NULL) {
+			register TLispEntity*	pEntNil	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_NIL) ;
+			lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntNil) ;
+		} else {
+			lispMachineCode_SetLReg (pLM, LM_LREG_ACC, lstCopied.m_pCar) ;
+			lispEntity_Release (pLispMgr, lstCopied.m_pCar) ;
+			lstCopied.m_pCar	= NULL ;
+		}
+		break ;
+	}
+	case	LISPENTITY_VECTOR:
+	{
+		TLispEntity**	ppEntElement ;
+		int				nElement ;
+		TLispEntity*	pEntRetval ;
+
+		(void) lispEntity_GetVectorValue (pLispMgr, pEntARG, &ppEntElement, &nElement) ;
+		if (TFAILED (lispMgr_CreateVector (pLispMgr, ppEntElement, nElement, &pEntRetval)))
+			return	LMR_ERROR ;
+		lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+		break ;
+	}
+	case	LISPENTITY_STRING:
+	{
+		const Char*		pString ;
+		int				nString ;
+		TLispEntity*	pEntRetval ;
+
+		(void) lispEntity_GetStringValue (pLispMgr, pEntARG, &pString, &nString) ;
+		if (TFAILED (lispMgr_CreateString (pLispMgr, pString, nString, &pEntRetval)))
+			return	LMR_ERROR ;
+		lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+		break ;
+	}
+	default:
+		if (TSUCCEEDED (lispEntity_Nullp (pLispMgr, pEntARG))) {
+			TLispEntity*	pEntRetval ;
+
+			lispMgr_CreateNil (pLispMgr, &pEntRetval);
+			lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+		} else {
+			lispMachineCode_SetError (pLM) ;
+		}
+		break ;
+	}
+	return	LMR_RETURN ;
+}
+
