@@ -24,10 +24,18 @@
 #include "lmachinep.h"
 #include "filename.h"
 
+typedef struct {
+	TLispEntity*	m_pEntBufferToKill ;
+	TLispEntity*	m_pEntNextBuffer ;
+}	TKillBufferEnumProcArg ;
+
 static	TLMRESULT	lispMachineState_saveCurrentBufferFinalize (TLispMachine*) ;
 
 static	TLMRESULT	lispMachineState_point	(TLispMachine*, Boolean (*)(TLispManager*, TLispEntity*, int*)) ;
 static	TLMRESULT	lispMachineState_pointMarker (TLispMachine*, Boolean (*)(TLispManager*, TLispEntity*, int*)) ;
+static	Boolean		lispMachine_killBufferFrameProc	(TLispMachine*, TLispEntity*, void*, Boolean*) ;
+static	Boolean		lispMachine_killBufferWindowProc	(TLispMachine*, TLispEntity*, void*, Boolean*) ;
+static	Boolean		lispMachine_enumBufferList	(TLispMachine*, TLispEntity*, void*, Boolean*) ;
 
 /*
  *	(current-buffer)
@@ -276,7 +284,7 @@ lispMachineState_Bobp (
 
 	if (TFAILED (lispMachineCode_GetCurrentBuffer (pLM, &pEntBuffer)) ||
 		TFAILED (lispBuffer_Point    (pLispMgr, pEntBuffer, &nPoint)) ||
-		TFAILED (lispBuffer_PointMax (pLispMgr, pEntBuffer, &nPointMin))) {
+		TFAILED (lispBuffer_PointMin (pLispMgr, pEntBuffer, &nPointMin))) {
 		lispMachineCode_SetError (pLM) ;
 		return	LMR_RETURN ;
 	}
@@ -821,6 +829,61 @@ lispMachineState_CharAfter (
 	return	LMR_RETURN ;
 }
 
+/*	(char-before &optional POS)
+ *
+ *	Return character in current buffer preceding position POS.
+ *	POS is an integer or a marker.
+ *	If POS is out of range, the value is nil.
+ */
+TLMRESULT
+lispMachineState_CharBefore (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	TLispEntity*	pEntBuffer ;
+	TLispEntity*	pEntArglist ;
+	TLispEntity*	pEntPOS ;
+	TLispEntity*	pEntRetval ;
+	int				nBufString ;
+	int				nPoint, nPointMin, nPointMax, nPointBTop ;
+	TBufStringMarker	mkBuffer ;
+	
+	if (TFAILED (lispMachineCode_GetCurrentBuffer (pLM, &pEntBuffer)) ||
+		TFAILED (lispBuffer_GetString (pLispMgr, pEntBuffer, &mkBuffer, &nBufString))) 
+		goto	error ;
+	assert (pEntBuffer != NULL) ;
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	lispEntity_GetCar (pLispMgr, pEntArglist, &pEntPOS) ;
+	if (TSUCCEEDED (lispEntity_Nullp (pLispMgr, pEntPOS))) {
+		lispBuffer_Point (pLispMgr, pEntBuffer, &nPoint) ;
+	} else {
+		TLispNumber	num ;
+
+		if (TFAILED (lispEntity_GetNumberOrMarkerValue (pLispMgr, pEntPOS, &num)) ||
+			TSUCCEEDED (num.m_fFloatp)) 
+			goto	error ;
+		nPoint	= num.m_Value.m_lLong ;
+	}
+	/*	preceding ʤΤ 1 롣*/
+	nPoint	-- ;
+
+	lispBuffer_PointMin (pLispMgr, pEntBuffer, &nPointMin) ;
+	lispBuffer_PointMax (pLispMgr, pEntBuffer, &nPointMax) ;
+	lispBuffer_PointBufferTop (pLispMgr, pEntBuffer, &nPointBTop) ;
+	if (nPoint < nPointMin || nPoint >= nPointMax) {
+		lispMgr_CreateNil (pLispMgr, &pEntRetval) ;
+	} else {
+		TBufStringMarker_Forward (&mkBuffer, nPoint - nPointBTop) ;
+		lispMgr_CreateInteger (pLispMgr, TBufStringMarker_GetChar (&mkBuffer), &pEntRetval) ;
+	}
+	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+	return	LMR_RETURN ;
+
+  error:
+	lispMachineCode_SetError (pLM) ;
+	return	LMR_RETURN ;
+}
+
 /*
  *	interactive built-in function:
  *		(narrow-to-region START END)
@@ -1158,11 +1221,6 @@ lispMachineState_GetFileBuffer (
 	/*	ե̾򸰤ˤơbuffer 򸡺롣*/
 	pStrNAME	= TVarbuffer_GetBuffer (&vbufNAME) ;
 	nStrNAME	= TVarbuffer_GetUsage  (&vbufNAME) ;
-#if defined (DEBUG)
-	fprintf (stderr, "search-buffer: \"") ;
-	cputstr (stderr, pStrNAME, nStrNAME) ;
-	fprintf (stderr, "\"\n") ;
-#endif
 	if (TFAILED (lispMachine_GetFileBuffer (pLM, pStrNAME, nStrNAME, &pEntRetval))) 
 		lispMgr_CreateNil (pLispMgr, &pEntRetval) ;
 	TVarbuffer_Uninitialize (&vbufNAME) ;
@@ -1241,4 +1299,278 @@ lispMachineState_BufferModifiedp (
 	return	LMR_RETURN ;
 }
 
+/*	built-in function:
+ *		(buffer-size &optional BUFFER)
+ */
+TLMRESULT
+lispMachineState_BufferSize (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	TLispEntity*		pEntArglist ;
+	TLispEntity*		pEntBuffer ;
+	TLispEntity*		pEntRetval ;
+	TBufStringMarker	mk ;
+	int					nLength ;
+
+	assert (pLM      != NULL) ;
+	assert (pLispMgr != NULL) ;
+
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	assert (pEntArglist != NULL) ;
+	lispEntity_GetCar (pLispMgr, pEntArglist, &pEntBuffer) ;
+	if (TSUCCEEDED (lispEntity_Nullp (pLispMgr, pEntBuffer))) 
+		lispMachineCode_GetCurrentBuffer (pLM, &pEntBuffer) ;
+	if (TFAILED (lispEntity_Bufferp (pLispMgr, pEntBuffer))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	if (TFAILED (lispBuffer_GetFullString (pLispMgr, pEntBuffer, &mk, &nLength))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	if (TFAILED (lispMgr_CreateInteger (pLispMgr, nLength, &pEntRetval)))
+		return	LMR_ERROR ;
+	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+	return	LMR_RETURN ;
+}
+
+/*	(local-variable-p VARIABLE &optional BUFFER)
+ */
+TLMRESULT
+lispMachineState_LocalVariablep (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	TLispEntity*		pEntArglist ;
+	TLispEntity*		pEntBuffer ;
+	TLispEntity*		pEntVAR ;
+	TLispEntity*		pEntRetval ;
+	TLispEntity*		pEntTmp ;
+
+	assert (pLM      != NULL) ;
+	assert (pLispMgr != NULL) ;
+
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	assert (pEntArglist != NULL) ;
+	if (TFAILED (lispEntity_GetCar  (pLispMgr, pEntArglist, &pEntVAR)) ||
+		TFAILED (lispEntity_GetCadr (pLispMgr, pEntArglist, &pEntBuffer))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	if (TSUCCEEDED (lispEntity_Nullp (pLispMgr, pEntBuffer))) 
+		lispMachineCode_GetCurrentBuffer (pLM, &pEntBuffer) ;
+
+	if (TFAILED (lispEntity_Bufferp (pLispMgr, pEntBuffer))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	if (TFAILED (lispMachine_GetBufferLocalSymbolValue (pLM, pEntBuffer, pEntVAR, &pEntTmp))) {
+		pEntRetval	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_NIL) ;
+	} else {
+		pEntRetval	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_T) ;
+	}
+	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+	return	LMR_RETURN ;
+}
+
+/*	(kill-buffer BUFFER)
+ *
+ *	kill-buffer is an interactive built-in function.
+ *
+ *	Kill the buffer BUFFER.
+ *	The argument may be a buffer or may be the name of a buffer.
+ *	An argument of nil means kill the current buffer.
+ *
+ *	Value is t if the buffer is actually killed, nil if user says no.
+ */
+TLMRESULT
+lispMachineState_KillBuffer (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	TLispEntity*		pEntArglist ;
+	TLispEntity*		pEntBUFFER ;
+	TLispEntity*		pEntBufferToKill ;
+	TLispEntity*		pEntNextBuffer ;
+	TLispEntity*		pEntRetval ;
+	const Char*			strBuffer ;
+	int					nstrBuffer ;
+
+	assert (pLM      != NULL) ;
+	assert (pLispMgr != NULL) ;
+
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	assert (pEntArglist != NULL) ;
+	if (TFAILED (lispEntity_GetCar  (pLispMgr, pEntArglist, &pEntBUFFER))) {
+		goto	error ;
+	}
+
+	/*	 nil ξˤϡcurrent-buffer 
+	 *	 string ξˤϡ̾ä buffer 
+	 *	 buffer ʤ顢 buffer 
+	 */
+	if (TSUCCEEDED (lispEntity_Nullp (pLispMgr, pEntBUFFER))) {
+		lispMachineCode_GetCurrentBuffer (pLM, &pEntBufferToKill) ;
+	} else if (TSUCCEEDED (lispEntity_GetStringValue (pLispMgr, pEntBUFFER, &strBuffer, &nstrBuffer))) {
+		if (TFAILED (lispMachine_GetBuffer (pLM, strBuffer, nstrBuffer, &pEntBufferToKill)))
+			goto	error ;
+	} else if (TSUCCEEDED (lispEntity_Bufferp (pLispMgr, pEntBUFFER))) {
+		pEntBufferToKill	= pEntBUFFER ;
+	} else {
+		goto	error ;
+	}
+	if (pEntBufferToKill == NULL || 
+		TFAILED (lispEntity_Bufferp (pLispMgr, pEntBufferToKill))) 
+		goto	error ;
+
+	if (TFAILED (lispBuffer_GetNext (pEntBufferToKill, &pEntNextBuffer)) || 
+		pEntNextBuffer == pEntBufferToKill) {
+		lispMgr_CreateNil (pLispMgr, &pEntRetval) ;
+	} else {
+		TKillBufferEnumProcArg	arg ;
+
+		arg.m_pEntBufferToKill	= pEntBufferToKill ;
+		arg.m_pEntNextBuffer	= pEntNextBuffer ;
+
+		/*	buffer դƤ Window ¸ߤȦʤΤǡν
+		 *	򤷤ʤФʤʤ
+		 *
+		 *	ȡ줬Ǹ buffer ȺǤʤ褦ˤʤСġ
+		 */
+		/*	Ƥ frame Ƥ window Ф...
+		 */
+		if (TFAILED (lispMachine_EnumFrame (pLM, lispMachine_killBufferFrameProc, &arg)))
+			goto	error ;
+		if (TFAILED (lispMachine_RemoveBuffer (pLM, pEntBufferToKill))) 
+			goto	error ;
+		lispMgr_CreateT (pLispMgr, &pEntRetval) ;
+	}
+	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+	return	LMR_RETURN ;
+
+  error:
+	lispMachineCode_SetError (pLM) ;
+	return	LMR_RETURN ;
+}
+
+/*	(buffer-name &optional BUFFER)
+ */
+TLMRESULT
+lispMachineState_BufferName (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	TLispEntity*		pEntArglist ;
+	TLispEntity*		pEntBUFFER ;
+	TLispEntity*		pEntRetval ;
+
+	assert (pLM      != NULL) ;
+	assert (pLispMgr != NULL) ;
+
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	assert (pEntArglist != NULL) ;
+	if (TFAILED (lispEntity_GetCar  (pLispMgr, pEntArglist, &pEntBUFFER))) 
+		goto	error ;
+
+	if (TSUCCEEDED (lispEntity_Nullp (pLispMgr, pEntBUFFER))) 
+		lispMachineCode_GetCurrentBuffer (pLM, &pEntBUFFER) ;
+	if (TFAILED (lispEntity_Bufferp (pLispMgr, pEntBUFFER))) 
+		goto	error ;
+	if (TFAILED (lispBuffer_GetName (pLispMgr, pEntBUFFER, &pEntRetval))) 
+		lispMgr_CreateNil (pLispMgr, &pEntRetval) ;
+	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+	return	LMR_RETURN ;
+
+  error:
+	lispMachineCode_SetError (pLM) ;
+	return	LMR_RETURN ;
+}
+
+/*	(buffer-list &optional FRAME)
+ *
+ *	¸ߤƤХåեƤʤꥹȤ֤롣
+ *	⤷FRAME  frame ʤ顢Υե졼 buffer list
+ *	롣
+ *	ࡢFRAME դưɤʬʤʡġޤ路
+ */
+TLMRESULT
+lispMachineState_BufferList (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	TLispEntity*		pEntArglist ;
+	TLispEntity*		pEntFRAME ;
+	TLispConscell		lstBuffer ;
+
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	assert (pEntArglist != NULL) ;
+	if (TFAILED (lispEntity_GetCar (pLispMgr, pEntArglist, &pEntFRAME)) ||
+		TFAILED (lispEntity_Nullp  (pLispMgr, pEntFRAME))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	lstBuffer.m_pCar = lstBuffer.m_pCdr = NULL ;
+	lispMachine_EnumBuffer (pLM, True, lispMachine_enumBufferList, &lstBuffer) ;
+	if (lstBuffer.m_pCar != NULL) {
+		lispMachineCode_SetLReg (pLM, LM_LREG_ACC, lstBuffer.m_pCar) ;
+		lispEntity_Release (pLispMgr, lstBuffer.m_pCar) ;
+	} else {
+		lispMachineCode_SetLReg (pLM, LM_LREG_ACC, lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_NIL)) ;
+	}
+	return	LMR_RETURN ;
+}
+
+Boolean
+lispMachine_killBufferFrameProc (
+	register TLispMachine*	pLM,
+	register TLispEntity*	pEntFrame,
+	register void*			pCaller,
+	register Boolean*		pfContinue)
+{
+	return	lispMachine_EnumWindow (pLM, pEntFrame, lispMachine_killBufferWindowProc, pCaller) ;
+}
+
+Boolean
+lispMachine_killBufferWindowProc (
+	register TLispMachine*	pLM,
+	register TLispEntity*	pEntWindow,
+	register void*			pCaller,
+	register Boolean*		pfContinue)
+{
+	register TLispManager*				pLispMgr ;
+	register TKillBufferEnumProcArg*	pArg ;
+	TLispEntity*	pEntWindowBuffer ;
+
+	assert (pLM != NULL) ;
+	assert (pEntWindow != NULL) ;
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr != NULL) ;
+	pArg		= (TKillBufferEnumProcArg*) pCaller ;
+	assert (pArg != NULL &&
+			pArg->m_pEntBufferToKill != NULL && 
+			pArg->m_pEntNextBuffer != NULL) ;
+	
+	if (TFAILED (lispWindow_GetBuffer (pEntWindow, &pEntWindowBuffer)))
+		return	False ;
+	if (pEntWindowBuffer == pArg->m_pEntBufferToKill) 
+		lispWindow_SetBuffer (pLispMgr, pEntWindow, pArg->m_pEntNextBuffer) ;
+	return	True ;
+}
+
+Boolean
+lispMachine_enumBufferList (
+	register TLispMachine*	pLM,
+	register TLispEntity*	pEntBuffer,
+	register void*			pCaller,
+	register Boolean*		pfContinue)
+{
+	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	register TLispConscell*	pList		= (TLispConscell*) pCaller ;
+
+	assert (pLispMgr != NULL) ;
+	assert (pList != NULL) ;
+
+	return	lispEntity_Push2List (pLispMgr, pList, pEntBuffer) ;
+}
 
