@@ -428,8 +428,7 @@ lispMachineState_EvalSymbol (
 
 	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pTarget) ;
 
-	if (lispEntity_Nullp (pLispMgr, pTarget) ||
-		lispEntity_Tp    (pLispMgr, pTarget)) 
+	if (TFAILED (lispMgr_SymbolRequireEvalp (pLispMgr, pTarget)))
 		return	LMR_RETURN ;
 
 	if (TFAILED (lispMachine_GetCurrentSymbolValue (pLM, pTarget, &pReturn)) ||
