@@ -22,14 +22,21 @@
 #include <stdio.h>
 #include <assert.h>
 #if !defined (__linux__)
+#if defined (__SVR4) && defined (__sun)
+#include <limits.h>
+#define DEFFILEMODE (S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)
+#else
 #include <sys/syslimits.h>
 #endif
+#endif
 #include <sys/param.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <fcntl.h>
 #include <errno.h>
 #include <unistd.h>
 #include <pwd.h>
+#include <time.h>
 #include "lmachinep.h"
 #include "lmstate.h"
 #include "filename.h"
@@ -350,6 +357,7 @@ lispMachineState_InsertFileContents (
 	KFILE			kfile ;
 	int				nReadCount ;
 	int				nPoint ;
+	register int	nCodingSys ;
 
 	assert (pLM != NULL) ;
 	assert (pLispMgr != NULL) ;
@@ -367,7 +375,11 @@ lispMachineState_InsertFileContents (
 	lispBuffer_Point (pLispMgr, pEntBuffer, &nPoint) ;
 	nReadCount	= 0 ;
 
-	if (!KFile_Open (&kfile, rchPath, KCODING_SYSTEM_UNKNOWN)) {
+	/*	ɤ߹ե coding-system  coding-system-for-read ޤ
+	 *	file-coding-system-alist Ƿꤹ롣
+	 */
+	nCodingSys	= lispMachine_GetCodingSystemForRead (pLM) ;
+	if (!KFile_Open (&kfile, rchPath, nCodingSys)) {
 #if defined (DEBUG) || 1
 		fprintf (stderr, "Can't open file: \"%s\"\n", rchPath) ;
 #endif
@@ -433,18 +445,19 @@ lispMachineState_RenameFile (
 	register TLispMachine*	pLM)
 {
 	register TLispManager*	pLispMgr ;
-	TLispEntity*	pEntArglist ;
-	TLispEntity*	pEntFile ;
-	TLispEntity*	pEntNewName ;
-	TLispEntity*	pEntOkIfAExists ;
-	TLispEntity*	pEntNil ;
-	char			bufOldPath [PATH_MAX + 1] ;
-	char			bufNewPath [PATH_MAX + 1] ;
-	char			bufCopy [4096] ;
-	struct stat		bufstat ;
-	register int	nRetval ;
-	register FILE*	fpSrc ;
-	register FILE*	fpDest ;
+	TLispEntity*		pEntArglist ;
+	TLispEntity*		pEntFile ;
+	TLispEntity*		pEntNewName ;
+	TLispEntity*		pEntOkIfAExists ;
+	TLispEntity*		pEntNil ;
+	char				bufOldPath [PATH_MAX + 1] ;
+	char				bufNewPath [PATH_MAX + 1] ;
+	char				bufCopy [4096] ;
+	struct stat			bufstat ;
+	register int		nRetval ;
+	register FILE*		fpSrc ;
+	register FILE*		fpDest ;
+	register Boolean	fError ;
 
 	assert (pLM != NULL) ;
 	pLispMgr	= pLM->m_pLispMgr ;
@@ -471,14 +484,17 @@ lispMachineState_RenameFile (
 	}
 
 	nRetval	= lstat (bufNewPath, &bufstat) ;
-	if ((nRetval == -1 && errno != ENOENT) ||
-		(nRetval == 0  && S_ISREG (bufstat.st_mode) == 0)) {
-		/*	ե뤬ʤȤǤϤʤ*/
-		lispMachineCode_SetError (pLM) ;
-		return	LMR_RETURN ;
-	} else if (TSUCCEEDED (lispEntity_Nullp (pLispMgr, pEntOkIfAExists))) {
-		lispMachineCode_SetError (pLM) ;
-		return	LMR_RETURN ;
+	if (nRetval == 0) {
+		if (S_ISREG (bufstat.st_mode) == 0 ||
+			TSUCCEEDED (lispEntity_Nullp (pLispMgr, pEntOkIfAExists))) {
+			lispMachineCode_SetError (pLM) ;
+			return	LMR_RETURN ;
+		}
+	} else {
+		if (errno != ENOENT) {
+			lispMachineCode_SetError (pLM) ;
+			return	LMR_RETURN ;
+		}
 	}
 
 	/*	rename  copy ʤΤ */
@@ -493,16 +509,31 @@ lispMachineState_RenameFile (
 		fclose (fpSrc) ;
 		return	LMR_RETURN ;
 	}
+
+	fError	= False ;
 	while (!feof (fpSrc)) {
 		register int nread	= fread (bufCopy, 1, sizeof (bufCopy), fpSrc) ;
-		if (nread <= 0)
+		if (nread <= 0) {
+			fError	= (nread < 0) ;
 			break ;
-		if (fwrite (bufCopy, 1, nread, fpDest) <= 0)
+		}
+		if (fwrite (bufCopy, 1, nread, fpDest) <= 0) {
+			fError	= True ;
 			break ;
+		}
 	}
-	fclose (fpSrc) ;
-	fclose (fpDest) ;
+	if (fclose (fpSrc) != 0)
+		fError	= True ;
+	if (fclose (fpDest) != 0)
+		fError	= True ;
 
+	if (! fError) {
+		if (unlink (bufOldPath) != 0)
+			fError	= True ;
+	}
+	if (fError) {
+		lispMachineCode_SetError (pLM) ;
+	}
 	lispMgr_CreateNil (pLispMgr, &pEntNil) ;
 	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntNil) ;
 	return	LMR_RETURN ;
@@ -591,26 +622,27 @@ lispMachineState_WriteRegion (
 	register TLispMachine*	pLM)
 {
 	register TLispManager*	pLispMgr ;
-	TLispEntity*	pEntArglist ;
-	TLispEntity*	pEntBuffer ;
-	TLispEntity*	pEntStart ;
-	TLispEntity*	pEntEnd ;
-	TLispEntity*	pEntFileName ;
-	TLispEntity*	pEntAppend ;
-	TLispEntity*	pEntVisit ;
-	TLispEntity*	pEntNil ;
-	const Char*		strFile ;
-	int				nstrFile ;
-	char			bufFile [PATH_MAX + 1] ;
-	char			buffer [2064] ;
-	register char*	ptr ;
-	TLispNumber		numStart, numEnd ;
-	register int	nUsage, nptr, nctext, nCopy, nStart, nEnd ;
-	int				nLength, nPointTop, nPointEnd ;
-	register FILE*	fp ;
-	register Char	cc ;
+	TLispEntity*		pEntArglist ;
+	TLispEntity*		pEntBuffer ;
+	TLispEntity*		pEntStart ;
+	TLispEntity*		pEntEnd ;
+	TLispEntity*		pEntFileName ;
+	TLispEntity*		pEntAppend ;
+	TLispEntity*		pEntVisit ;
+	TLispEntity*		pEntNil ;
+	const Char*			strFile ;
+	int					nstrFile ;
+	char				bufFile [PATH_MAX + 1] ;
+	char				buffer [2064] ;
+	register char*		ptr ;
+	TLispNumber			numStart, numEnd ;
+	register int		nUsage, nptr, nctext, nCopy, nStart, nEnd ;
+	int					nLength, nPointTop, nPointEnd ;
+	register FILE*		fp ;
+	register Char		cc ;
 	KANJISTATEMACHINE	ksm ;
 	TBufStringMarker	mk ;
+	register int		nCodingSys ;
 
 	assert (pLM != NULL) ;
 	pLispMgr	= pLM->m_pLispMgr ;
@@ -678,7 +710,17 @@ lispMachineState_WriteRegion (
 		return	LMR_RETURN ;
 	}
 
-	InitializeKanjiFiniteStateMachine (&ksm, KCODING_SYSTEM_ISO2022JP2) ;
+	/*	coding-system-for-write,
+	 *	buffer-file-coding-system ν֤ǽ񤭹 coding-system 
+	 *	ꤹ롣
+	 *	file-coding-system-alist ϥե̾ coding-system 
+	 *	ꤹΤ˻Ȥ롢
+	 */
+	nCodingSys	= lispMachine_GetCodingSystemForWrite (pLM) ;
+	if (nCodingSys < 0 || nCodingSys >= MAX_KCODING_SYSTEM)
+		nCodingSys	= KCODING_SYSTEM_ISO2022JP2 ;
+
+	InitializeKanjiFiniteStateMachine (&ksm, nCodingSys) ;
 
 	ptr		= buffer ;
 	nptr	= 0 ;
@@ -788,6 +830,358 @@ lispMachineState_SetFileModes (
 	return	LMR_RETURN ;
 }
 
+TLMRESULT
+lispMachineState_FileWritablep (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr ;
+	TLispEntity*		pEntArglist ;
+	TLispEntity*		pEntFILENAME ;
+	TLispEntity*		pEntRetval ;
+	char				bufFile [PATH_MAX + 1] ;
+	register Boolean	fWritable ;
+	struct stat			bufstat ;
+
+	assert (pLM != NULL) ;
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr != NULL) ;
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	if (TFAILED (lispEntity_GetCar (pLispMgr, pEntArglist, &pEntFILENAME)) ||
+		TFAILED (lispMachine_getFileNameFromEntity (pLispMgr, pEntFILENAME, bufFile, PATH_MAX))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	if (stat (bufFile, &bufstat) >= 0) {
+		fWritable	= (access (bufFile, 2) == 0) ;
+	} else {
+		const char*	pDir ;
+		int			nDir, nPath ;
+
+		/* parent directory դʤС*/
+		nPath	= strlen (bufFile) ;
+		(void) GetFileDirectoryAnsi (bufFile, nPath, &pDir, &nDir) ;
+		if (nDir > nPath || nDir > PATH_MAX) {
+			lispMachineCode_SetError (pLM) ;
+			return	LMR_RETURN ;
+		}
+		bufFile [nDir]	= '\0' ;
+		fWritable	= (access (bufFile, 2) == 0) ;
+	}
+	pEntRetval	= lispMgr_GetReservedEntity (pLispMgr, (fWritable)? LISPMGR_INDEX_T : LISPMGR_INDEX_NIL) ;
+	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+	return	LMR_RETURN ;
+}
+
+TLMRESULT
+lispMachineState_FileReadablep (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr ;
+	TLispEntity*		pEntArglist ;
+	TLispEntity*		pEntFILENAME ;
+	TLispEntity*		pEntRetval ;
+	char				bufFile [PATH_MAX + 1] ;
+	register Boolean	fReadable ;
+	struct stat			bufstat ;
+
+	assert (pLM != NULL) ;
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr != NULL) ;
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	if (TFAILED (lispEntity_GetCar (pLispMgr, pEntArglist, &pEntFILENAME)) ||
+		TFAILED (lispMachine_getFileNameFromEntity (pLispMgr, pEntFILENAME, bufFile, PATH_MAX))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	if (stat (bufFile, &bufstat) >= 0) {
+		fReadable	= (access (bufFile, R_OK) == 0) ;
+	} else {
+		fReadable	= False ;
+	}
+	pEntRetval	= lispMgr_GetReservedEntity (pLispMgr, (fReadable)? LISPMGR_INDEX_T : LISPMGR_INDEX_NIL) ;
+	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+	return	LMR_RETURN ;
+}
+
+TLMRESULT
+lispMachineState_MakeDirectoryInternal (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr ;
+	TLispEntity*	pEntArglist ;
+	TLispEntity*	pEntDIRECTORY ;
+	TLispEntity*	pEntRetval ;
+	char			bufDir [PATH_MAX + 1] ;
+
+	assert (pLM != NULL) ;
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr != NULL) ;
+
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	if (TFAILED (lispEntity_GetCar  (pLispMgr, pEntArglist, &pEntDIRECTORY)) ||
+		TFAILED (lispMachine_getFileNameFromEntity (pLispMgr, pEntDIRECTORY, bufDir, PATH_MAX))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	if (mkdir (bufDir, S_IRWXU) != 0) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	lispMgr_CreateNil (pLispMgr, &pEntRetval) ;
+	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+	return	LMR_RETURN ;
+}
+
+static char		make_temp_name_tbl [64] = {
+	'A','B','C','D','E','F','G','H',
+	'I','J','K','L','M','N','O','P',
+	'Q','R','S','T','U','V','W','X',
+	'Y','Z','a','b','c','d','e','f',
+	'g','h','i','j','k','l','m','n',
+	'o','p','q','r','s','t','u','v',
+	'w','x','y','z','0','1','2','3',
+	'4','5','6','7','8','9','-','_'
+};
+static unsigned	make_temp_name_count = 0, make_temp_name_count_initialized_p = 0 ;
+
+/*	(make-temp-name PREFIX)
+ */
+TLMRESULT
+lispMachineState_MakeTempName (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr ;
+	TLispEntity*		pEntArglist ;
+	TLispEntity*		pEntPREFIX ;
+	TLispEntity*		pEntRetval	= NULL ;
+	const Char*			strPREFIX ;
+	int					nstrPREFIX ;
+	register char*		pDest ;
+	register char*		strFile ;
+	TVarbuffer			vbuf ;
+	TVarbuffer			vbufResult ;
+	register Char*		strResult ;
+	register int		nResult, nfd ;
+	register unsigned	num = make_temp_name_count ;
+	register pid_t		pid ;
+
+	assert (pLM != NULL) ;
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr != NULL) ;
+
+	TVarbuffer_Initialize (&vbufResult, sizeof (Char)) ;
+	TVarbuffer_Initialize (&vbuf,       sizeof (char)) ;
+
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	if (TFAILED (lispEntity_GetCar (pLispMgr, pEntArglist, &pEntPREFIX)) ||
+		TFAILED (lispEntity_GetStringValue (pLispMgr, pEntPREFIX, &strPREFIX, &nstrPREFIX))) 
+		goto	error ;
+
+	if (TFAILED (TVarbuffer_Require (&vbuf, nstrPREFIX + 3 + 3 + 1)))
+		goto	error ;
+
+	/*	ࡢfile-system  coding-system ߤȤ... */
+	strFile	= TVarbuffer_GetBuffer (&vbuf) ;
+	pDest	= strFile ;
+	while (nstrPREFIX -- > 0) {
+		if (! Char_IsAscii (*strPREFIX)) 
+			goto	error ;
+		*pDest ++	= (char) *strPREFIX ++ ;
+	}
+
+	pid			= (int) getpid ();
+	pDest [0]	= make_temp_name_tbl [pid & 63], pid >>= 6 ;
+	pDest [1]	= make_temp_name_tbl [pid & 63], pid >>= 6 ;
+	pDest [2]	= make_temp_name_tbl [pid & 63], pid >>= 6 ;
+	pDest		+= 3 ;
+	pDest [3]	= '\0' ;
+	  
+	if (!make_temp_name_count_initialized_p) {
+		make_temp_name_count	= (unsigned) time (NULL) ;
+		make_temp_name_count_initialized_p	= 1 ;
+    }
+
+	pDest [0]	= make_temp_name_tbl[num & 63], num >>= 6 ;
+	pDest [1]	= make_temp_name_tbl[num & 63], num >>= 6 ;
+	pDest [2]	= make_temp_name_tbl[num & 63], num >>= 6 ;
+
+	/* Poor man's congruential RN generator.  Replace with
+	   ++make_temp_name_count for debugging.  */
+	make_temp_name_count += 25229 ;
+	make_temp_name_count %= 225307 ;
+	
+	/*	եǤʤä餱NFS  tmpfile ꤹ뤳Ȥ
+	 *	ʤȻפΤ... */
+	nfd	= open (strFile, O_CREAT | O_SYNC | O_EXCL | O_RDWR, S_IRUSR | S_IWUSR) ;
+	if (nfd < 0) {
+		fprintf (stderr, "Cannot create temporary name for prefix `") ;
+		lispEntity_Print (pLispMgr, pEntPREFIX) ;
+		fprintf (stderr, "'\n") ;
+		goto	error ;
+	}
+	close (nfd) ;
+
+	nResult	= strlen (strFile) ;
+	if (TFAILED (TVarbuffer_Require    (&vbufResult, nResult)))
+		goto	error ;
+	
+	strResult	= TVarbuffer_GetBuffer (&vbufResult) ;
+	strtocstr (strResult, strFile, nResult) ;
+	if (TFAILED (lispMgr_CreateString (pLispMgr, strResult, nResult, &pEntRetval)))
+		return	LMR_ERROR ;
+
+  error:
+	TVarbuffer_Uninitialize (&vbufResult) ;
+	TVarbuffer_Uninitialize (&vbuf) ;
+	if (pEntRetval != NULL) {
+		lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+	} else {
+		lispMachineCode_SetError (pLM) ;
+	}
+	return	LMR_RETURN ;
+}
+
+/*	(file-attributes FILENAME)
+ *
+ *	0. t for directory, string (name linked to) for symbolic link, or nil.
+ *	1. Number of links to file.
+ *	2. File uid.
+ *	3. File gid.
+ *	4. Last access time, as a list of two integers.
+ *		First integer has high-order 16 bits of time, second has low 16 bits.
+ *	5. Last modification time, likewise.
+ *	6. Last status change time, likewise.
+ *	7. Size in bytes.
+ *		This is a floating point number if the size is too large for an integer.
+ *	8. File modes, as a string of ten letters or dashes as in ls -l.
+ *	9. t iff file's gid would change if file were deleted and recreated.
+ *	10. inode number.  If inode number is larger than the Emacs integer,
+ *	this is a cons cell containing two integers: first the high part,
+ *	then the low 16 bits.
+ *	11. Device number.  If it is larger than the Emacs integer, this is
+ *	a cons cell, similar to the inode number.
+ */
+TLMRESULT
+lispMachineState_FileAttributes (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr ;
+	TLispEntity*			pEntArglist ;
+	TLispEntity*			pEntFILENAME ;
+	TLispEntity*			pEntRetval ;
+	char					bufFile [PATH_MAX + 1] ;
+	struct stat				bufstat ;
+	TLispEntity*			rEntResult [12] ;
+	Char					rbufMode [10] ;
+	register TLMRESULT		nRetval	= LMR_ERROR ;
+	register int			i ;
+	register unsigned int	uMask ;
+
+	assert (pLM != NULL) ;
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr != NULL) ;
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	if (TFAILED (lispEntity_GetCar (pLispMgr, pEntArglist, &pEntFILENAME)) ||
+		TFAILED (lispMachine_getFileNameFromEntity (pLispMgr, pEntFILENAME, bufFile, PATH_MAX))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	if (stat (bufFile, &bufstat) < 0) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	for (i = 0 ; i < NELEMENTS (rEntResult) ; i ++)
+		rEntResult [i]	= NULL ;
+
+	if ((bufstat.st_mode & S_IFMT) == S_IFLNK ||
+		(bufstat.st_mode & S_IFMT) == S_IFDIR) {
+		lispMgr_CreateT   (pLispMgr, &rEntResult [0]) ;
+	} else {
+		lispMgr_CreateNil (pLispMgr, &rEntResult [0]) ;
+	}
+	lispEntity_AddRef     (pLispMgr, rEntResult [0]) ;
+
+	if (TFAILED (lispMgr_CreateInteger (pLispMgr, bufstat.st_nlink, &rEntResult [1])))
+		goto	exit_func ;
+	lispEntity_AddRef     (pLispMgr, rEntResult [1]) ;
+	if (TFAILED (lispMgr_CreateInteger (pLispMgr, bufstat.st_uid,   &rEntResult [2])))
+		goto	exit_func ;
+	lispEntity_AddRef     (pLispMgr, rEntResult [2]) ;
+	if (TFAILED (lispMgr_CreateInteger (pLispMgr, bufstat.st_gid,   &rEntResult [3])))
+		goto	exit_func ;
+	lispEntity_AddRef     (pLispMgr, rEntResult [3]) ;
+	if (TFAILED (lispMgr_CreateTime    (pLispMgr, bufstat.st_atime, &rEntResult [4])))
+		goto	exit_func ;
+	lispEntity_AddRef     (pLispMgr, rEntResult [4]) ;
+	if (TFAILED (lispMgr_CreateTime    (pLispMgr, bufstat.st_mtime, &rEntResult [5])))
+		goto	exit_func ;
+	lispEntity_AddRef     (pLispMgr, rEntResult [5]) ;
+	if (TFAILED (lispMgr_CreateTime    (pLispMgr, bufstat.st_ctime, &rEntResult [6])))
+		goto	exit_func ;
+	lispEntity_AddRef     (pLispMgr, rEntResult [6]) ;
+
+	/*	ե륵Ǥʬˤ float ˤ롣*/
+	if (((long) bufstat.st_size) != bufstat.st_size) {
+		if (TFAILED (lispMgr_CreateFloat   (pLispMgr, (float) bufstat.st_size,  &rEntResult [7])))
+			goto	exit_func ;
+	} else {
+		if (TFAILED (lispMgr_CreateInteger (pLispMgr, bufstat.st_size,  &rEntResult [7])))
+			goto	exit_func ;
+	}
+	lispEntity_AddRef     (pLispMgr, rEntResult [7]) ;
+
+	uMask	= 1 ;
+	for (i = 0 ; i < 9 ; i ++) {
+		if ((bufstat.st_mode & uMask) != 0){
+			rbufMode [9 - i]	= "xwr" [i % 3] ;
+		} else {
+			rbufMode [9 - i]	= '-' ;
+		}
+		uMask	<<= 1 ;
+	}
+	if ((bufstat.st_mode & S_ISVTX) != 0) {
+		rbufMode [0]	= 't' ;
+	} else {
+		rbufMode [0]	= '-' ;
+	}
+	if ((bufstat.st_mode & S_ISUID) != 0) 
+		rbufMode [9 - 6]	= (bufstat.st_mode & S_IXUSR)? 's' : 'S' ;
+	if ((bufstat.st_mode & S_ISGID) != 0)
+		rbufMode [9 - 3]	= (bufstat.st_mode & S_IXGRP)? 's' : 'S' ;
+
+	if (TFAILED (lispMgr_CreateString (pLispMgr, rbufMode, 10, &rEntResult [8])))
+		goto	exit_func ;
+	lispEntity_AddRef     (pLispMgr, rEntResult [8]) ;
+
+	if (bufstat.st_gid != getegid ()) {
+		lispMgr_CreateT   (pLispMgr, &rEntResult [9]) ;
+	} else {
+		lispMgr_CreateNil (pLispMgr, &rEntResult [9]) ;
+	}
+	lispEntity_AddRef     (pLispMgr, rEntResult [9]) ;
+
+	if (TFAILED (lispMgr_CreateInteger (pLispMgr, bufstat.st_ino,   &rEntResult [10])))
+		goto	exit_func ;
+	lispEntity_AddRef     (pLispMgr, rEntResult [10]) ;
+	if (TFAILED (lispMgr_CreateInteger (pLispMgr, bufstat.st_dev,   &rEntResult [11])))
+		goto	exit_func ;
+	lispEntity_AddRef     (pLispMgr, rEntResult [11]) ;
+
+	if (TSUCCEEDED (lispMgr_CreateList (pLispMgr, rEntResult, NELEMENTS (rEntResult), &pEntRetval))) {
+		lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+		nRetval	= LMR_RETURN ;
+	}
+  exit_func:
+	for (i = 0 ; i < NELEMENTS (rEntResult) ; i ++) {
+		if (rEntResult [i] != NULL) {
+			lispEntity_Release (pLispMgr, rEntResult [i]) ;
+		} else {
+			break ;
+		}
+	}
+	return	nRetval ;
+}
+
 Boolean
 lispMachine_getFileNameFromEntity (
 	register TLispManager*	pLispMgr,
