@@ -36,7 +36,7 @@ static	TLMRESULT	lispMachineState_callIntLambda2 (TLispMachine*) ;
 
 static	Boolean		lispMachine_getIntArg (TLispManager*, TLispEntity*, TLispEntity**) ;
 static	Boolean		lispMachine_addEntityToArglist (TLispMachine*, TLispEntity**, TLispEntity*) ;
-static	Boolean		lispMachine_commandp (TLispMachine*, TLispEntity*) ;
+static	Boolean		lispMachine_commandp (TLispMachine*, TLispEntity*, TLispEntity**) ;
 
 /*
  *	Interactiveness ι eval, funcall, apply, call-interactively
@@ -164,11 +164,8 @@ lispMachineState_Commandp (
 		lispMachineCode_SetError (pLM) ;
 		return	LMR_RETURN ;
 	}
-	if (TSUCCEEDED (lispMachine_commandp (pLM, pEntFunc))) {
-		lispMgr_CreateT   (pLispMgr, &pEntRetval) ;
-	} else {
-		lispMgr_CreateNil (pLispMgr, &pEntRetval) ;
-	}
+	if (TFAILED (lispMachine_commandp (pLM, pEntFunc, &pEntRetval)))
+		return	LMR_ERROR ;
 	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
 	return	LMR_RETURN ;
 }
@@ -303,7 +300,7 @@ lispMachineState_CallInteractively (
 #endif
 	if (TSUCCEEDED (lispEntity_Symbolp (pLispMgr, pEntValue)) &&
 		TFAILED (lispMachine_GetFinalSymbolFunctionValue (pLM, pEntValue, &pEntValue))) {
-#if defined (DEBUG) 
+#if defined (DEBUG) || 0
 		fprintf (stderr, "symbol's function: ") ;
 		lispEntity_Print (pLispMgr, pFUNCTION) ;
 		fprintf (stderr, "is void.\n") ;
@@ -318,10 +315,10 @@ lispMachineState_CallInteractively (
 		 *	ոȡinteractive Ƭϡɾפ롣ա*/ 
 		if (TFAILED (lispMachine_getIntArg (pLispMgr, pEntValue, &pIntArg))) {
 			/*	ޥɰʳϼ¹ԤǤʤ*/
-#if defined (DEBUG) 
+#if defined (DEBUG)  || 0
 			fprintf (stderr, "symbol's function: ") ;
 			lispEntity_Print (pLispMgr, pFUNCTION) ;
-			fprintf (stderr, "is not interactive.\n") ;
+			fprintf (stderr, " is not interactive.\n") ;
 #endif
 			lispMachineCode_SetError (pLM) ;
 			return	LMR_RETURN ;
@@ -331,7 +328,7 @@ lispMachineState_CallInteractively (
 			TFAILED (lispSubr_GetProc (pLispMgr, pEntSubr, &pProcInfo))) {
 			goto	error ;
 		}
-#if defined (DEBUG) 
+#if defined (DEBUG) || 0
 		fprintf (stderr, "symbol-function = ") ;
 		lispEntity_Print (pLispMgr, pEntValue) ;
 		fprintf (stderr, "\n") ;
@@ -351,7 +348,7 @@ lispMachineState_CallInteractively (
 			return	LMR_CONTINUE ;
 		}
 	}
-#if defined (DEBUG) 
+#if defined (DEBUG)
 	fprintf (stderr, "(call-interactive ") ;
 	lispEntity_Print (pLispMgr, pFUNCTION) ;
 	fprintf (stderr, ")\n") ;
@@ -818,8 +815,8 @@ lispMachine_getIntArg (
 	pEntIntact	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_INTERACTIVE) ;
 	while (TFAILED (lispEntity_Nullp (pLispMgr, pEntity))) {
 		if (TSUCCEEDED (lispEntity_GetCar (pLispMgr, pEntity, &pEntCar)) &&
-			TSUCCEEDED (lispEntity_GetCar (pLispMgr, pEntCar,    &pEntCaar)) &&
-			pEntCaar == pEntIntact) {
+			TSUCCEEDED (lispEntity_GetCar (pLispMgr, pEntCar, &pEntCaar)) &&
+			TSUCCEEDED (lispEntity_Eq (pLispMgr, pEntCaar, pEntIntact))) {
 			return	lispEntity_GetCadr (pLispMgr, pEntCar, ppEntReturn) ;
 		}
 		if (TFAILED (lispEntity_GetCdr (pLispMgr, pEntity, &pEntNext)))
@@ -877,26 +874,63 @@ lispMachine_addEntityToArglist (
 Boolean
 lispMachine_commandp (
 	register TLispMachine*	pLM,
-	register TLispEntity*	pEntSymbol)
+	register TLispEntity*	pEntARG,
+	register TLispEntity**	ppEntRetval)
 {
 	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	register TLispEntity*	pEntInt ;
 	LMCMDINFO const*		pCmdInfo ;
 	TLispEntity*			pEntValue ;
-	Boolean					fRetval		= False ;
+	TLispEntity*			arList [2] ;
+	int						nType ;
 
-	if (TSUCCEEDED (lispSubr_GetProc (pLispMgr, pEntSymbol, &pCmdInfo))) {
-		return	(pCmdInfo->m_pInteractive != NULL) ;
-	}
-	if (TSUCCEEDED (lispMachine_GetSymbolFunctionValue (pLM, pEntSymbol, &pEntValue))) {
-		if (TSUCCEEDED (lispSubr_GetProc (pLispMgr, pEntValue, &pCmdInfo))) {
-			fRetval	= (pCmdInfo->m_pInteractive != NULL) ;
-		} else {
-			TLispEntity*	pIntArg ;
-			fRetval	= lispMachine_getIntArg (pLispMgr, pEntValue, &pIntArg) ;
+	pEntInt		= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_INTERACTIVE) ;
+	arList [0]	= pEntInt ;
+
+	if (TFAILED (lispEntity_GetType (pLispMgr, pEntARG, &nType)))
+		return	False ;
+
+	switch (nType) {
+	case	LISPENTITY_SYMBOL:
+		if (TFAILED (lispMachine_GetSymbolFunctionValue (pLM, pEntARG, &pEntValue))) {
+			return	lispMgr_CreateNil (pLispMgr, ppEntRetval) ;
 		}
-	} else {
-		fRetval	= False ;
+		if (TSUCCEEDED (lispSubr_GetProc (pLispMgr, pEntValue, &pCmdInfo))) 
+			goto	entity_is_subr ;
+		if (TFAILED (lispMachine_getIntArg (pLispMgr, pEntValue, &arList [1])))
+			return	lispMgr_CreateNil (pLispMgr, ppEntRetval) ;
+		return	lispMgr_CreateList (pLispMgr, arList, 2, ppEntRetval) ;
+
+	case	LISPENTITY_CONSCELL:
+	{
+		TLispEntity*	pEntCar ;
+
+		if (TSUCCEEDED (lispEntity_GetCar  (pLispMgr, pEntARG, &pEntCar)) &&
+			TSUCCEEDED (lispEntity_Lambdap (pLispMgr, pEntCar))) 
+			if (TSUCCEEDED (lispMachine_getIntArg (pLispMgr, pEntARG, &arList [1])))
+				return	lispMgr_CreateList (pLispMgr, arList, 2, ppEntRetval) ;
+		break ;
 	}
-	return	fRetval ;
+	case	LISPENTITY_SUBR:
+	  entity_is_subr:
+	{
+		register const Char*	pString ;
+		register int			nString ;
+
+		if (TFAILED (lispSubr_GetProc (pLispMgr, pEntARG, &pCmdInfo))) 
+			break ;
+		if (pCmdInfo->m_pInteractive == NULL) 
+			return	lispMgr_CreateNil (pLispMgr, ppEntRetval) ;
+		pString	= pCmdInfo->m_pInteractive ;
+		nString	= Cstrlen (pString) ;
+		if (TFAILED (lispMgr_CreateString (pLispMgr, pString, nString, &arList [1])))
+			return	False ;
+		arList [0]	= pEntInt ;
+		return	lispMgr_CreateList (pLispMgr, arList, 2, ppEntRetval) ;
+	}
+	default:
+		break ;
+	}
+	return	lispMgr_CreateNil (pLispMgr, ppEntRetval) ;
 }
 
