@@ -22,8 +22,30 @@
 #include <stdio.h>
 #include <assert.h>
 #include "lmachinep.h"
+#include "lmkeymap.h"
 
-static	int	lispMachineState_keyBindingLookupKey (TLispMachine*, TLispEntity*, TLispEntity*, TLispEntity*, int, Boolean, Boolean, TLispEntity**) ;
+typedef struct {
+	TLispEntity*	m_pEntDefinition ;
+	TVarbuffer*		m_pvbufKeySeq ;
+	TLispConscell*	m_plstResult ;
+	Boolean			m_fOnce ;
+}	TLispWhereIsInternalArg ;
+
+typedef struct {
+	TLispEntity*	m_pEntKey ;
+	int				m_nLength ;
+	Boolean			m_fNearMatch ;
+	Boolean			m_fAcceptDefault ;
+	int				m_nMatch ;
+	TLispEntity*	m_pEntFunc ;
+}	TLispKeyBindingArg ;
+
+static	Boolean	lispMachine_keybindingLookupKey	(TLispMachine*, TLispEntity*, TLispKeyBindingArg*) ;
+static	Boolean	lispMachine_keybindingEnum		(TLispMachine*, TLispEntity*, void*, Boolean*) ;
+static	Boolean	lispMachine_whereIsInternal1keymap(TLispMachine*, TLispEntity*, TLispEntity*, TVarbuffer*, TLispConscell*) ;
+static	Boolean	lispMachine_whereIsInternalVector(TLispMachine*, TLispEntity*, TLispEntity*, TVarbuffer*, TLispConscell*) ;
+static	Boolean	lispMachine_whereIsInternalCons	(TLispMachine*, TLispEntity*, TLispEntity*, TVarbuffer*, TLispConscell*) ;
+static	Boolean	lispMachine_whereIsInternalEnum	(TLispMachine*, TLispEntity*, void*, Boolean*) ;
 
 /*
  *	(keymap CHARTABLE . ALIST) Ȥοޥåפä֤
@@ -286,7 +308,8 @@ lispMachineState_DefineKey (
  *	(keymap1 keymap2 ...) Ȥ list 롣
  */
 TLMRESULT
-lispMachineState_CurrentMinorModeMaps (register TLispMachine* pLM)
+lispMachineState_CurrentMinorModeMaps (
+	register TLispMachine* pLM)
 {
 	register TLispManager*	pLispMgr ;
 	TLispEntity*	pEntKeymaps ;
@@ -337,17 +360,16 @@ lispMachineState_UseLocalMap (register TLispMachine* pLM)
  *	nil ֤
  */
 TLMRESULT
-lispMachineState_CurrentLocalMap (register TLispMachine* pLM)
+lispMachineState_CurrentLocalMap (
+	register TLispMachine* pLM)
 {
 	register TLispManager*	pLispMgr ;
-	TLispEntity*	pEntArglist ;
 	TLispEntity*	pEntCurBuffer ;
 	TLispEntity*	pEntKeymap ;
 
 	assert (pLM != NULL) ;
 	pLispMgr	= pLM->m_pLispMgr ;
 	assert (pLispMgr != NULL) ;
-	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
 	if (TFAILED (lispMachineCode_GetCurrentBuffer (pLM, &pEntCurBuffer))) {
 		lispMachineCode_SetError (pLM) ;
 	} else {
@@ -384,14 +406,100 @@ lispMachineState_CurrentLocalMap (register TLispMachine* pLM)
  *	current-minor-mode-maps, current-local-map, global-map
  */
 TLMRESULT
-lispMachineState_WhereIsInternal (register TLispMachine* pLM)
+lispMachineState_WhereIsInternal (
+	register TLispMachine* pLM)
 {
 	register TLispManager*	pLispMgr ;
-	TLispEntity*	pEntBuffer ;
-	TLispEntity*	pEntKeymap ;
+	TLispEntity*			pEntArglist ;
+	TLispEntity*			pEntDefinition ;
+	TLispEntity*			pEntKeymap ;
+	register TLispEntity*	pEntGlobalMap ;
+	TLispConscell			lstResult ;
+	TVarbuffer				vbufKeySeq ;
 
-	/*	not-implemented */
-	lispMachineCode_GetCurrentBuffer (pLM, &pEntBuffer) ;
+	assert (pLM != NULL) ;
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr != NULL) ;
+
+	/*	ΤȤ34ưʤ*/
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	if (TFAILED (lispEntity_GetCar (pLispMgr, pEntArglist, &pEntDefinition)) ||
+		TFAILED (lispEntity_GetCdr (pLispMgr, pEntArglist, &pEntArglist))    ||
+		TFAILED (lispEntity_GetCar (pLispMgr, pEntArglist, &pEntKeymap))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	if (TFAILED (TVarbuffer_Initialize (&vbufKeySeq, sizeof (TLispEntity*))))
+		return	LMR_ERROR ;
+
+	lstResult.m_pCar	= lstResult.m_pCdr	= NULL ;
+	if (TFAILED (lispEntity_Nullp (pLispMgr, pEntKeymap))) {
+		if (TSUCCEEDED (lispEntity_Keymapp (pLispMgr, pEntKeymap))) {
+			if (TFAILED (lispMachine_whereIsInternal1keymap (pLM, pEntKeymap, pEntDefinition, &vbufKeySeq, &lstResult)))
+				goto	error ;
+			TVarbuffer_Clear (&vbufKeySeq) ;
+		} else {
+			TLispEntity*	pEntTarget ;
+			TLispEntity*	pEntNext ;
+
+			if (TFAILED (lispEntity_Consp (pLispMgr, pEntKeymap)))
+				goto	error ;
+
+			while (TFAILED (lispEntity_Nullp (pLispMgr, pEntKeymap))) {
+				if (TFAILED (lispEntity_GetCar (pLispMgr, pEntKeymap, &pEntTarget)) ||
+					TFAILED (lispEntity_GetCdr (pLispMgr, pEntKeymap, &pEntNext)))
+					goto	error ;
+				if (TFAILED (lispMachine_whereIsInternal1keymap (pLM, pEntTarget, pEntDefinition, &vbufKeySeq, &lstResult)))
+					goto	error ;
+				TVarbuffer_Clear (&vbufKeySeq) ;
+				pEntKeymap	= pEntNext ;
+			}
+		}
+	} else {
+		TLispWhereIsInternalArg	arg ;
+		TLispEntity*			pEntCurBuffer ;
+
+		/*	 active  keymap Ƹ... sigh
+		 *	Ĥޤꡢ
+		 *		1. minor-mode-map-alist 
+		 *		2. local-map 
+		 *		3. global-map 
+		 *	뤳Ȥˤʤ롣
+		 */
+		arg.m_pEntDefinition	= pEntDefinition ;
+		arg.m_pvbufKeySeq		= &vbufKeySeq ;
+		arg.m_plstResult		= &lstResult ;
+		lispMachine_EnumMinorModeMaps (pLM, lispMachine_whereIsInternalEnum, &arg) ;
+
+		if (TFAILED (lispMachineCode_GetCurrentBuffer (pLM, &pEntCurBuffer))) 
+			goto	error ;
+		if (TSUCCEEDED (lispBuffer_GetKeymap (pEntCurBuffer, &pEntKeymap)) ||
+			pEntKeymap != NULL) {
+			if (TFAILED (lispMachine_whereIsInternal1keymap (pLM, pEntKeymap, pEntDefinition, &vbufKeySeq, &lstResult)))
+				goto	error ;
+			TVarbuffer_Clear (&vbufKeySeq) ;
+		}
+	}
+
+	/*	 global-map 򸡺롣
+	 */
+	pEntGlobalMap	= lispMachine_GetGlobalMap (pLM) ;
+	if (TFAILED (lispMachine_whereIsInternal1keymap (pLM, pEntGlobalMap, pEntDefinition, &vbufKeySeq, &lstResult)))
+		goto	error ;
+
+	if (lstResult.m_pCar == NULL) {
+		lispMachineCode_SetLReg (pLM, LM_LREG_ACC, lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_NIL)) ;
+	} else {
+		lispMachineCode_SetLReg (pLM, LM_LREG_ACC, lstResult.m_pCar) ;
+		lispEntity_Release (pLispMgr, lstResult.m_pCar) ;
+	}
+	return	LMR_RETURN ;
+
+  error:
+	TVarbuffer_Uninitialize (&vbufKeySeq) ;
+	if (lstResult.m_pCar != NULL)
+		lispEntity_Release (pLispMgr, lstResult.m_pCar) ;
+	lispMachineCode_SetError (pLM) ;
 	return	LMR_RETURN ;
 }
 
@@ -410,16 +518,15 @@ lispMachineState_KeyBinding (
 	register TLispMachine*	pLM)
 {
 	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
-	TLispEntity*	pEntArglist ;
-	TLispEntity*	pEntKey ;
-	TLispEntity*	pEntAcceptDefault ;
-	TLispEntity*	pEntKeymap ;
-	TLispEntity*	pEntFunc ;
-	TLispEntity*	pEntSymGlobalMap ;
-	TLispEntity*	pEntGlobalKeymap ;
-	TLispEntity*	pEntCurBuffer ;
-	Boolean			fAcceptDefault ;
-	int				nLength ;
+	TLispEntity*		pEntArglist ;
+	TLispEntity*		pEntKey ;
+	TLispEntity*		pEntAcceptDefault ;
+	TLispEntity*		pEntKeymap ;
+	TLispEntity*		pEntFunc ;
+	TLispKeyBindingArg	arg ;
+	TLispEntity*		pEntCurBuffer ;
+	Boolean				fAcceptDefault ;
+	int					nLength ;
 
 	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
 	lispEntity_GetCar  (pLispMgr, pEntArglist, &pEntKey) ;
@@ -431,15 +538,36 @@ lispMachineState_KeyBinding (
 		return	LMR_RETURN ;
 	}
 
+	/*		1. minor-mode-map-alist 򸡺롣*/
+	arg.m_pEntKey			= pEntKey ;
+	arg.m_nLength			= nLength ;
+	arg.m_fAcceptDefault	= fAcceptDefault ;
+	arg.m_fNearMatch		= False ;
+	arg.m_pEntFunc			= NULL ;
+	lispMachine_EnumMinorModeMaps (pLM, lispMachine_keybindingEnum, &arg) ;
+	if (arg.m_nMatch > 0 && arg.m_pEntFunc != NULL) {
+		pEntFunc	= arg.m_pEntFunc ;
+		goto	found ;
+	}
+	/*		2. local-map 򸡺롣*/
 	lispMachineCode_GetCurrentBuffer (pLM, &pEntCurBuffer) ;
 	assert (pEntCurBuffer != NULL) ;
 	(void) lispBuffer_GetKeymap (pEntCurBuffer, &pEntKeymap) ;
-	
-	pEntSymGlobalMap	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_GLOBAL_MAP) ;
-	if (TFAILED (lispMachine_GetCurrentSymbolValue (pLM, pEntSymGlobalMap, &pEntGlobalKeymap)) ||
-		TSUCCEEDED (lispEntity_Voidp (pLispMgr, pEntGlobalKeymap))) 
-		pEntGlobalKeymap	= NULL ;
-	(void) lispMachineState_keyBindingLookupKey (pLM, pEntKeymap, pEntGlobalKeymap, pEntKey, nLength, False, fAcceptDefault, &pEntFunc) ;
+	(void) lispMachine_keybindingLookupKey (pLM, pEntKeymap, &arg) ;
+	if (arg.m_nMatch > 0 && arg.m_pEntFunc != NULL) {
+		pEntFunc	= arg.m_pEntFunc ;
+		goto	found ;
+	}
+
+	/*		3. global-map 򸡺롣*/
+	pEntKeymap	= lispMachine_GetGlobalMap (pLM) ;
+	(void) lispMachine_keybindingLookupKey (pLM, pEntKeymap, &arg) ;
+	if (arg.m_nMatch > 0 && arg.m_pEntFunc != NULL) {
+		pEntFunc	= arg.m_pEntFunc ;
+	} else {
+		pEntFunc	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_NIL) ;
+	}
+  found:
 	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntFunc) ;
 	return	LMR_RETURN ;
 }
@@ -464,17 +592,18 @@ lispMachineState_JKeyBinding (
 	register TLispMachine*	pLM)
 {
 	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
-	TLispEntity*	pEntArglist ;
-	TLispEntity*	pEntKey ;
-	TLispEntity*	pEntAcceptDefault ;
-	TLispEntity*	pEntKeymap ;
-	TLispEntity*	pEntFunc ;
-	TLispEntity*	pEntSymGlobalMap ;
-	TLispEntity*	pEntGlobalKeymap ;
-	TLispEntity*	pEntCurBuffer ;
-	Boolean			fAcceptDefault ;
-	int				nLength ;
-	register int	nMatch ;
+	TLispEntity*		pEntArglist ;
+	TLispEntity*		pEntKey ;
+	TLispEntity*		pEntAcceptDefault ;
+	TLispEntity*		pEntKeymap ;
+	TLispEntity*		pEntGlobalKeymap ;
+	TLispEntity*		pEntFunc ;
+	TLispKeyBindingArg	arg ;
+	TLispEntity*		pEntCurBuffer ;
+	Boolean				fAcceptDefault ;
+	int					nLength ;
+	register int		i ;
+	static Boolean		afNearMatch []	= { False, True, } ;
 
 	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
 	lispEntity_GetCar  (pLispMgr, pEntArglist, &pEntKey) ;
@@ -489,85 +618,266 @@ lispMachineState_JKeyBinding (
 	lispMachineCode_GetCurrentBuffer (pLM, &pEntCurBuffer) ;
 	assert (pEntCurBuffer != NULL) ;
 	(void) lispBuffer_GetKeymap (pEntCurBuffer, &pEntKeymap) ;
-	
-	pEntSymGlobalMap	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_GLOBAL_MAP) ;
-	if (TFAILED (lispMachine_GetCurrentSymbolValue (pLM, pEntSymGlobalMap, &pEntGlobalKeymap)) ||
-		TSUCCEEDED (lispEntity_Voidp (pLispMgr, pEntGlobalKeymap))) 
-		pEntGlobalKeymap	= NULL ;
-#if defined (DEBUG)
-	fprintf (stderr, "key(%d, %d) = ", nLength, fAcceptDefault) ;
-	lispEntity_Print (pLispMgr, pEntKey) ;
-	fprintf (stderr, "\n") ;
-#endif
-	nMatch	= lispMachineState_keyBindingLookupKey (pLM, pEntKeymap, pEntGlobalKeymap, pEntKey, nLength, False, fAcceptDefault, &pEntFunc) ;
-#if defined (DEBUG)
-	fprintf (stderr, "[1] key-bind(%d) = ", nMatch) ;
-	lispEntity_Print (pLispMgr, pEntFunc) ;
-	fprintf (stderr, "\n") ;
-#endif
-	if (nMatch <= 0 || TSUCCEEDED (lispEntity_Nullp (pLispMgr, pEntFunc))) {
-		nMatch	= lispMachineState_keyBindingLookupKey (pLM, pEntKeymap, pEntGlobalKeymap, pEntKey, nLength, True, fAcceptDefault, &pEntFunc) ;
-#if defined (DEBUG)
-		fprintf (stderr, "[2] key-bind(%d) = ", nMatch) ;
-		lispEntity_Print (pLispMgr, pEntFunc) ;
-		fprintf (stderr, "\n") ;
-#endif
+	pEntGlobalKeymap	= lispMachine_GetGlobalMap (pLM) ;
+
+	arg.m_pEntKey			= pEntKey ;
+	arg.m_nLength			= nLength ;
+	arg.m_fAcceptDefault	= fAcceptDefault ;
+	arg.m_pEntFunc			= NULL ;
+
+	for (i = 0 ; i < NELEMENTS (afNearMatch) ; i ++) {
+		/*		1. minor-mode-map-alist 򸡺롣*/
+		arg.m_fNearMatch		= afNearMatch [i] ;
+		lispMachine_EnumMinorModeMaps (pLM, lispMachine_keybindingEnum, &arg) ;
+		if (arg.m_nMatch > 0 && arg.m_pEntFunc != NULL) {
+			pEntFunc	= arg.m_pEntFunc ;
+			goto	found ;
+		}
+		/*		2. local-map 򸡺롣*/
+		(void) lispMachine_keybindingLookupKey (pLM, pEntKeymap, &arg) ;
+		if (arg.m_nMatch > 0 && arg.m_pEntFunc != NULL) {
+			pEntFunc	= arg.m_pEntFunc ;
+			goto	found ;
+		}
+		/*		3. global-map 򸡺롣*/
+		(void) lispMachine_keybindingLookupKey (pLM, pEntGlobalKeymap, &arg) ;
+		if (arg.m_nMatch > 0 && arg.m_pEntFunc != NULL) {
+			pEntFunc	= arg.m_pEntFunc ;
+			goto	found ;
+		}
 	}
+	pEntFunc	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_NIL) ;
+  found:
 	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntFunc) ;
 	return	LMR_RETURN ;
 }
 
-int
-lispMachineState_keyBindingLookupKey (
-	register TLispMachine*	pLM,
-	register TLispEntity*	pEntLocalKeymap,
-	register TLispEntity*	pEntGlobalKeymap,
-	register TLispEntity*	pEntKey,
-	register int			nLength,
-	register Boolean		fNearMatch,
-	register Boolean		fAcceptDefault,
-	register TLispEntity**	ppEntRetval)
+/*========================================================================*
+ *	private functions
+ */
+Boolean
+lispMachine_keybindingLookupKey (
+	register TLispMachine*			pLM,
+	register TLispEntity*			pEntKeymap,
+	register TLispKeyBindingArg*	pArg)
 {
 	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
-	TLispEntity*	pEntFunc ;
-	int				nMatch ;
+	register TLispEntity*	pEntKey ;
+	register int			nLength ;
+	register Boolean		fNearMatch ;
+	register Boolean		fAcceptDefault ;
+	int						nMatch ;
+	TLispEntity*			pEntFunc ;
 
-	nMatch		= 0 ;
-	pEntFunc	= NULL ;
-	if (pEntLocalKeymap != NULL &&
-		TSUCCEEDED (lispKeymap_Lookup (pLispMgr, pEntLocalKeymap, pEntKey, fAcceptDefault, fNearMatch, &nMatch, &pEntFunc)) &&
+	pEntKey			= pArg->m_pEntKey ;
+	nLength			= pArg->m_nLength ;
+	fNearMatch		= pArg->m_fNearMatch ;
+	fAcceptDefault	= pArg->m_fAcceptDefault ;
+	nMatch			= 0 ;
+	pEntFunc		= NULL ;
+	if (TSUCCEEDED (lispKeymap_Lookup (pLispMgr, pEntKeymap, pEntKey, fAcceptDefault, fNearMatch, &nMatch, &pEntFunc)) &&
 		TFAILED (lispEntity_Nullp (pLispMgr, pEntFunc)) &&
 		nMatch > 0 && (nMatch == nLength || fAcceptDefault)) {
-#if defined (DEBUG)
-		fprintf (stderr, "[a] pEntFunc(%p), nMatch(%d), fAcceptDefault(%d), nLength(%d)\n",
-				 pEntFunc, nMatch, fAcceptDefault, nLength) ;
-#endif
-		goto	keyhit ;
-	}
-
-	nMatch	= 0 ;
-	if (pEntGlobalKeymap != NULL &&
-		TFAILED (lispEntity_Voidp (pLispMgr, pEntGlobalKeymap))) {
-		/*	ХåեˤޤäƤϤѤƥޥåפ*/
-		lispKeymap_Lookup (pLispMgr, pEntGlobalKeymap, pEntKey, fAcceptDefault, fNearMatch, &nMatch, &pEntFunc) ;
-#if defined (DEBUG)
-		fprintf (stderr, "[b] pEntFunc(%p), nMatch(%d), fAcceptDefault(%d), nLength(%d)\n",
-				 pEntFunc, nMatch, fAcceptDefault, nLength) ;
-#endif
-		if (pEntFunc == NULL || nMatch <= 0 || (nMatch != nLength && !fAcceptDefault)) 
-			lispMgr_CreateNil (pLispMgr, &pEntFunc) ;
+		pArg->m_nMatch		= nMatch ;
+		pArg->m_pEntFunc	= pEntFunc ;
 	} else {
-#if defined (DEBUG)
-		fprintf (stderr, "pEntGlobalKeymap (%p)\n", pEntGlobalKeymap) ;
-#endif
-		/*	ˤҥåȤʤС undefined Ǥ롣*/
-		lispMgr_CreateNil (pLispMgr, &pEntFunc) ;
+		pArg->m_nMatch		= 0 ;
+		pArg->m_pEntFunc	= NULL ;
 	}
-
- keyhit:
-	if (ppEntRetval != NULL)
-		*ppEntRetval	= pEntFunc ;
-	return	nMatch ;
+	return	True ;
 }
 
+Boolean
+lispMachine_keybindingEnum (
+	register TLispMachine*	pLM,
+	register TLispEntity*	pEntKeymap,
+	register void*			pCaller,
+	register Boolean*		pfContinue)
+{
+	register TLispKeyBindingArg*	pArg	= (TLispKeyBindingArg *) pCaller ;
+
+	if (TFAILED (lispMachine_keybindingLookupKey (pLM, pEntKeymap, pArg)))
+		return	False ;
+	if (pArg->m_nMatch > 0 && pArg->m_pEntFunc != NULL) 
+		*pfContinue	= False ;
+	return	True ;
+}
+
+/*	1 keymap Ф where-is-internal ưԤ
+ */
+Boolean
+lispMachine_whereIsInternal1keymap (
+	register TLispMachine*	pLM,
+	register TLispEntity*	pEntKeymap,
+	register TLispEntity*	pEntSymbol,
+	register TVarbuffer*	pvbufKeySeq,
+	register TLispConscell*	pEntRetval)
+{
+	register TLispManager*	pLispMgr ;
+	TLispEntity*	pEntTarget ;
+	TLispEntity*	pEntNext ;
+	int				nType ;
+
+	assert (pLM != NULL) ;
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr != NULL) ;
+
+	if (pEntKeymap == NULL ||
+		TFAILED (lispEntity_Keymapp (pLispMgr, pEntKeymap)))	/* null ̵뤹롣*/
+		return	True ;
+
+	(void) lispEntity_GetCdr (pLispMgr, pEntKeymap, &pEntNext) ;
+	pEntKeymap	= pEntNext ;
+
+	while (TFAILED (lispEntity_Nullp (pLispMgr, pEntKeymap))) {
+		if (TFAILED (lispEntity_GetCar (pLispMgr, pEntKeymap, &pEntTarget)) ||
+			TFAILED (lispEntity_GetCdr (pLispMgr, pEntKeymap, &pEntNext)))
+			return	False ;
+		(void) lispEntity_GetType (pLispMgr, pEntTarget, &nType) ;
+		switch (nType) {
+		case	LISPENTITY_VECTOR:
+			if (TFAILED (lispMachine_whereIsInternalVector (pLM, pEntTarget, pEntSymbol, pvbufKeySeq, pEntRetval)))
+				return	False ;
+			break ;
+		case	LISPENTITY_CONSCELL:
+			if (TFAILED (lispMachine_whereIsInternalCons (pLM, pEntTarget, pEntSymbol, pvbufKeySeq, pEntRetval)))
+				return	False ;
+			break ;
+		default:
+			/*	¿ʬ˥顼ʡ*/
+			return	False ;
+		}
+		pEntKeymap	= pEntNext ;
+	}
+	return	True ;
+}
+
+Boolean
+lispMachine_whereIsInternalVector (
+	register TLispMachine*	pLM,
+	register TLispEntity*	pEntVector,
+	register TLispEntity*	pEntSymbol,
+	register TVarbuffer*	pvbufKeySeq,
+	register TLispConscell*	pEntRetval)
+{
+	register TLispManager*	pLispMgr ;
+	TLispEntity**			ppElements ;
+	int						nElements ;
+	register TLispEntity**	ptr ;
+	register int			i ;
+	TLispEntity*			pEntKey ;
+	TLispEntity*			pEntResult ;
+	register Boolean		f ;
+
+	assert (pLM != NULL) ;
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr != NULL) ;
+
+	if (TFAILED (lispEntity_GetVectorValue (pLispMgr, pEntVector, &ppElements, &nElements)))
+		return	False ;
+
+	for (i = 0, ptr = ppElements ; i < nElements ; i ++, ptr ++) {
+		if (TSUCCEEDED (lispEntity_Eq (pLispMgr, *ptr, pEntSymbol))) {
+			register TLispEntity**	pEntKeySeq ;
+			register int			nKeySeq ;
+
+			if (TFAILED (lispMgr_CreateInteger (pLispMgr, (long)i, &pEntKey)))
+				return	False ;
+			if (TFAILED (TVarbuffer_Add (pvbufKeySeq, &pEntKey, 1)))
+				return	False ;
+			pEntKeySeq	= (TLispEntity **)TVarbuffer_GetBuffer (pvbufKeySeq) ;
+			nKeySeq		= TVarbuffer_GetUsage (pvbufKeySeq) ;
+			lispEntity_AddRef (pLispMgr, pEntKey) ;
+			f	= lispMgr_CreateVector (pLispMgr, pEntKeySeq, nKeySeq, &pEntResult) ;
+			lispEntity_Release (pLispMgr, pEntKey) ;
+			if (TFAILED (f) || pEntResult == NULL)
+				return	False ;
+			lispEntity_AddRef (pLispMgr, pEntResult) ;
+			f	= lispEntity_Push2List (pLispMgr, pEntRetval, pEntResult) ;
+			lispEntity_Release (pLispMgr, pEntResult) ;
+			if (TFAILED (f))
+				return	False ;
+		}
+	}
+	return	True ;
+}
+
+Boolean
+lispMachine_whereIsInternalCons (
+	register TLispMachine*	pLM,
+	register TLispEntity*	pEntCons,
+	register TLispEntity*	pEntSymbol,
+	register TVarbuffer*	pvbufKeySeq,
+	register TLispConscell*	pEntRetval)
+{
+	register TLispManager*	pLispMgr ;
+	TLispEntity*			pEntKey ;
+	TLispEntity*			pEntFunc ;
+	TLispEntity*			pEntResult ;
+	register Boolean		f ;
+	
+	assert (pLM != NULL) ;
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr != NULL) ;
+
+	if (TFAILED (lispEntity_GetCar (pLispMgr, pEntCons, &pEntKey)) ||
+		TFAILED (lispEntity_GetCdr (pLispMgr, pEntCons, &pEntFunc))) 
+		return	False ;
+
+	if (TSUCCEEDED (lispEntity_Symbolp (pLispMgr, pEntFunc))) {
+		if (TSUCCEEDED (lispEntity_Eq (pLispMgr, pEntFunc, pEntSymbol))) {
+			register TLispEntity**	pEntKeySeq ;
+			register int			nKeySeq ;
+
+			if (TFAILED (TVarbuffer_Add (pvbufKeySeq, &pEntKey, 1)))
+				return	False ;
+
+			pEntKeySeq	= (TLispEntity **)TVarbuffer_GetBuffer (pvbufKeySeq) ;
+			nKeySeq		= TVarbuffer_GetUsage (pvbufKeySeq) ;
+			lispEntity_AddRef (pLispMgr, pEntKey) ;
+			f	= lispMgr_CreateVector (pLispMgr, pEntKeySeq, nKeySeq, &pEntResult) ;
+			lispEntity_Release (pLispMgr, pEntKey) ;
+			if (TFAILED (f) || pEntResult == NULL)
+				return	False ;
+
+			lispEntity_AddRef (pLispMgr, pEntResult) ;
+			f	= lispEntity_Push2List (pLispMgr, pEntRetval, pEntResult) ;
+			lispEntity_Release (pLispMgr, pEntResult) ;
+			if (TFAILED (f))
+				return	False ;
+		}
+		return	True ;
+	}
+	if (TFAILED (lispEntity_Keymapp (pLispMgr, pEntFunc)))
+		return	False ;
+
+	if (TFAILED (TVarbuffer_Add (pvbufKeySeq, &pEntKey, 1)))
+		return	False ;
+
+	f	= lispMachine_whereIsInternal1keymap (pLM, pEntFunc, pEntSymbol, pvbufKeySeq, pEntRetval) ;
+	(void) TVarbuffer_Sub (pvbufKeySeq, 1) ;
+	return	f ;
+}
+
+Boolean
+lispMachine_whereIsInternalEnum (
+	register TLispMachine*	pLM,
+	register TLispEntity*	pEntKeymap,
+	register void*			pCaller,
+	register Boolean*		pfContinue)
+{
+	register TLispWhereIsInternalArg*	pArg	= (TLispWhereIsInternalArg*) pCaller ;
+	register TLispEntity*	pEntDef ;
+	register TVarbuffer*	pvbuf ;
+	register TLispConscell*	pResult ;
+
+	pEntDef	= pArg->m_pEntDefinition ;
+	pvbuf	= pArg->m_pvbufKeySeq ;
+	pResult	= pArg->m_plstResult ;
+	if (TFAILED (lispMachine_whereIsInternal1keymap (pLM, pEntKeymap, pEntDef, pvbuf, pResult)))
+		return	False ;
+
+	TVarbuffer_Clear (pvbuf) ;
+	return	True ;
+}
 
