@@ -274,7 +274,7 @@ lispMachineState_evalLambdaBindArg (
 
 		if (TFAILED (lispEntity_GetCdr (pLispMgr, pSymbollist, &pNextSymbol)) ||
 			TFAILED (lispEntity_GetCdr (pLispMgr, pValueList,  &pNextValue))) {
-			return	FALSE ;
+			goto	error_occur ;
 		}
 		pSymbollist	= pNextSymbol ;
 		pValueList	= pNextValue ;
@@ -354,8 +354,11 @@ lispMachineState_evalLambdaFinalize (TLispMachine* pLM)
 	lispMachineCode_GetLReg (pLM, LM_LREG_5, &pValuelist) ;
 
 	while (TFAILED (lispEntity_Nullp (pLispMgr, pSymbollist))) {
-		(void) lispEntity_GetCar (pLispMgr, pSymbollist, &pSymbol) ;
-		(void) lispEntity_GetCar (pLispMgr, pValuelist,  &pValue) ;
+		if (TFAILED (lispEntity_GetCar (pLispMgr, pSymbollist, &pSymbol)) ||
+			TFAILED (lispEntity_GetCar (pLispMgr, pValuelist,  &pValue))) {
+			lispMachineCode_SetError (pLM) ;
+			break ;
+		}
 #if defined (DEBUG_LV99)
 		fprintf (stderr, "lambda-finalize: ") ;
 		lispEntity_Print (pLispMgr, pSymbol) ;
@@ -364,8 +367,11 @@ lispMachineState_evalLambdaFinalize (TLispMachine* pLM)
 		fprintf (stderr, "\n") ;
 #endif
 		lispMachine_SetCurrentSymbolValue (pLM, pSymbol, pValue) ;
-		(void) lispEntity_GetCdr (pLispMgr, pSymbollist, &pSymbollist) ;
-		(void) lispEntity_GetCdr (pLispMgr, pValuelist,  &pValuelist) ;
+		if (TFAILED (lispEntity_GetCdr (pLispMgr, pSymbollist, &pSymbollist)) ||
+			TFAILED (lispEntity_GetCdr (pLispMgr, pValuelist,  &pValuelist))) {
+			lispMachineCode_SetError (pLM) ;
+			break ;
+		}
 	}
 
 	lispMachineCode_PopLReg (pLM, LM_LREG_7) ;
