@@ -101,72 +101,46 @@ lispMachineState_Defmacro (register TLispMachine* pLM)
  *	Сlisp machine ϶ĴưƤ뤱ɡ 
  *	symbol-value ï˲Ǥʡġ
  */
-#if 0
 TLMRESULT
 lispMachineState_Macroexpand (
 	register TLispMachine*	pLM)
 {
 	register TLispManager*	pLispMgr ;
-	register TLispEntity*	pEntSymValue ;
-	register TLispEntity*	pEntSymbol ;
-	TLispEntity*	pEntValTail ;
-	TLispEntity*	pEntSymTail ;
-	TLispEntity*	pEntForm ;
 	TLispEntity*	pEntArglist ;
-	TLispEntity*	pEntEnv ;
-	TLispEntity*	pEntPair ;
-	TLispEntity*	pEntOrgSymValue ;
-	TLispEntity*	pEntNil ;
-	TLispEntity*	pEntVoid ;
+	TLispEntity*	pEntMacroForm ;
+	TLispEntity*	pEntSym ;
+	TLispEntity*	pEntFunc ;
+	TLispEntity*	pEntMacro ;
 
-	assert (pLM != NULL) ;
 	pLispMgr	= pLM->m_pLispMgr ;
-	assert (pLispMgr != NULL) ;
 	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
-	if (TFAILED (lispEntity_GetCar (pLispMgr, pEntArglist, &pEntForm)) ||
-		TFAILED (lispEntity_GetCdr (pLispMgr, pEntArglist, &pEntArglist)) ||
-		TFAILED (lispEntity_GetCar (pLispMgr, pEntArglist, &pEntEnv))) 
-		goto	error ;
-
-	lispMgr_CreateNil  (pLispMgr, &pEntNil) ;
-	lispMgr_CreateVoid (pLispMgr, &pEntVoid) ;
-	lispMachineCode_PushLReg (pLM, LM_LREG_1) ;
-	lispMachineCode_PushLReg (pLM, LM_LREG_2) ;
-	pEntValTail	= pEntNil ;
-	pEntSymTail	= pEntNil ;
-	while (TFAILED (lispEntity_Nullp (pLispMgr, pEntEnv))) {
-		if (TFAILED (lispEntity_GetCar (pLispMgr, pEntEnv, &pEntPair)))
-			goto	error ;
-		if (TFAILED (lispEntity_GetCar (pLispMgr, pEntPair,  &pEntSymbol))) {
-			pEntSymbol		= pEntPair ;
-			pEntSymValue	= pEntVoid ;
-		} else {
-			lispEntity_SetCar (pLispMgr, pEntPair, pEntMacro) ;
-			pEntSymValue	= pEntPair ;
-		}
-		if (TFAILED (lispEntity_Symbolp (pLispMgr, pEntSymbol))) 
-			goto	error ;
-		lispMachine_GetSymbolFunctionValue (pLM, pEntSymbol, &pEntOrgSymValue) ;
-		lispMachine_SetSymbolFunctionValue (pLM, pEntSymbol, pEntSymValue) ;
-
-		lispMgr_CreateConscell (pLispMgr, pEntOrgSymValue, pEntNil, &pEntNewTail) ;
-		if (TFAILED (lispEntity_SetCdr (pLispMgr, pEntValTail, pEntNewTail))) {
-			lispMachineCode_SetLReg (pLM, LM_LREG_1, pEntNewTail) ;
-			pEntValTail	= pEntNewTail ;
-		}
-		lispMgr_CreateConscell (pLispMgr, pEntSymbol, pEntNil, &pEntNewTail) ;
-		if (TFAILED (lispEntity_SetCdr (pLispMgr, pEntSymTail, pEntNewTail))) {
-			lispMachineCode_SetLReg (pLM, LM_LREG_2, pEntNewTail) ;
-			pEntSymTail	= pEntNewTail ;
-		}
-		if (TFAILED (lispEntity_GetCdr (pLispMgr, pEntEnv, &pEntEnv)))
-			goto	error ;
+	/*	pEntMacroForm  ((macro lambda ...) ARG...) ηˤʤäƤ
+	 *	롣ꤷƤ롣*/
+	if (TFAILED (lispEntity_GetCar (pLispMgr, pEntArglist, &pEntMacroForm))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
 	}
-  error:
-	lispMachineCode_SetError (pLM) ;
-	return	LMR_RETURN ;
-}
+	if (TFAILED (lispEntity_GetCar (pLispMgr, pEntMacroForm, &pEntSym)) ||
+		TFAILED (lispMachine_GetSymbolFunctionValue (pLM, pEntSym, &pEntFunc)) ||
+		TFAILED (lispEntity_GetCar (pLispMgr, pEntFunc, &pEntMacro)) ||
+		TFAILED (lispEntity_Macrop (pLispMgr, pEntMacro))) {
+		return	LMR_RETURN ;
+	}
+	if (TFAILED (lispEntity_GetCdr (pLispMgr, pEntFunc, &pEntFunc))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	lispEntity_SetCar (pLispMgr, pEntMacroForm, pEntFunc) ;
+#if defined (DEBUG)
+	fprintf (stderr, "macroexpand start = ") ;
+	lispEntity_Print (pLispMgr, pEntMacroForm) ;
+	fprintf (stderr, "\n") ;
 #endif
+	lispMachineCode_SetLReg   (pLM, LM_LREG_ACC, pEntMacroForm) ;
+	lispMachineCode_PushState (pLM, &lispMachineState_ReturnOnly) ;
+	lispMachineCode_SetState  (pLM, &lispMachineState_EvalLambdaForm) ;
+	return	LMR_CONTINUE ;
+}
 
 /*
  *	(` ARG)
@@ -639,4 +613,3 @@ lispMachineState_evalMacroFinalize (
 }
 
 
-
