@@ -116,6 +116,35 @@ lispMachineState_SetMarker (
 	return	LMR_RETURN ;
 }
 
+/*	(set-marker-insertion-type MARKER TYPE)
+ */
+TLMRESULT
+lispMachineState_SetMarkerInsertionType (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr ;
+	TLispEntity*		pEntArglist ;
+	TLispEntity*		pEntMarker ;
+	TLispEntity*		pEntType ;
+	register Boolean	fType ;
+
+	assert (pLM != NULL) ;
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr != NULL) ;
+
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	if (TFAILED (lispEntity_GetCar  (pLispMgr, pEntArglist, &pEntMarker)) ||
+		TFAILED (lispEntity_Markerp (pLispMgr, pEntMarker)) ||
+		TFAILED (lispEntity_GetCadr (pLispMgr, pEntArglist, &pEntType))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	fType	= TFAILED (lispEntity_Nullp (pLispMgr, pEntType)) ;
+	lispMarker_SetInsertionType (pLispMgr, pEntMarker, fType) ;
+	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntType) ;
+	return	LMR_RETURN ;
+}
+
 /*
  *	(marker-position MARKER)
  *
