@@ -1,3 +1,23 @@
+/* # skkinput (Simple Kana-Kanji Input)
+ *
+ * This file is part of skkinput.
+ * Copyright (C) 2002
+ * Takashi SAKAMOTO (PXG01715@nifty.ne.jp)
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with skkinput; see the file COPYING.  If not, write to
+ * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
 #include "local.h"
 #include <stdio.h>
 #include <assert.h>
@@ -11,9 +31,10 @@ static	TLMRESULT	lispMachineState_compareNumberCommon	(TLispMachine*, Boolean (*
  *	Return sum of any number of arguments, which are numbers or markers.
  */
 TLMRESULT
-lispMachineState_Plus (TLispMachine* pLM)
+lispMachineState_Plus (
+	register TLispMachine*		pLM)
 {
-	TLispManager*	pLispMgr ;
+	register TLispManager*	pLispMgr ;
 	TLispEntity*	pTarget ;
 	TLispEntity*	pCar ;
 	TLispEntity*	pCdr ;
@@ -98,9 +119,10 @@ lispMachineState_Plus (TLispMachine* pLM)
  *	subtracts all but the first from the first.
  */
 TLMRESULT
-lispMachineState_Minus (TLispMachine* pLM)
+lispMachineState_Minus (
+	register TLispMachine*	pLM)
 {
-	TLispManager*	pLispMgr ;
+	register TLispManager*	pLispMgr ;
 	TLispEntity*	pTarget ;
 	TLispEntity*	pCar ;
 	TLispEntity*	pCdr ;
@@ -245,9 +267,10 @@ lispMachineState_Minus (TLispMachine* pLM)
  *	Returns product of any number of arguments, which are numbers or markers.
  */
 TLMRESULT
-lispMachineState_Multiply (TLispMachine* pLM)
+lispMachineState_Multiply (
+	register TLispMachine*	pLM)
 {
-	TLispManager*	pLispMgr ;
+	register TLispManager*	pLispMgr ;
 	TLispEntity*	pTarget ;
 	TLispEntity*	pCar ;
 	TLispEntity*	pCdr ;
@@ -327,9 +350,10 @@ lispMachineState_Multiply (TLispMachine* pLM)
  *	/ is a built-in function.
  */
 TLMRESULT
-lispMachineState_Divide (TLispMachine* pLM)
+lispMachineState_Divide (
+	register TLispMachine*	pLM)
 {
-	TLispManager*	pLispMgr ;
+	register TLispManager*	pLispMgr ;
 	TLispEntity*	pTarget ;
 	TLispEntity*	pCar ;
 	TLispEntity*	pCdr ;
@@ -466,6 +490,55 @@ lispMachineState_Divide (TLispMachine* pLM)
 	return	LMR_RETURN ;
 }
 
+/*	% is a built-in function.
+ *
+ *	Returns remainder of X divided by Y.
+ *	Both must be integers or markers.
+ *
+ *	(% X Y)
+ */
+TLMRESULT
+lispMachineState_Remainder (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr ;
+	TLispEntity*	pEntArglist ;
+	TLispEntity*	pEntX ;
+	TLispEntity*	pEntY ;
+	TLispEntity*	pEntRetval ;
+	TLispNumber		numX, numY ;
+	register long	lX, lY ;
+
+	assert (pLM != NULL) ;
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr != NULL) ;
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	assert (pEntArglist  != NULL) ;
+	if (TFAILED (lispEntity_GetCar  (pLispMgr, pEntArglist, &pEntX)) ||
+		TFAILED (lispEntity_IntegerOrMarkerp (pLispMgr, pEntX)) ||
+		TFAILED (lispEntity_GetCadr (pLispMgr, pEntArglist, &pEntY)) ||
+		TFAILED (lispEntity_IntegerOrMarkerp (pLispMgr, pEntY))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	(void) lispEntity_GetNumberValueOrMarkerPosition (pLispMgr, pEntX, &numX) ;
+	(void) lispEntity_GetNumberValueOrMarkerPosition (pLispMgr, pEntY, &numY) ;
+	lX	= numX.m_Value.m_lLong ;
+	lY	= numY.m_Value.m_lLong ;
+	if (lY == 0) {
+#if defined (DEBUG)
+		fprintf (stderr, "Arthmetic Error\n") ;
+#endif
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	if (TFAILED (lispMgr_CreateInteger (pLispMgr, lX % lY, &pEntRetval)) ||
+		pEntRetval == NULL)
+		return	LMR_ERROR ;
+	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+	return	LMR_RETURN ;
+}
+
 TLMRESULT
 lispMachineState_MathGreaterThan (TLispMachine* pLM)
 {
