@@ -142,8 +142,8 @@ lispMachineState_CurrentTimeString (
 	pStrTime	= ctime (&t) ;
 	if (pStrTime == NULL)
 		goto	error ;
-	strncpy (achBuffer, pStrTime, sizeof (achBuffer) - 1) ;
-	achBuffer [sizeof (achBuffer) - 1]	= '\0' ;
+	strncpy (achBuffer, pStrTime, NELEMENTS (achBuffer) - 1) ;
+	achBuffer [NELEMENTS (achBuffer) - 1]	= '\0' ;
 	nLength		= strlen (achBuffer) ;
 	while (nLength > 0 && (achBuffer [nLength - 1] == '\n' || achBuffer [nLength - 1] == '\r'))
 		nLength	-- ;
