@@ -74,10 +74,8 @@ lispMachineState_StringMatch (
 		lispMachineCode_SetError (pLM) ;
 		return	LMR_RETURN ;
 	}
-	pStrSTRING	+= lStart ;
-	nStrSTRING	-= lStart ;
 	lispMachineCode_SetRegmatchTarget (pLM, pEntSTRING) ;
-	if (TFAILED (lispMachineCode_StringMatch (pLM, pStrREGEXP, nStrREGEXP, pStrSTRING, nStrSTRING))) {
+	if (TFAILED (lispMachineCode_StringMatch (pLM, pStrREGEXP, nStrREGEXP, pStrSTRING, nStrSTRING, lStart))) {
 		lispMgr_CreateNil (pLispMgr, &pEntRetval) ;
 	} else {
 		int		nPosition ;
@@ -216,10 +214,146 @@ TLMRESULT
 lispMachineState_MatchData (
 	register TLispMachine*	pLM)
 {
-	TLispEntity*	pEntRetval ;
-	/*	marker ˤΤ񤷤ʤäȥѥ뤫ʡ*/
-	lispMachineCode_MatchData (pLM, &pEntRetval) ;
-	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+	register TLispManager*	pLispMgr ;
+	TLispEntity*			pEntArglist ;
+	TLispEntity*			pEntINTEGERS ;
+	TLispEntity*			pEntREUSE ;
+	TLispEntity*			pEntNode ;
+	register TLispEntity*	pEntNil ;
+	register TLispEntity*	pEntBuffer	= NULL ;
+	TLispConscell			conslst ;
+	register int			i ;
+	register regmatch_t*	pMatch ;
+	register Boolean		fInteger ;
+
+	assert (pLM != NULL) ;
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr != NULL) ;
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	if (TFAILED (lispEntity_GetCar  (pLispMgr, pEntArglist, &pEntINTEGERS)) ||
+		TFAILED (lispEntity_GetCadr (pLispMgr, pEntArglist, &pEntREUSE))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	fInteger	= lispEntity_Nullp (pLispMgr, pEntINTEGERS) ;
+	if (!fInteger &&
+		pLM->m_pEntRegMatch != NULL && 
+		TSUCCEEDED (lispEntity_Bufferp (pLispMgr, pLM->m_pEntRegMatch))) {
+		pEntBuffer	= pLM->m_pEntRegMatch ;
+	} else {
+		fInteger	= True ;
+	}
+		
+	pEntNil	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_NIL) ;
+	conslst.m_pCar	= conslst.m_pCdr	= NULL ;
+	pMatch	= pLM->m_aRegMatch ;
+	for (i = 0 ; i < MAX_REGEXP_MATCHES ; i ++, pMatch ++) {
+		if (pMatch->rm_so < 0)
+			break ;
+		if (fInteger) {
+			if (TFAILED (lispMgr_CreateInteger (pLispMgr, (long) pMatch->rm_so, &pEntNode)))
+				return	LMR_ERROR ;
+			lispEntity_Push2List (pLispMgr, &conslst, pEntNode) ;
+			if (TFAILED (lispMgr_CreateInteger (pLispMgr, (long) pMatch->rm_eo, &pEntNode)))
+				return	LMR_ERROR ;
+			lispEntity_Push2List (pLispMgr, &conslst, pEntNode) ;
+		} else {
+			if (TFAILED (lispMgr_CreateMarker (pLispMgr, &pEntNode)))
+				return	LMR_ERROR ;
+			lispEntity_Push2List (pLispMgr, &conslst, pEntNode) ;
+			lispBuffer_AddMarker (pLispMgr, pEntBuffer, pEntNode) ;
+			lispMarker_SetBufferPosition (pLispMgr, pEntNode, pEntBuffer, pMatch->rm_so) ;
+			if (TFAILED (lispMgr_CreateMarker (pLispMgr, &pEntNode)))
+				return	LMR_ERROR ;
+			lispEntity_Push2List (pLispMgr, &conslst, pEntNode) ;
+			lispBuffer_AddMarker (pLispMgr, pEntBuffer, pEntNode) ;
+			lispMarker_SetBufferPosition (pLispMgr, pEntNode, pEntBuffer, pMatch->rm_eo) ;
+		}
+	}
+	if (! fInteger) {
+		while (i < MAX_REGEXP_MATCHES) {
+			lispEntity_Push2List (pLispMgr, &conslst, pEntNil) ;
+			lispEntity_Push2List (pLispMgr, &conslst, pEntNil) ;
+			i	++ ;
+		}
+	}
+	if (conslst.m_pCar != NULL) {
+		lispMachineCode_SetLReg (pLM, LM_LREG_ACC, conslst.m_pCar) ;
+		lispEntity_Release (pLispMgr, conslst.m_pCar) ;
+		conslst.m_pCar	= NULL ;
+	} else {
+		lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntNil) ;
+	}
+	return	LMR_RETURN ;
+}
+
+/*	(set-match-data LIST)
+ *
+ *	Set internal data on last search match from elements of LIST.
+ *	LIST should have been created by calling `match-data' previously.
+ */
+TLMRESULT
+lispMachineState_SetMatchData (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr ;
+	TLispEntity*			pEntArglist ;
+	TLispEntity*			pEntLIST ;
+	TLispEntity*			pEntBuffer	= NULL ;
+	register TLispEntity*	pEntNil ;
+	register int			nMatch ;
+	register regmatch_t*	pMatch ;
+
+	assert (pLM != NULL) ;
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr != NULL) ;
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	if (TFAILED (lispEntity_GetCar (pLispMgr, pEntArglist, &pEntLIST)) ||
+		TFAILED (lispEntity_Listp  (pLispMgr, pEntLIST))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	
+	pEntNil	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_NIL) ;
+	pMatch	= pLM->m_aRegMatch ;
+	nMatch	= 0 ;
+	while (nMatch < MAX_REGEXP_MATCHES && TFAILED (lispEntity_Nullp (pLispMgr, pEntLIST))) {
+		TLispEntity*	pEntSO ;
+		TLispEntity*	pEntEO ;
+		TLispNumber		num ;
+
+		if (TFAILED (lispEntity_GetCar (pLispMgr, pEntLIST, &pEntSO))   ||
+			TFAILED (lispEntity_GetCdr (pLispMgr, pEntLIST, &pEntLIST)) ||
+			TFAILED (lispEntity_GetCar (pLispMgr, pEntLIST, &pEntEO))   ||
+			TFAILED (lispEntity_GetCdr (pLispMgr, pEntLIST, &pEntLIST))) {
+			lispMachineCode_SetError (pLM) ;
+			return	LMR_RETURN ;
+		}
+		if (pEntBuffer == NULL &&
+			TSUCCEEDED (lispEntity_Markerp (pLispMgr, pEntSO))) {
+			int	nPosition ;
+			(void) lispMarker_GetBufferPosition (pLispMgr, pEntSO, &pEntBuffer, &nPosition) ;
+		}
+		if (TFAILED (lispEntity_GetNumberOrMarkerValue (pLispMgr, pEntSO, &num))) {
+			pMatch->rm_so	= -1 ;
+		} else {
+			pMatch->rm_so	= (num.m_fFloatp)? -1 : num.m_Value.m_lLong ;
+		}
+		if (TFAILED (lispEntity_GetNumberOrMarkerValue (pLispMgr, pEntEO, &num))) {
+			pMatch->rm_eo	= -1 ;
+		} else {
+			pMatch->rm_eo	= (num.m_fFloatp)? -1 : num.m_Value.m_lLong ;
+		}
+		pMatch	++ ;
+		nMatch	++ ;
+	}
+	while (nMatch < MAX_REGEXP_MATCHES) {
+		pMatch->rm_so = pMatch->rm_eo	= -1 ;
+		pMatch	++ ;
+		nMatch	++ ;
+	}
+	lispMachineCode_SetRegmatchTarget (pLM, pEntBuffer) ;
+	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntNil) ;
 	return	LMR_RETURN ;
 }
 
@@ -328,7 +462,9 @@ lispMachineState_RegexpQuote (
 	return	nResult ;
 }
 
-
+/*========================================================================
+ *	private functions
+ */
 TLMRESULT
 lispMachine_search (
 	register TLispMachine*	pLM,
@@ -370,10 +506,11 @@ lispMachine_search (
 		lispBuffer_PointMin (pLispMgr, pEntBuffer, &nBOUND) ;
 	}
 	if (TFAILED (lispEntity_Nullp (pLispMgr, pEntBOUND))) {
-		long	lBOUND ;
-		if (TFAILED (lispEntity_GetIntegerValue (pLispMgr, pEntBOUND, &lBOUND)))
+		TLispNumber	numBOUND ;
+		if (TFAILED (lispEntity_GetNumberOrMarkerValue (pLispMgr, pEntBOUND, &numBOUND)) ||
+			numBOUND.m_fFloatp)
 			goto	error ;
-		nBOUND	= lBOUND ;
+		nBOUND	= numBOUND.m_Value.m_lLong ;
 	}
 	lispEntity_GetCdr  (pLispMgr, pEntArglist, &pEntArglist) ;
 	lispEntity_GetCar  (pLispMgr, pEntArglist, &pEntNOERROR) ;
@@ -387,6 +524,7 @@ lispMachine_search (
 			goto	error ;
 		nCOUNT	= lCOUNT ;
 	}
+	lispMachineCode_SetRegmatchTarget (pLM, pEntBuffer) ;
 	if (TFAILED ((*pCmd) (pLM, pEntBuffer, pStrSTRING, nStrSTRING, nBOUND, nCOUNT))) {
 		if (TSUCCEEDED (lispEntity_Nullp (pLispMgr, pEntNOERROR)))
 			goto	error ;
@@ -398,19 +536,17 @@ lispMachine_search (
 		TLispEntity*	pEntPoint ;
 		int				nPosition ;
 
-		(void) lispMachineCode_MatchBeginning (pLM, 0, &nPosition) ;
-#if defined (DEBUG)
-		fprintf (stderr, "search-%s: result = %d\n",
-				 (nDir > 0)? "forward" : "backward", nPosition) ;
-#endif
+		if (nDir > 0) {
+			(void) lispMachineCode_MatchEnd       (pLM, 0, &nPosition) ;
+		} else {
+			(void) lispMachineCode_MatchBeginning (pLM, 0, &nPosition) ;
+		}
 		if (TFAILED (lispMgr_CreateInteger (pLispMgr, nPosition, &pEntRetval)))
 			return	LMR_ERROR ;
 
 		/*	backward ξˤϡposition  beginning ΰ֤ˤʤ롣
 		 *	forward  ξˤϡposition  end       ΰ֤ˤʤ롣
 		 */
-		if (nDir > 0)
-			(void) lispMachineCode_MatchEnd       (pLM, 0, &nPosition) ;
 		lispBuffer_PointMarker (pLispMgr, pEntBuffer, &pEntPoint) ;
 		lispMarker_SetBufferPosition (pLispMgr, pEntPoint, pEntBuffer, nPosition) ;
 	}
