@@ -25,21 +25,29 @@
 #include "jisyo.h"
 #include "cstring.h"
 
-static	Boolean	lispMachine_skkOkuriArip (TLispMachine*) ;
-static	Boolean	lispMachine_skkMergeAllEntryp (TLispMachine*) ;
-static	Boolean	lispMachine_skkProcessOkuriEarlyp (TLispMachine*) ;
-static	Boolean	lispMachine_skkGetSearchKey (TLispMachine*, const Char**, int*) ;
-static	Boolean	lispMachine_jSearchServerSubr (TLispMachine*, PSKKSERVERJISYO, TLispEntity**) ;
-static	Boolean	lispMachine_jSearchJisyoFile (TLispMachine*, const Char*, int, int, TLispEntity**) ;
-static	Boolean	lispMachine_jSearchJisyoFileSubr (const Char*, int, const Char*, int, Boolean, int, TVarbuffer*) ;
-static	Boolean	lispMachine_jOkuriSearchSubr (TLispMachine*, const Char*, int, const Char*, int, TLispEntity**) ;
+static	Boolean	lispMachine_jOkuriArip			(TLispMachine*) ;
+static	Boolean	lispMachine_jMergeAllEntryp		(TLispMachine*) ;
+static	Boolean	lispMachine_jProcessOkuriEarlyp	(TLispMachine*) ;
+static	Boolean	lispMachine_jGetSearchKey		(TLispMachine*, const Char**, int*) ;
+static	Boolean	lispMachine_jSearchServerSubr	(TLispMachine*, PSKKSERVERJISYO, TLispEntity**) ;
+static	Boolean	lispMachine_jSearchJisyoFile	(TLispMachine*, const Char*, int, int, TLispEntity**) ;
+static	Boolean	lispMachine_jSearchJisyoFileSubr(const Char*, int, const Char*, int, Boolean, int, TVarbuffer*) ;
+static	Boolean	lispMachine_jOkuriSearchSubr	(TLispMachine*, const Char*, int, const Char*, int, TLispEntity**) ;
 static	Boolean	lispMachine_makeCandidateVectorFromString (TLispManager*, TVarbuffer*, TLispEntity**) ;
-static	Boolean	lispMachine_computeHenkanVectors (TLispMachine*, const Char*, int, const Char*, int, Boolean, Boolean, TLispEntity**) ;
-static	Boolean	lispMachine_jVectorAdd (TLispMachine*, TLispEntity*, TLispEntity*, TLispEntity**) ;
-static	Boolean	lispMachine_jVectorMem (TLispMachine*, TLispEntity*, TLispEntity*, Boolean*) ;
-static	Boolean	lispMachine_getSkkServerList (TLispMachine*, TLispEntity**) ;
-static	Boolean	lispMachine_getSkkPortnum (TLispMachine*, int*) ;
+static	Boolean	lispMachine_computeHenkanVectors(TLispMachine*, const Char*, int, const Char*, int, Boolean, Boolean, TLispEntity**) ;
+static	Boolean	lispMachine_jVectorAdd			(TLispMachine*, TLispEntity*, TLispEntity*, TLispEntity**) ;
+static	Boolean	lispMachine_jVectorMem			(TLispMachine*, TLispEntity*, TLispEntity*, Boolean*) ;
+static	Boolean	lispMachine_getSkkServerList	(TLispMachine*, int, TLispEntity**) ;
+static	Boolean	lispMachine_getSkkPortnum		(TLispMachine*, int*) ;
 
+static	Boolean	lispMachine_skkSearchServerSubr	(TLispMachine*, PSKKSERVERJISYO, const Char*, int, TLispEntity**) ;
+static	Boolean	lispMachine_skkComputeHenkanLists(TLispMachine*, const Char*, int, const Char*, int, TLispEntity**) ;
+
+/*========================================================================
+ *	skk8.6 Τδؿ
+ */
+/*	(j-search-server file limit &optional nomsg)
+ */
 TLMRESULT
 lispMachineState_JSearchServer (
 	register TLispMachine*	pLM)
@@ -52,7 +60,7 @@ lispMachineState_JSearchServer (
 	TLispEntity*				pEntRetval ;
 	int							nSkkPort ;
 
-	if (TFAILED (lispMachine_getSkkServerList (pLM, &pValServerListOrg))) {
+	if (TFAILED (lispMachine_getSkkServerList (pLM, LISPMGR_INDEX_SKK_SERVER_LIST, &pValServerListOrg))) {
 		lispMachineCode_SetError (pLM) ;
 		return	LMR_RETURN ;
 	}
@@ -173,7 +181,7 @@ lispMachineState_JOkuriSearchSubr (
 	if (TFAILED (lispEntity_GetStringValue (pLispMgr, pEntJisyo, &pStrJisyo, &nStrJisyo))) 
 		goto	error ;
 
-	if (TFAILED (lispMachine_skkGetSearchKey (pLM, &pStrKey, &nStrKey)))
+	if (TFAILED (lispMachine_jGetSearchKey (pLM, &pStrKey, &nStrKey)))
 		goto	error ;
 
 	if (TFAILED (lispMachine_jOkuriSearchSubr (pLM, pStrJisyo, nStrJisyo, pStrKey, nStrKey, &pEntRetval)))
@@ -197,9 +205,8 @@ lispMachineState_JCompleteion (
 	return	LMR_ERROR ;
 }
 
-
 Boolean
-lispMachine_skkOkuriArip (
+lispMachine_jOkuriArip (
 	register TLispMachine*	pLM)
 {
 	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
@@ -214,14 +221,14 @@ lispMachine_skkOkuriArip (
 }
 
 Boolean
-lispMachine_skkMergeAllEntryp (
+lispMachine_jMergeAllEntryp (
 	register TLispMachine*	pLM)
 {
 	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
 	register TLispEntity*	pEntOkuriStrict ;
 	TLispEntity*			pValOkuriStrict ;
 
-	if (TFAILED (lispMachine_skkOkuriArip (pLM)))
+	if (TFAILED (lispMachine_jOkuriArip (pLM)))
 		return	True ;
 
 	pEntOkuriStrict	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_J_HENKAN_OKURI_STRICTLY) ;
@@ -232,7 +239,7 @@ lispMachine_skkMergeAllEntryp (
 }
 
 Boolean
-lispMachine_skkProcessOkuriEarlyp (
+lispMachine_jProcessOkuriEarlyp (
 	register TLispMachine*	pLM)
 {
 	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
@@ -270,7 +277,7 @@ lispMachine_skkGetOkurigana (
 }
 
 Boolean
-lispMachine_skkGetSearchKey (
+lispMachine_jGetSearchKey (
 	register TLispMachine*	pLM,
 	register const Char**	ppStrKey,
 	register int*			pnStrKey)
@@ -280,7 +287,7 @@ lispMachine_skkGetSearchKey (
 	register TLispEntity*	pEntKey ;
 	TLispEntity*			pValKey ;
 
-	fOkuriAri	= lispMachine_skkOkuriArip (pLM) ;
+	fOkuriAri	= lispMachine_jOkuriArip (pLM) ;
 	if (fOkuriAri) {
 		pEntKey	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_J_HENKAN_KEY) ;
 	} else {
@@ -314,8 +321,8 @@ lispMachine_jSearchServerSubr (
 	Boolean			fOkuriAri ;
 	TLispEntity*	pEntRetval ;
 
-	fOkuriAri	= lispMachine_skkOkuriArip (pLM) ;
-	if (TFAILED (lispMachine_skkGetSearchKey (pLM, &pStrKey, &nStrKey)))
+	fOkuriAri	= lispMachine_jOkuriArip (pLM) ;
+	if (TFAILED (lispMachine_jGetSearchKey (pLM, &pStrKey, &nStrKey)))
 		goto	error ;
 
 	TVarbuffer_Initialize (&vbuf, sizeof (Char)) ;
@@ -340,11 +347,11 @@ lispMachine_jSearchServerSubr (
 		TLispEntity*	pEnt0th ;
 		TLispEntity*	pEnt2th ;
 		
-		fOkuriEarly	= lispMachine_skkProcessOkuriEarlyp (pLM) ;
+		fOkuriEarly	= lispMachine_jProcessOkuriEarlyp (pLM) ;
 		if (TFAILED (lispMachine_computeHenkanVectors (pLM, pStrResult, nStrResult, pStrOkurigana, nStrOkurigana, fOkuriAri, fOkuriEarly, &pEntRetval))) 
 			goto	error ;
 		lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
-		if (TSUCCEEDED (lispMachine_skkMergeAllEntryp (pLM))) {
+		if (TSUCCEEDED (lispMachine_jMergeAllEntryp (pLM))) {
 			lispEntity_GetCar   (pLispMgr, pEntRetval, &pEnt0th) ;
 			lispEntity_GetCaddr (pLispMgr, pEntRetval, &pEnt2th) ;
 			lispMachine_jVectorAdd (pLM, pEnt0th, pEnt2th, &pEntRetval) ;
@@ -386,15 +393,15 @@ lispMachine_jSearchJisyoFile (
 
 	if (TFAILED (TVarbuffer_Initialize (&vbuf, sizeof (Char))))
 		goto	error ;
-	fOkuri	= lispMachine_skkOkuriArip (pLM) ;
-	if (TFAILED (lispMachine_skkGetSearchKey (pLM, &pStrKey, &nStrKey)))
+	fOkuri	= lispMachine_jOkuriArip (pLM) ;
+	if (TFAILED (lispMachine_jGetSearchKey (pLM, &pStrKey, &nStrKey)))
 		goto	error ;
 	if (TFAILED (lispMachine_jSearchJisyoFileSubr (pStrFile, nStrFile, pStrKey, nStrKey, fOkuri, nLimit, &vbuf)))
 		goto	error ;
 	if (fOkuri) {
 		if (TFAILED (lispMachine_skkGetOkurigana  (pLM, &pStrOkurigana, &nStrOkurigana)))
 			goto	error ;
-		fOkuriEarly		= lispMachine_skkProcessOkuriEarlyp (pLM) ;
+		fOkuriEarly		= lispMachine_jProcessOkuriEarlyp (pLM) ;
 	} else {
 		pStrOkurigana	= NULL ;
 		nStrOkurigana	= 0 ;
@@ -404,7 +411,7 @@ lispMachine_jSearchJisyoFile (
 	nStrResult	= TVarbuffer_GetUsage  (&vbuf) ;
 	if (TFAILED (lispMachine_computeHenkanVectors (pLM, pStrResult, nStrResult, pStrOkurigana, nStrOkurigana, fOkuri, fOkuriEarly, &pEntRetval))) 
 		goto	error ;
-	if (TSUCCEEDED (lispMachine_skkMergeAllEntryp (pLM))) {
+	if (TSUCCEEDED (lispMachine_jMergeAllEntryp (pLM))) {
 		lispEntity_GetCar   (pLispMgr, pEntRetval, &pEnt0th) ;
 		lispEntity_GetCaddr (pLispMgr, pEntRetval, &pEnt2th) ;
 		lispMachine_jVectorAdd (pLM, pEnt2th, pEnt0th, &pEntRetval) ;
@@ -720,6 +727,7 @@ lispMachine_jVectorMem (
 Boolean
 lispMachine_getSkkServerList (
 	register TLispMachine*	pLM,
+	register int			nEntityIndex,
 	register TLispEntity**	ppValServerList)
 {
 	register TLispManager*	pLispMgr ;
@@ -735,7 +743,7 @@ lispMachine_getSkkServerList (
 	pLispMgr	= pLM->m_pLispMgr ;
 	assert (pLispMgr != NULL) ;
 
-	pEntServerList	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_SKK_SERVER_LIST) ;
+	pEntServerList	= lispMgr_GetReservedEntity (pLispMgr, nEntityIndex) ;
 	assert (pEntServerList != NULL) ;
 	if (TFAILED (lispMachine_GetCurrentSymbolValue (pLM, pEntServerList, &pValServerList)) ||
 		TSUCCEEDED (lispEntity_Voidp (pLispMgr, pValServerList))) 
@@ -792,6 +800,276 @@ lispMachine_getSkkPortnum (
 	}
 	if (pnSkkPort != NULL)
 		*pnSkkPort	= nPort ;
-	return	TRUE ;
+	return	True ;
+}
+
+/*========================================================================*
+ *	skk10.62a Τδؿ
+ */
+TLMRESULT
+lispMachineState_SkkinputSearchServerSubr (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*		pLispMgr	= pLM->m_pLispMgr ;
+	TLispEntity*				pEntArglist ;
+	TLispEntity*				pEntKey ;
+	register PSKKSERVERJISYO	pJisyo ;
+	register TLispEntity*		pValServerList ;
+	TLispEntity*				pValServerListOrg ;
+	register TLMRESULT			lmResult	= LMR_RETURN ;
+	const Char*					pStrKey ;
+	int							nStrKey ;
+	TLispEntity*				pEntRetval ;
+	int							nSkkPort ;
+
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	if (TFAILED (lispEntity_GetCar (pLispMgr, pEntArglist, &pEntKey)) ||
+		TFAILED (lispEntity_GetStringValue (pLispMgr, pEntKey, &pStrKey, &nStrKey))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+
+	if (TFAILED (lispMachine_getSkkServerList (pLM, LISPMGR_INDEX_SKK_SERVERS_LIST, &pValServerListOrg))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	(void) lispMachine_getSkkPortnum (pLM, &nSkkPort) ;
+
+	pJisyo				= NULL ;
+	pEntRetval			= NULL ;
+	pValServerList		= pValServerListOrg ;
+	while (TFAILED (lispEntity_Nullp (pLispMgr, pValServerList))) {
+		TLispEntity*	pEntHost ;
+		TLispEntity*	pEntCar ;
+		TLispEntity*	pEntCdr ;
+		const Char*		pStrHost ;
+		int				nStrHost ;
+
+		if (TFAILED (lispEntity_GetCar (pLispMgr, pValServerList, &pEntCar)) ||
+			TFAILED (lispEntity_GetCdr (pLispMgr, pValServerList, &pEntCdr)) ||
+			TFAILED (lispEntity_GetCar (pLispMgr, pEntCar, &pEntHost))) {
+			lispMachineCode_SetError (pLM) ;
+			goto	exit_func ;
+		}
+		if (TFAILED (lispEntity_GetStringValue (pLispMgr, pEntHost, &pStrHost, &nStrHost)))
+			goto	skip ;
+
+		pJisyo	= SkkServerJisyo_Find (pStrHost, nStrHost) ;
+		if (pJisyo == NULL) 
+			pJisyo	= SkkServerJisyo_Register (pStrHost, nStrHost, nSkkPort /*1178*/) ;
+
+		if (pJisyo != NULL) {
+			if (TFAILED (lispMachine_skkSearchServerSubr (pLM, pJisyo, pStrKey, nStrKey, &pEntRetval))) {
+				lispMachineCode_SetError (pLM) ;
+				goto	exit_func ;
+			}
+			if (pEntRetval != NULL) 
+				break ;
+		}
+	  skip:
+		pValServerList	= pEntCdr ;
+	}
+
+	/*	ʤäФϼĴ٤ʤ褦ˤ롣
+	 */
+	if (pValServerList != NULL) {
+		register TLispEntity*	pEntServerList ;
+
+		pEntServerList	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_SKK_SERVERS_LIST) ;
+		lispMachine_SetCurrentSymbolValue (pLM, pEntServerList, pValServerList) ;
+	}
+
+	/*	
+	 */
+	if (pEntRetval == NULL)
+		pEntRetval	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_NIL) ;
+
+	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+ exit_func:
+	lispEntity_Release (pLispMgr, pValServerListOrg) ;
+	return	lmResult ;
+}
+
+TLMRESULT
+lispMachineState_SkkinputComputeHenkanLists (
+	register TLispMachine*		pLM)
+{
+	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	TLispEntity*			pEntArglist ;
+	TLispEntity*			pEntItem ;
+	TLispEntity*			pEntOkurigana ;
+	const Char*				pStrItem ;
+	int						nStrItem ;
+	TLispEntity*			pEntRetval ;
+	const Char*				pStrOkuri ;
+	int						nStrOkuri ;
+
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	if (TFAILED (lispEntity_GetCar  (pLispMgr, pEntArglist, &pEntItem)) ||
+		TFAILED (lispEntity_GetStringValue (pLispMgr, pEntItem, &pStrItem, &nStrItem)) ||
+		TFAILED (lispEntity_GetCadr (pLispMgr, pEntArglist, &pEntOkurigana))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	if (TFAILED (lispEntity_GetStringValue (pLispMgr, pEntOkurigana, &pStrOkuri, &nStrOkuri))) {
+		pStrOkuri	= NULL ;
+		nStrOkuri	= 0 ;
+	}
+	if (TFAILED (lispMachine_skkComputeHenkanLists (pLM, pStrItem, nStrItem, pStrOkuri, nStrOkuri, &pEntRetval))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+	return	LMR_RETURN ;
+}
+
+Boolean
+lispMachine_skkSearchServerSubr (
+	register TLispMachine*		pLM,
+	register PSKKSERVERJISYO	pJisyo,
+	register const Char*		pStrKey,
+	register int				nStrKey,
+	register TLispEntity**		ppEntRetval)
+{
+	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	TVarbuffer				vbuf ;
+	register const Char*	pStrResult ;
+	register int			nStrResult ;
+	TLispEntity*			pEntRetval	= NULL ;
+	register Boolean		fRetval		= False ;
+
+	TVarbuffer_Initialize (&vbuf, sizeof (Char)) ;
+	if (TSUCCEEDED (SkkServerJisyo_Search (pJisyo, pStrKey, nStrKey, &vbuf))) {
+		pStrResult	= TVarbuffer_GetBuffer (&vbuf) ;
+		nStrResult	= TVarbuffer_GetUsage (&vbuf) ;
+		fRetval		= lispMgr_CreateString (pLispMgr, pStrResult, nStrResult, &pEntRetval) ;
+	}
+	TVarbuffer_Uninitialize (&vbuf) ;
+	*ppEntRetval	= pEntRetval ;
+	return	fRetval ;
+}
+
+Boolean
+lispMachine_skkComputeHenkanLists (
+	register TLispMachine*	pLM,
+	register const Char*	pString,
+	register int			nString,
+	register const Char*	strOkurigana,
+	register int			nstrOkurigana,
+	register TLispEntity**	ppEntResult)
+{
+	register TLispManager*	pLispMgr ;
+	register int			i ;
+	register Boolean		fRetval	= False ;
+	register const Char*	ptr ;
+	register int			nptr, nItem, nStage ;
+	register Char			ccHead ;
+	register TLispEntity*	pEntNil ;
+	TLispConscell	rQ [4] ;
+	TLispEntity*	pEntItem ;
+	TLispEntity*	rList [4] ;
+
+	assert (pLM != NULL) ;
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr != NULL) ;
+	nStage		= 1 ;
+
+	for (i = 0 ; i < NELEMENTS (rQ) ; i ++) 
+		rQ [i].m_pCar = rQ [i].m_pCdr = NULL ;
+
+	while (nString > 0 && *pString == '/') {
+		pString	++ ;
+		nString	-- ;
+	}
+	while (nString > 0) {
+		ptr		= pString ;
+		nptr	= nString ;
+		while (nptr > 0 && *ptr != '/') {
+			ptr		++ ;
+			nptr	-- ;
+		}
+		nItem	= ptr - pString ;
+		if (nItem <= 0) {
+			ccHead	= 0 ;
+		} else {
+			ccHead	= *pString ;
+		}
+		if (nItem > 0) {
+			if (TFAILED (lispMgr_CreateString (pLispMgr, pString, nItem, &pEntItem)))
+				goto	error ;
+		} else {
+			if (TFAILED (lispMgr_CreateString (pLispMgr, NULL, 0, &pEntItem)))
+				goto	error ;
+		}
+
+		if (ccHead == '[' && nStage <= 2) {
+			if (nItem == (nstrOkurigana + 1) && !Cstrncmp (strOkurigana, pString + 1, nstrOkurigana)) {
+				/*	Q2  item ɲá*/
+				if (TFAILED (lispEntity_Push2List (pLispMgr, &rQ [1], pEntItem)))
+					goto	error ;
+				nStage	= 3 ;
+			} else {
+				nStage	= 2 ;
+				/*	Q2  item ɲá*/
+				if (TFAILED (lispEntity_Push2List (pLispMgr, &rQ [1], pEntItem)))
+					goto	error ;
+			}
+		} else {
+			switch (nStage) {
+			case	1:
+				/*	Q1  item ɲá*/
+				if (TFAILED (lispEntity_Push2List (pLispMgr, &rQ [0], pEntItem)))
+					goto	error ;
+				break ;
+			case	2:
+				/*	Q2  item ɲá*/
+				if (TFAILED (lispEntity_Push2List (pLispMgr, &rQ [1], pEntItem)))
+					goto	error ;
+				break ;
+			case	3:
+				if (ccHead == ']') {
+					nStage	= 4 ;
+					/*	Q4  item ɲá*/
+					if (TFAILED (lispEntity_Push2List (pLispMgr, &rQ [3], pEntItem)))
+						goto	error ;
+				} else {
+					/*	Q3  item ɲá*/
+					if (TFAILED (lispEntity_Push2List (pLispMgr, &rQ [2], pEntItem)))
+						goto	error ;
+				}
+				break ;
+			case	4:
+				/*	Q4  item ɲá*/
+				if (TFAILED (lispEntity_Push2List (pLispMgr, &rQ [3], pEntItem)))
+					goto	error ;
+				break ;
+			default:
+				/*	顼*/
+				break ;
+			}
+		}
+		pString	= ptr  + 1 ;
+		nString	= nptr - 1 ;
+	}
+	/* (list (queue-all q1) (queue-all q2) (queue-all q3) (queue-all q4)) ֤*/
+
+	pEntNil	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_NIL) ;
+	for (i = 0 ; i < NELEMENTS (rQ) ; i ++) {
+		if (rQ [i].m_pCar != NULL) {
+			rList [i]	= rQ [i].m_pCar ;
+		} else {
+			rList [i]	= pEntNil ;
+		}
+	}
+	fRetval	= lispMgr_CreateList (pLispMgr, rList, NELEMENTS (rQ), ppEntResult) ;
+
+  error:
+	for (i = 0 ; i < NELEMENTS (rQ) ; i ++) {
+		if (rQ [i].m_pCar != NULL) {
+			lispEntity_Release (pLispMgr, rQ [i].m_pCar) ;
+			rQ [i].m_pCar	= NULL ;
+		}
+	}
+	return	fRetval ;
 }
 
