@@ -25,6 +25,13 @@
 #include "varbuffer.h"
 #include "cstring.h"
 
+typedef struct tagEmacsCharsetDef {
+	const char*		m_strName ;
+	int				m_nID ;
+	unsigned long	m_uOffset ;
+	unsigned long	m_uMask ;
+} TEmacsCharsetDef ;
+
 static	TLMRESULT	lispMachineState_mapconcatListApply		(TLispMachine*) ;
 static	TLMRESULT	lispMachineState_mapconcatListPostApply	(TLispMachine*) ;
 static	TLMRESULT	lispMachineState_mapconcatVectorApply	(TLispMachine*) ;
@@ -37,6 +44,53 @@ static	TLMRESULT	lispMachineState_tryCompletionAlist		(TLispMachine*) ;
 static	TLMRESULT	lispMachineState_tryCompAlistPostPredict(TLispMachine*) ;
 static	TLMRESULT	lispMachineState_tryCompAlistFin		(TLispMachine*) ;
 
+static const char*	astrSkkinputCharset []	= {
+	"ascii",					"iso8859-1",
+	"iso8859-2",				"iso8859-3",
+	"iso8859-4",				"iso8859-5",
+	"iso8859-6",				"iso8859-7",
+	"iso8859-8",				"iso8859-9",
+	"iso8859-14",				"iso8859-15",
+	"japanese-jisx0201-1976",	"japanese-jisx0208-1978",
+	"japanese-jisx0208-1983",	"japanese-jisx0212-1990",
+	"chinese-gb2312-1980",		"chinese-gb12345-90",
+	"chinese-gb7589-87",		"chinese-gb7590-87",
+	"chinese-gb13131-91",		"chinese-gb13132-91",
+	"korean-ksc5601-1987",		"korean-ksc5601-1992",
+	"chinese-cns11643-1992",
+} ;
+
+/*	ࡢemacs  version ˤä charset-list ֤ͤɼ˰㤦Ȥ
+ *	դʤä*/
+static TEmacsCharsetDef	arEmacsCharsets []	= {
+	{ "ascii",					KCHARSET_ASCII, 			0,			0x7F, },
+	{ "latin-iso8859-1",		KCHARSET_ISO8859_1,			0,			0xFF, },
+	{ "latin-iso8859-2",		KCHARSET_ISO8859_2,			0,			0xFF, },
+	{ "latin-iso8859-3",		KCHARSET_ISO8859_3,			0,			0xFF, },
+	{ "latin-iso8859-4",		KCHARSET_ISO8859_4,			0,			0xFF, },
+	{ "cyrillic-iso8859-5",		KCHARSET_ISO8859_5,			0,			0xFF, },
+	{ "arabic-iso8859-6",		KCHARSET_ISO8859_6,			0,			0xFF, },
+	{ "latin-iso8859-7",		KCHARSET_ISO8859_7,			0,			0xFF, },
+	{ "hebrew-iso8859-8",		KCHARSET_ISO8859_8,			0,			0xFF, },
+	{ "latin-iso8859-9",		KCHARSET_ISO8859_9,			0,			0xFF, },
+	{ "latin-iso8859-14",		KCHARSET_ISO8859_14,		0,			0xFF, },
+	{ "latin-iso8859-15",		KCHARSET_ISO8859_15,		0,			0xFF, },
+	{ "latin-jisx0201",			KCHARSET_JISX0201_1976,		0,			0x7F, },
+	{ "katakana-jisx0201",		KCHARSET_JISX0201_1976,		0x80,		0x7F, },
+	{ "japanese-jisx0208-1978",	KCHARSET_JISX0208_1978,		0,			0x7F7F, },
+	{ "japanese-jisx0208",		KCHARSET_JISX0208_1983,		0,			0x7F7F, },
+	{ "japanese-jisx0212",		KCHARSET_JISX0212_1990,		0,			0x7F7F, },
+	{ "korean-ksc5601",			KCHARSET_KSC5601_1987,		0,			0x7F7F, },
+	{ "korean-ksc5601-1992",	KCHARSET_KSC5601_1992,		0,			0x7F7F, },
+	{ "chinese-gb2312",			KCHARSET_GB2312_1980,		0,			0x7F7F,	},
+	{ "chinese-cns11643-1",		KCHARSET_CNS11643_1992,		0x010000,	0x7F7F, },
+	{ "chinese-cns11643-2",		KCHARSET_CNS11643_1992,		0x020000,	0x7F7F, },
+	{ "chinese-cns11643-3",		KCHARSET_CNS11643_1992,		0x030000,	0x7F7F, },
+	{ "chinese-cns11643-4",		KCHARSET_CNS11643_1992,		0x040000,	0x7F7F, },
+	{ "chinese-cns11643-5",		KCHARSET_CNS11643_1992,		0x050000,	0x7F7F, },
+	{ "chinese-cns11643-6",		KCHARSET_CNS11643_1992,		0x060000,	0x7F7F, },
+	{ "chinese-cns11643-7",		KCHARSET_CNS11643_1992,		0x070000,	0x7F7F, },
+} ;
 
 TLMRESULT
 lispMachineState_String (
@@ -1118,21 +1172,6 @@ TLMRESULT
 lispMachineState_CharCharset (
 	register TLispMachine*	pLM)
 {
-	static const char*	astrCharset []	= {
-		"ascii",					"iso8859-1",
-		"iso8859-2",				"iso8859-3",
-		"iso8859-4",				"iso8859-5",
-		"iso8859-6",				"iso8859-7",
-		"iso8859-8",				"iso8859-9",
-		"iso8859-14",				"iso8859-15",
-		"japanese-jisx0201-1976",	"japanese-jisx0208-1978",
-		"japanese-jisx0208-1983",	"japanese-jisx0212-1990",
-		"chinese-gb2312-1980",		"chinese-gb12345-90",
-		"chinese-gb7589-87",		"chinese-gb7590-87",
-		"chinese-gb13131-91",		"chinese-gb13132-91",
-		"korean-ksc5601-1987",		"korean-ksc5601-1992",
-		"chinese-cns11643-1992",
-	} ;
 	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
 	TLispEntity*	pEntArglist ;
 	TLispEntity*	pEntCH ;
@@ -1155,7 +1194,7 @@ lispMachineState_CharCharset (
 		lispMachineCode_SetError (pLM) ;
 		return	LMR_RETURN ;
 	}
-	if (TFAILED (lispMgr_InternSymbolA (pLispMgr, astrCharset [nSet], strlen (astrCharset [nSet]), &pEntRetval)))
+	if (TFAILED (lispMgr_InternSymbolA (pLispMgr, astrSkkinputCharset [nSet], strlen (astrSkkinputCharset [nSet]), &pEntRetval)))
 		return	LMR_ERROR ;
 	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
 	return	LMR_RETURN ;
@@ -1201,3 +1240,186 @@ lispMachineState_StringLessp (
 	return	LMR_RETURN ;
 }
 
+/*	(string-width STR)
+ *
+ *	Return width of STRING when displayed in the current buffer.
+ *	Width is measured by how many columns it occupies on the screen.
+ *	When calculating width of a multibyte character in STRING,
+ *	only the base leading-code is considered; the validity of
+ *	the following bytes is not checked.
+ */
+TLMRESULT
+lispMachineState_StringWidth (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	TLispEntity*	pEntArglist ;
+	TLispEntity*	pEntSTR ;
+	register int	nWidth ;
+	TLispEntity*	pEntRetval ;
+	const Char*		pString ;
+	int				nString ;
+	register int	nCharset ;
+
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	lispEntity_GetCar  (pLispMgr, pEntArglist, &pEntSTR) ;
+	if (TFAILED (lispEntity_GetStringValue (pLispMgr, pEntSTR, &pString, &nString))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	nWidth	= 0 ;
+	while (nString > 0) {
+		nWidth	++ ;
+		nCharset	= Char_Charset (*pString) ;
+		if (KCHARSET_2BYTES_CHARACTER <= nCharset && nCharset < MAX_CHARSET)
+			nWidth	++ ;
+		pString	++ ;
+		nString	-- ;
+	}
+	if (TFAILED (lispMgr_CreateInteger (pLispMgr, (long) nWidth, &pEntRetval)))
+		return	LMR_ERROR ;
+	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+	return	LMR_RETURN ;
+}
+
+/*	(make-char CHARSET &optional C1 C2)
+ */
+TLMRESULT
+lispMachineState_MakeChar (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	TLispEntity*	pEntArglist ;
+	TLispEntity*	pEntCharset ;
+	TLispEntity*	pEntC1 ;
+	TLispEntity*	pEntC2 ;
+	const Char*		strSymName ;
+	int				nSymName ;
+	register int	i ;
+	register const TEmacsCharsetDef*	pCharset ;
+
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	if (TFAILED (lispEntity_GetCar  (pLispMgr, pEntArglist, &pEntCharset)) ||
+		TFAILED (lispEntity_Symbolp (pLispMgr, pEntCharset)) ||
+		TFAILED (lispEntity_GetCdr  (pLispMgr, pEntArglist, &pEntArglist)) ||
+		TFAILED (lispEntity_GetCar  (pLispMgr, pEntArglist, &pEntC1)) ||
+		TFAILED (lispEntity_GetCadr (pLispMgr, pEntArglist, &pEntC2))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	(void) lispEntity_GetSymbolName (pLispMgr, pEntCharset, &strSymName, &nSymName) ;
+	pCharset	= arEmacsCharsets ;
+	for (i = 0 ; i < NELEMENTS (arEmacsCharsets) ; i ++, pCharset ++) {
+		register int	nLength	= strlen (pCharset->m_strName) ;
+		if (!cmemccmp (strSymName, nSymName, pCharset->m_strName, nLength) &&
+			nSymName == nLength) {
+			TLispEntity*	pEntRetval ;
+			register Char	cc ;
+			long			lC1, lC2 ;
+
+			if (lispEntity_Nullp (pLispMgr, pEntC1)) {
+				lC1	= 0 ;
+			} else {
+				if (TFAILED (lispEntity_GetIntegerValue (pLispMgr, pEntC1, &lC1)))
+					goto	error ;
+				lC1	&= 0x00FF ;
+			}
+			if (lispEntity_Nullp (pLispMgr, pEntC2)) {
+				lC2	=0 ;
+			} else {
+				if (TFAILED (lispEntity_GetIntegerValue (pLispMgr, pEntC2, &lC2)))
+					goto	error ;
+				lC2	&= 0x00FF ;
+			}
+			cc	= Char_Make (pCharset->m_nID, pCharset->m_uOffset + (((lC1 << 8) | lC2) & pCharset->m_uMask)) ;
+			if (TFAILED (lispMgr_CreateInteger (pLispMgr, cc, &pEntRetval)))
+				return	LMR_ERROR ;
+			lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+			return	LMR_RETURN ;
+		}
+	}
+  error:
+	lispMachineCode_SetError (pLM) ;
+	return	LMR_RETURN ;
+}
+
+/*
+ *	Return t if OBJECT is a valid normal character.
+ *	If optional arg GENERICP is non-nil, also return t if OBJECT is
+ *	a valid generic character.
+ *
+ *	ࡢvalid normal character  valid generic character 
+ *	褬㤦
+ */
+TLMRESULT
+lispMachineState_CharValidp (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr	= pLM->m_pLispMgr ;
+	TLispEntity*	pEntArglist ;
+	TLispEntity*	pEntOBJECT ;
+	TLispEntity*	pEntGENERICP ;
+	TLispEntity*	pEntRetval ;
+	Char			cc ;
+	register int	nCharset ;
+	register unsigned long	uCode ;
+	register Boolean	fRetval, fGenericp ;
+
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	if (TFAILED (lispEntity_GetCar  (pLispMgr, pEntArglist, &pEntOBJECT)) ||
+		TFAILED (lispEntity_GetCadr (pLispMgr, pEntArglist, &pEntGENERICP))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	if (TFAILED (lispEntity_GetIntegerValue (pLispMgr, pEntOBJECT, &cc))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	fGenericp	= TFAILED (lispEntity_Nullp (pLispMgr, pEntGENERICP)) ;
+	nCharset	= Char_Charset (cc) ;
+	uCode		= Char_Code (cc) ;
+	if (nCharset < KCHARSET_ASCII || nCharset >= MAX_CHARSET) {
+		fRetval	= False ;
+	} else {
+		if (KCHARSET_7BIT_CHARACTER <= nCharset &&
+			nCharset < KCHARSET_1BYTE_CHARACTER) {
+			fRetval	= (uCode < 0x80) ;
+		} else if (KCHARSET_1BYTE_CHARACTER <= nCharset &&
+				   nCharset < KCHARSET_2BYTES_CHARACTER) {
+			fRetval	= (uCode < 0x100) ;
+		} else if (KCHARSET_2BYTES_CHARACTER <= nCharset &&
+				   nCharset < KCHARSET_3BYTES_CHARACTER) {
+			/*	󡢿ܤ 96^2 Υåʤܤ*/
+			if (fGenericp) {
+				fRetval	= (uCode < 0x10000) ;
+			} else {
+				register int	nLower	= (int)((uCode >> 0) & 0x00FF) ;
+				register int	nUpper	= (int)((uCode >> 8) & 0x00FF) ;
+				fRetval	= ((uCode < 0x10000) &&
+						   (0x21 <= nLower && nLower <= 0x7E) &&
+						   (0x21 <= nUpper && nUpper <= 0x7E)) ;
+			}
+		} else {
+			if (fGenericp) {
+				fRetval	= (uCode < 0x1000000) ;
+			} else {
+				register int	nLower	= (int)((uCode >>  0) & 0x00FF) ;
+				register int	nMiddle	= (int)((uCode >>  8) & 0x00FF) ;
+				register int	nUpper	= (int)((uCode >> 16) & 0x00FF) ;
+				fRetval	= ((uCode < 0x1000000) &&
+						   (0x21 <= nLower  && nLower  <= 0x7E) &&
+						   (0x21 <= nMiddle && nMiddle <= 0x7E) &&
+						   (0x21 <= nUpper  && nUpper  <= 0x7E)) ;
+			}
+		}
+	}
+	if (fRetval) {
+		pEntRetval	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_T) ;
+	} else {
+		pEntRetval	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_NIL) ;
+	}
+	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntRetval) ;
+	return	LMR_RETURN ;
+}
+
+
