@@ -25,6 +25,7 @@
 #include "lmstate.h"
 
 static	Boolean	lispMachine_makeLocalHook (TLispMachine*, TLispEntity*, TLispEntity*) ;
+static	Boolean	lispMachine_enumForceModeLineUpdate (TLispMachine*, TLispEntity*, void*, Boolean*) ;
 
 TLMRESULT
 lispMachineState_Nth (
@@ -430,3 +431,91 @@ lispMachineState_Documentation (
 	return	LMR_RETURN ;
 }
 
+TLMRESULT
+lispMachineState_ForceModeLineUpdate (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr ;
+	TLispEntity*			pEntArglist ;
+	TLispEntity*			pEntAll ;
+	TLispEntity*			pEntBuffer ;
+	register TLispEntity*	pEntT ;
+
+	assert (pLM != NULL) ;
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr != NULL) ;
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	assert (pEntArglist != NULL) ;
+	if (TFAILED (lispEntity_GetCar  (pLispMgr, pEntArglist, &pEntAll))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	if (TSUCCEEDED (lispEntity_Nullp (pLispMgr, pEntAll))) {
+		if (TFAILED (lispMachineCode_GetCurrentBuffer (pLM, &pEntBuffer))) {
+			lispMachineCode_SetError (pLM) ;
+			return	LMR_RETURN ;
+		}
+		lispBuffer_Desync (pLispMgr, pEntBuffer) ;
+	} else {
+		/*	ƤΥХåե sync 򳰤*/
+		lispMachine_EnumBuffer (pLM, False, lispMachine_enumForceModeLineUpdate, NULL) ;
+	}
+	pEntT	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_T) ;
+	lispMachineCode_SetLReg (pLM, LM_LREG_ACC, pEntT) ;
+	return	LMR_RETURN ;
+}
+
+Boolean
+lispMachine_enumForceModeLineUpdate (
+	register TLispMachine*	pLM,
+	register TLispEntity*	pEntBuffer,
+	register void*			pCaller,
+	register Boolean*		pfContinue)
+{
+	lispBuffer_Desync (pLM->m_pLispMgr, pEntBuffer) ;
+	return	True ;
+}
+
+#if 0
+/*	(remove-hook HOOK FUNCTION &optional LOCAL)
+ */
+TLMRESULT
+lispMachineState_RemoveHook (
+	register TLispMachine*	pLM)
+{
+	register TLispManager*	pLispMgr ;
+	TLispEntity*	pEntArglist ;
+	TLispEntity*	pEntHook ;
+	TLispEntity*	pEntFunction ;
+	TLispEntity*	pEntLocal ;
+
+	assert (pLM != NULL) ;
+	pLispMgr	= pLM->m_pLispMgr ;
+	assert (pLispMgr != NULL) ;
+
+	lispMachineCode_GetLReg (pLM, LM_LREG_ACC, &pEntArglist) ;
+	if (TFAILED (lispEntity_GetCar (pLispMgr, pEntArglist, &pEntHook)) ||
+		TFAILED (lispEntity_GetCdr (pLispMgr, pEntArglist, &pEntArglist)) ||
+		TFAILED (lispEntity_GetCar (pLispMgr, pEntArglist, &pEntFunction)) ||
+		TFAILED (lispEntity_GetCadr (pLispMgr, pEntArglist, &pEntLocal))) {
+		lispMachineCode_SetError (pLM) ;
+		return	LMR_RETURN ;
+	}
+	if (TFAILED (lispMachine_GetGlobalSymbolValue (pLM, pEntHook, &pEntHookValue)) ||
+		TSUCCEEDED (lispEntity_Voidp (pLispMgr, pEntHookValue))) 
+		goto	exit_func ;
+	if (TFAILED (lispMachine_GetCurrentSymbolValue (pLM, pEntHook, &pEntHookValue)) ||
+		TSUCCEEDED (lispEntity_Voidp (pLispMgr, pEntHookValue)))
+		goto	exit_func ;
+	if (TSUCCEEDED (lispEntity_Nullp (pLispMgr, pEntHookValue)) ||
+		TSUCCEEDED (lispEntity_Nullp (pLispMgr, pEntFunction))) 
+		goto	exit_func ;
+	if (TFAILED (lispEntity_Nullp (pLispMgr, pEntLocal)) ||
+		(TSUCCEEDED (lispMachine_LocalVariableIfSetp (pLM, pEntHook)) &&
+		 TSUCCEEDED (lispEntity_Consp (pLispMgr, pEntHookValue)) &&
+		 TFAILED (lispEntity_Memq (pLispMgr, pEntT, pEntHookValue, &pEntTmp)))) {
+	}
+}
+#endif
+
+
