@@ -22,6 +22,15 @@
 #include <stdio.h>
 #include <assert.h>
 #include "lmachinep.h"
+#include "lmkeymap.h"
+
+typedef struct {
+	TLispEntity*		m_pEntKey ;
+	TLispEntity*		m_pEntFunc ;
+	int					m_nMatch ;
+	Boolean				m_fNearMatch ;
+	Boolean				m_fAcceptDefault ;
+}	TWindowProcLookupKeymapArg ;
 
 static	TLMRESULT	lispMachineState_postPreCommandHook (TLispMachine*) ;
 static	TLMRESULT	lispMachineState_windowProcAfterCall(TLispMachine*) ;
@@ -29,6 +38,7 @@ static	TLMRESULT	lispMachineState_windowProcFinalize (TLispMachine*) ;
 static	TLMRESULT	lispMachineState_runCommandHook (TLispMachine*, TLispEntity*, TLMRESULT (*)(TLispMachine*)) ;
 static	TLMRESULT	lispMachineState_returnWithSignal (TLispMachine*) ;
 static	Boolean		lispMachine_windowProcLookupKeymap	(TLispMachine*, TLispEntity*, int*, TLispEntity**) ;
+static	Boolean		lispMachine_handleMinorModeMap	(TLispMachine*, TLispEntity*, void*, Boolean*) ;
 
 /*
  *	Lisp Machine ˤơCall-Interactively ƤӽФ Event Ԥ Loop
@@ -68,6 +78,10 @@ lispMachineState_WindowProc (
 	if (TFAILED (lispMachine_windowProcLookupKeymap (pLM, pEntKey, &nMatch, &pEntFunc)))
 		return	LMR_SUSPEND ;
 
+#if defined (DEBUG)
+	fprintf (stderr, "nmatch = %d\n", nMatch) ;
+#endif
+
 	/*	ҥåȤϤοӤơҥåȤϤοʤ
 	 *	С󥻥뤹롣*/
 	if (TSUCCEEDED (lispEntity_Nullp (pLispMgr, pEntFunc))) {
@@ -305,10 +319,10 @@ lispMachine_windowProcLookupKeymap (
 	register int*			pnMatch,
 	register TLispEntity**	ppEntFunc)
 {
-	register TLispManager*	pLispMgr ;
+	register TLispManager*		pLispMgr ;
+	TWindowProcLookupKeymapArg	arg ;
 	TLispEntity*		pCurBuffer ;
 	TLispEntity*		pEntKeymap ;
-	TLispEntity*		pEntGlobalMap ;
 	TLispEntity*		pEntFunc ;
 	int					nMatch ;
 	TLispEntity*		rEntKeymaps [2] ;
@@ -338,22 +352,32 @@ lispMachine_windowProcLookupKeymap (
 	} else {
 		rEntKeymaps [0]	= NULL ;
 	}
-
-	pEntKeymap		= NULL ;
-	pEntGlobalMap	= lispMgr_GetReservedEntity (pLispMgr, LISPMGR_INDEX_GLOBAL_MAP) ;
-	if (pEntGlobalMap != NULL)
-		lispMachine_GetCurrentSymbolValue (pLM, pEntGlobalMap, &pEntKeymap) ;
-	if (TFAILED (lispEntity_Voidp (pLispMgr, pEntKeymap))) {
-		rEntKeymaps [1]	= pEntKeymap ;
-	} else {
-		rEntKeymaps [1]	= NULL ;
-	}
+	rEntKeymaps [1]	= lispMachine_GetGlobalMap (pLM) ;
+	arg.m_pEntKey	= pEntKey ;
 
 	for (i = 0 ; i < NELEMENTS (srfLookGlobal) ; i ++) {
-		nMatch		= 0 ;
-		pEntKeymap	= rEntKeymaps [(int) srfLookGlobal [i]] ;
+		if (TFAILED (srfLookGlobal [i])) {
+			arg.m_pEntFunc			= NULL ;
+			arg.m_nMatch			= 0 ;
+			arg.m_fAcceptDefault	= srAcceptDefault [i] ;
+			arg.m_fNearMatch		= srNearMatch     [i] ;
+			if (TFAILED (lispMachine_EnumMinorModeMaps (pLM, lispMachine_handleMinorModeMap, &arg)))
+				return	False ;
+			if (arg.m_pEntFunc != NULL &&
+				TFAILED (lispEntity_Nullp (pLispMgr, arg.m_pEntFunc)) &&
+				arg.m_nMatch > 0) {
+				pEntFunc	= arg.m_pEntFunc ;
+				nMatch		= arg.m_nMatch ;
+				goto	key_hit ;
+			}
+			pEntKeymap	= rEntKeymaps [0] ;
+		} else {
+			pEntKeymap	= rEntKeymaps [1] ;
+		}
 		if (pEntKeymap == NULL)
 			continue ;
+
+		nMatch			= 0 ;
 		fAcceptDefault	= srAcceptDefault [i] ;
 		fNearMatch		= srNearMatch     [i] ;
 		fRetval			= lispKeymap_Lookup (pLispMgr, pEntKeymap, pEntKey, fAcceptDefault, fNearMatch, &nMatch, &pEntFunc) ;
@@ -373,3 +397,28 @@ lispMachine_windowProcLookupKeymap (
 	return	True ;
 }
 
+Boolean
+lispMachine_handleMinorModeMap (
+	register TLispMachine*	pLM,
+	register TLispEntity*	pEntKeymap,
+	register void*			pCaller,
+	register Boolean*		pfContinue)
+{
+	register TWindowProcLookupKeymapArg*	pArg	= (TWindowProcLookupKeymapArg*) pCaller ;
+	register TLispManager*	pLispMgr				= pLM->m_pLispMgr ;
+	register TLispEntity*	pEntKey ;
+	register Boolean		fNearMatch ;
+	register Boolean		fAcceptDefault ;
+	register Boolean		fRetval ;
+
+	pEntKey			= pArg->m_pEntKey ;
+	fAcceptDefault	= pArg->m_fAcceptDefault ;
+	fNearMatch		= pArg->m_fNearMatch ;
+	fRetval			= lispKeymap_Lookup (pLispMgr, pEntKeymap, pEntKey, fAcceptDefault, fNearMatch, &pArg->m_nMatch, &pArg->m_pEntFunc) ;
+	if (fRetval && 
+		TFAILED (lispEntity_Nullp (pLispMgr, pArg->m_pEntFunc)) &&
+		pArg->m_nMatch > 0) 
+		*pfContinue	= False ;
+	return	True ;
+}
+
