@@ -1,12 +1,32 @@
+/* # skkinput (Simple Kana-Kanji Input)
+ *
+ * This file is part of skkinput.
+ * Copyright (C) 2002
+ * Takashi SAKAMOTO (PXG01715@nifty.ne.jp)
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with skkinput; see the file COPYING.  If not, write to
+ * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
 #ifndef charset_h
 #define charset_h
 
 /*========================================================================*
- *	L萔B
+ *	
  *========================================================================*/
 
 /*
- *	WB
+ *	ʸ硣
  */
 enum {
 	KCHARSET_NOTHING = -1,
@@ -26,19 +46,19 @@ enum {
 	KCHARSET_JISX0201_1976,
 	
 	/*
-	 *	{(2oCgW)
+	 *	ܸ(2Хʸ)
 	 */
 	KCHARSET_JISX0208_1978,	KCHARSET_2BYTES_CHARACTER	= KCHARSET_JISX0208_1978,
 	KCHARSET_JISX0208_1983,
 	KCHARSET_JISX0212_1990,
 
 	/*
-	 *	(2oCgW)
+	 *	(2Хʸ)
 	 */
 	KCHARSET_GB2312_1980,
 	KCHARSET_GB12345_90,
 	/*
-	 *	(2oCgW)̕⏕W(supplementary character sets)
+	 *	(2Хʸ)ʸ(supplementary character sets)
 	 */
 	KCHARSET_GB7589_87,
 	KCHARSET_GB7590_87,
@@ -46,13 +66,13 @@ enum {
 	KCHARSET_GB13132_91,
 
 	/*
-	 * 	؍(2oCgW)
+	 * 	ڹ(2Хʸ)
 	 */
 	KCHARSET_KSC5601_1987,
 	KCHARSET_KSC5601_1992,
 
 	/*
-	 *	(3oCgW) 
+	 *	(3Хʸ) 
 	 */
 	KCHARSET_CNS11643_1992,	KCHARSET_3BYTES_CHARACTER	= KCHARSET_CNS11643_1992,
 
@@ -60,7 +80,7 @@ enum {
 } ;
 
 /*
- *	ȕWBWindow Event `B
+ *	üʸ硣Window Event 롣
  */
 enum {
 	KCHARSET_XKEYSYM	= MAX_CHARSET + 1,
