@@ -212,7 +212,7 @@ cmemcmp (
 		pRight	++ ;
 	}
 	if (nLeft < nRight)
-		return	*pRight ;
+		return	- *pRight ;
 	if (nLeft > nRight)
 		return	*pLeft ;		
 	return	0 ;
@@ -239,7 +239,7 @@ cmemccmp (
 		pRight	++ ;
 	}
 	if (nLeft < nRight)
-		return	Char_MakeAscii (*pRight) ;
+		return	- Char_MakeAscii (*pRight) ;
 	if (nLeft > nRight)
 		return	*pLeft ;		
 	return	0 ;
