@@ -22,8 +22,12 @@
 #include <assert.h>
 #include <unistd.h>
 #if !defined (__linux__)
+#if defined (__SVR4) && defined (__sun)
+#include <limits.h>
+#else
 #include <sys/syslimits.h>
 #endif
+#endif
 #include <sys/param.h>
 #include <pwd.h>
 #include <sys/types.h>
@@ -80,21 +84,23 @@ ExpandFileName (
 	char	achPath [PATH_MAX + 1] ;
 	char	achLoginName [MAXLOGNAME + 1] ;
 
-	if (*pPath == '.') {
-		if (pDefaultPath != NULL) {
-			if (TFAILED (ExpandFileName (pvbuf, pDefaultPath, nDefaultPath, NULL, 0)))
-				return	False ;
-		} else {
-			int		len ;
-			if (getcwd (achPath, NELEMENTS (achPath) - 1) == NULL)
-				return	False ;
-			achPath [NELEMENTS (achPath) - 1]	= '\0' ;
-			len		= strlen (achPath) ;
-			if (TFAILED (TVarbuffer_Require (pvbuf, len)))
-				return	False ;
-			strtocstr (TVarbuffer_GetBuffer (pvbuf), achPath, len) ;
-		}
+	if (pDefaultPath != NULL) {
+		if (TFAILED (ExpandFileName (pvbuf, pDefaultPath, nDefaultPath, NULL, 0)))
+			return	False ;
+		/* [̤]*** Ǹ夬/ǤʤСɲäʤФʤʤ*** */
 	}
+	
+ 	if (*pPath == '.') {
+		int		len ;
+		if (getcwd (achPath, NELEMENTS (achPath) - 1) == NULL)
+			return	False ;
+		achPath [NELEMENTS (achPath) - 1]	= '\0' ;
+		len		= strlen (achPath) ;
+		if (TFAILED (TVarbuffer_Require (pvbuf, len)))
+			return	False ;
+		strtocstr (TVarbuffer_GetBuffer (pvbuf), achPath, len) ;
+ 	}
+
 	if (*pPath == '~') {
 		register const Char*	ptr ;
 		register int			nptr ;
@@ -178,4 +184,36 @@ GetFileNameNondirectory (
 	return	True ;
 }
 
+Boolean
+GetFileDirectoryAnsi (
+	register const char*	pPath,
+	register int			nPath,
+	register const char**	ppDirOut,
+	register int*			pnDirOut)
+{
+	register const char*	ptr ;
+	register const char*	pLastPos	= NULL ;
+	register int			nptr ;
+
+	assert (pPath != NULL || nPath <= 0) ;
+	assert (ppDirOut != NULL) ;
+	assert (pnDirOut != NULL) ;
+			
+	ptr		= pPath ;
+	nptr	= nPath ;
+	while (nptr > 0) {
+		if (*ptr == '/') 
+			pLastPos	= ptr ;
+		ptr		++ ;
+		nptr	-- ;
+	}
+	if (pLastPos == NULL) {
+		*ppDirOut	= "" ;
+		*pnDirOut	= 0 ;
+	} else {
+		*ppDirOut	= pPath ;
+		*pnDirOut	= pLastPos - pPath ;
+	}
+	return	True ;
+}
 
