@@ -562,7 +562,7 @@ kinputClient_onFocusKeyPress (
 {
 	register KinputClient*	pClient	= pClosure ;
 
-#if defined (DEBUG) 
+#if defined (DEBUG) || 0
 	fprintf (stderr, "kinputClient_onFocusKeyPress(%d)\n", pEvent->type) ;
 #endif
 	if (pEvent->type != KeyPress)
@@ -595,7 +595,7 @@ kinputClient_onKeyPress (
 {
 	register KinputClient*		pClient	= (KinputClient *) closure ;
 
-#if defined (DEBUG) 
+#if defined (DEBUG) || 0
 	{
 		XEvent*	pEvent	= (XEvent*) call_data ;
 		fprintf (stderr, "Event: type(%d), window(%lx)",
@@ -660,6 +660,7 @@ kinputClient_createFrame (
 	XtSetArg (rArg [nArg], XtNmappedWhenManaged,	False) ;		nArg ++ ;
 	XtSetArg (rArg [nArg], XtNclientWindow,			wndClient) ;	nArg ++ ;
 	XtSetArg (rArg [nArg], XtNfocusWindow,			wndFocus) ;		nArg ++ ;
+	XtSetArg (rArg [nArg], XtNinput,				False) ;		nArg ++ ;
 
 	wgTop	= AfxGetToplevelWidget () ;
 	switch (pAttribute->m_nInputStyle) {
@@ -745,10 +746,6 @@ kinputClient_onKeyNotify (
 	unsigned int			uKeyCode, uState, uMask ;
 	XEvent					ev ;
 
-#if defined (DEBUG) 
-	fprintf (stderr, "kinputClient_onKeyNotify (%p, %lx)\n", pvClient, cc) ;
-#endif
-
 	wndFocus	= (pClient->m_wndFocus != None)? pClient->m_wndFocus : pClient->m_wndRequestor ;
 	if (!pEv->m_fChar) {
 		register XEvent*	pXEv	= pEv->u.m_pEv ;
@@ -837,17 +834,6 @@ kinputClient_onTextNotify (
 		}
 		pCH	++ ;
 	}
-#if defined (DEBUG)
-	ptr	= pPropvalue ;
-	for (i = 0 ; i < nctext ; i ++) {
-		if (*ptr < 0x20) {
-			fprintf (stderr, "0x%02x, ", *ptr) ;
-		} else {
-			fprintf (stderr, "0x%02x(%c), ", *ptr, *ptr) ;
-		}
-		ptr	++ ;
-	}
-#endif
 	KinputServer_FixText (pClient->m_wgProtocol, pClient, pctext, nctext) ;
 	FREE (pctext) ;
 	return ;
