@@ -444,7 +444,7 @@ kinputServer_acceptConversion (
 	XEvent				ev ;
 
 #if defined (DEBUG)
-	fprintf (stderr, "Requestor(%x), Selection(%x), Target(%x), Property(%x), Client(%x)\n",
+	fprintf (stderr, "Requestor(%lx), Selection(%lx), Target(%lx), Property(%lx), Client(%lx)\n",
 			 KinputClient_GetRequestor (pClient),
 			 KinputClient_GetSelection (pClient),
 			 KinputClient_GetTarget (pClient),
