@@ -37,9 +37,6 @@ static	XtResource	srTFrameResource []	= {
 	{	XtNforeground,				XtCForeground,
 		XtRPixel,					sizeof (Pixel),
 		offset (m_pxlForeground),	XtRString,			XtDefaultForeground, },
-	{	XtNinput,					XtCInput,
-		XtRBoolean,					sizeof (Boolean),
-		offset (m_fInput),			XtRImmediate,		(XtPointer) True, },
 	{	XtNkeyPressCallback,		XtCCallback,
 		XtRCallback,				sizeof (XtCallbackList),
 		offset (m_lstCbkKeyPress),	XtRCallback,		(XtPointer) NULL, },
