@@ -183,9 +183,9 @@ tnormalFrame_onRealize (
 	 *	 Window ˤʤ롣
 	 */
 	nArg	= 0 ;
-	XtSetArg (rArg [nArg], XtNwidth,	gw->core.width) ;			nArg ++ ;
-	XtSetArg (rArg [nArg], XtNheight,	gw->core.height) ;			nArg ++ ;
-	XtSetArg (rArg [nArg], XtNinput,	wgThis->tframe.m_fInput) ;	nArg ++ ;
+	XtSetArg (rArg [nArg], XtNwidth,	gw->core.width) ;		nArg ++ ;
+	XtSetArg (rArg [nArg], XtNheight,	gw->core.height) ;		nArg ++ ;
+	XtSetArg (rArg [nArg], XtNinput,	True) ;					nArg ++ ;
 	/*XtSetArg (rArg [nArg], XtNmappedWhenManaged, False) ;		nArg ++ ;*/
 	assert (nArg < (sizeof (rArg) / sizeof (rArg [0]))) ;
 #if defined (DEBUG)
