@@ -287,12 +287,15 @@ toffTheSpotWindow_onRealize (
 		SET_CONVATTR_MASK (wgThis->toffwin.m_caCurrent, CAPreeditMask) ;
 		SET_CONVATTR_PREEDIT_MASK (wgThis->toffwin.m_caCurrent, CAClientArea) ;
 		XGetRectangleOfWindow (pDisplay, wgThis->toffwin.m_wndFocus, &rect) ;
+		rect.x = rect.y = 0 ;
 		CONVATTR_PREEDIT_CLIENTAREA (wgThis->toffwin.m_caCurrent)	= rect ;
 	}
 	/*	FocusWindow  Resize ƻ뤹롣 Preedit  CAClientArea 
 	 *	ꤷʤ Client кǤ롣*/
 	AfxRegisterWindow (pDisplay, wgThis->toffwin.m_wndFocus, ConfigureNotify, StructureNotifyMask, gw, toffTheSpotWindow_onFocusResize) ;
 
+	toffTheSpotWindow_configureFocusOffset (gw) ;
+
 	if (!toffTheSpotWindow_createTerminalWindow (gw) ||
 		!toffTheSpotWindow_createModeshellWindow (gw)) {
 		XtDestroyWidget (gw) ;
@@ -301,7 +304,6 @@ toffTheSpotWindow_onRealize (
 
 	/*	Modeshell 礭ɽʸԤ
 	 */
-	toffTheSpotWindow_configureFocusOffset (gw) ;
 	toffTheSpotWindow_resizeModeshellWindow (gw) ;
 
 	/*	Minibuffer  Realize ˤĤƤϤޤȤ⡣󤷤ˤ롣
@@ -616,7 +618,9 @@ toffTheSpotWindow_setattribute (
 		toffTheSpotWindow_onSetStatusAttribute,
 	} ;
 	register TOffTheSpotWindowWidget	wgThis	= (TOffTheSpotWindowWidget) gw ;
-	register int			i ;
+	register TConversionAttribute*	pCurAttr ;
+	register XRectangle*			pCurArea ;
+	register int			i, nOldX, nOldY, nX, nY ;
 	register unsigned int	uMask ;
 	register Boolean		fReconf ;
 	
@@ -626,8 +630,18 @@ toffTheSpotWindow_setattribute (
 	if (wgThis->toffwin.m_fDestroy)
 		return ;
 
-	uMask	= 1 << CAFocusWindowBit ;
-	fReconf	= False ;
+	pCurAttr	= &wgThis->toffwin.m_caCurrent ;
+	pCurArea	= &CONVATTR_PREEDIT_CLIENTAREA (*pCurAttr) ;
+	nOldX		= wgThis->toffwin.m_ptFocus.x + pCurArea->x ;
+	nOldY		= wgThis->toffwin.m_ptFocus.y + pCurArea->y ;
+	if (IS_CONVATTR_MASK (*pCurAttr, CAPreeditMask) &&
+		IS_CONVATTR_PREEDIT_MASK (*pCurAttr, CAClientArea)) {
+		nOldX	+= pCurArea->x ;
+		nOldY	+= pCurArea->y ;
+	}
+
+	uMask		= 1 << CAFocusWindowBit ;
+	fReconf		= False ;
 	for (i = 0 ; i <= (CAStatusMaskBit - CAFocusWindowBit) ; i ++) {
 		if (IS_CONVATTR_MASK (*pAttribute, uMask)) 
 			fReconf	= (rpSetAttrProcTbl [i])(gw, pAttribute) | fReconf ;
@@ -636,6 +650,24 @@ toffTheSpotWindow_setattribute (
 	if (XtIsRealized (gw) && fReconf) {
 		/*	ľ*/
 		toffTheSpotWindow_configureFocusOffset (gw) ;
+
+		nX	= wgThis->toffwin.m_ptFocus.x ;
+		nY	= wgThis->toffwin.m_ptFocus.y ;
+		if (IS_CONVATTR_MASK (*pCurAttr, CAPreeditMask) &&
+			IS_CONVATTR_PREEDIT_MASK (*pCurAttr, CAClientArea)) {
+			nX	+= pCurArea->x ;
+			nY	+= pCurArea->y ;
+		}
+		if (nX != nOldX || nY != nOldY) {
+#if defined (DEBUG) || 0
+			fprintf (stderr, "terminal move (x:%d, y:%d) -> (x:%d, y:%d / focus(%lx):x:%d, y:%d, Area:(%d, %d))\n",
+					 nOldX, nOldY, nX, nY,
+					 (unsigned long)wgThis->toffwin.m_wndFocus,
+					 wgThis->toffwin.m_ptFocus.x, wgThis->toffwin.m_ptFocus.y,
+					 pCurArea->x, pCurArea->y) ;
+#endif
+			XtMoveWidget (wgThis->toffwin.m_wgTerminal, nX, nY) ;
+		}
 	}
 	return ;
 }
@@ -656,7 +688,7 @@ toffTheSpotWindow_onSetAttributeFocusWindow (
 
 	wndNewFocus	= CONVATTR_FOCUSWINDOW (*pAttribute) ; 
 	pDisplay	= XtDisplay (gw) ;
-#if defined (DEBUG)
+#if defined (DEBUG) || 0
 	fprintf (stderr, "FocusWindow: (%lx)\n", wndNewFocus) ;
 #endif
 	if (!XtIsRealized (gw)) {
@@ -742,7 +774,7 @@ toffTheSpotWindow_onSetPAttributeClientArea (
 	assert (gw != NULL) ;
 	assert (pAttribute != NULL) ;
 
-#if defined (DEBUG)
+#if defined (DEBUG) || 0
 	fprintf (stderr, "ClientArea: (%d, %d, %d, %d)\n", 
 			 pAttribute->m_Preedit.m_area.x,
 			 pAttribute->m_Preedit.m_area.y,
@@ -1018,7 +1050,7 @@ toffTheSpotWindow_onSetSAttributeClientArea (
 	assert (gw != NULL) ;
 	assert (pAttribute != NULL) ;
 
-#if defined (DEBUG)
+#if defined (DEBUG) || 0
 	fprintf (stderr, "Status: ClientArea: (%d, %d, %d, %d)\n", 
 			 pAttribute->m_Status.m_area.x,
 			 pAttribute->m_Status.m_area.y,
@@ -1351,8 +1383,8 @@ toffTheSpotWindow_configureFocusOffset (
 		fRedraw	= True ;
 	}
 	if (wgThis->toffwin.m_ptFocus.x != nFocusX || wgThis->toffwin.m_ptFocus.y != nFocusY) {
-		wgThis->toffwin.m_ptFocus.x	= x ;
-		wgThis->toffwin.m_ptFocus.y	= y ;
+		wgThis->toffwin.m_ptFocus.x	= nFocusX ;
+		wgThis->toffwin.m_ptFocus.y	= nFocusY ;
 		fRedraw	= True ;
 	}
 	return	fRedraw ;
@@ -1454,6 +1486,7 @@ toffTheSpotWindow_createTerminalWindow (
 	Boolean								fInput ;
 	Arg									rArg [12] ;
 	register int						nArg ;
+	register int						nX, nY ;
 
 	nArg	= 0 ;
 	XtSetArg (rArg [nArg],	XtNinput,			&fInput) ;	nArg ++ ;
@@ -1462,7 +1495,7 @@ toffTheSpotWindow_createTerminalWindow (
 	nArg	= 0 ;
 	XtSetArg (rArg [nArg], XtNmappedWhenManaged, False) ;	nArg ++ ;
 	XtSetArg (rArg [nArg], XtNclientWindow,		wgThis->toffwin.m_wndClient) ;	nArg ++ ;
-	XtSetArg (rArg [nArg], XtNfocusWindow,		wgThis->toffwin.m_wndFocus) ;	nArg ++ ;
+	XtSetArg (rArg [nArg], XtNfocusWindow,		None) ;		nArg ++ ;
 	XtSetArg (rArg [nArg], XtNinput,			fInput) ;	nArg ++ ;	
 
 	pCurAttr	= &wgThis->toffwin.m_caCurrent ;
@@ -1472,8 +1505,17 @@ toffTheSpotWindow_createTerminalWindow (
 	//wgThis->toffwin.m_wndClient == wgThis->toffwin.m_wndFocus
 
 	pRect	= &pCurAttr->m_Preedit.m_area ;
-	XtSetArg (rArg [nArg], XtNx,		pRect->x) ;			nArg ++ ;
-	XtSetArg (rArg [nArg], XtNy,		pRect->y) ;			nArg ++ ;
+	nX		= pRect->x + wgThis->toffwin.m_ptFocus.x ;
+	nY		= pRect->y + wgThis->toffwin.m_ptFocus.y ;
+#if defined (DEBUG) || 0
+	fprintf (stderr, "offthespotwindow: (focus:%lx, client:%lx, x:%d(%d), y:%d(%d), %d x %d)\n",
+			 (unsigned long) wgThis->toffwin.m_wndFocus, 
+			 (unsigned long) wgThis->toffwin.m_wndClient,
+			 nX, pRect->x, nY, pRect->y, 
+			 pRect->width, pRect->height) ;
+#endif
+	XtSetArg (rArg [nArg], XtNx,		nX) ;				nArg ++ ;
+	XtSetArg (rArg [nArg], XtNy,		nY) ;				nArg ++ ;
 	XtSetArg (rArg [nArg], XtNwidth,	pRect->width) ;		nArg ++ ;
 	XtSetArg (rArg [nArg], XtNheight,	pRect->height) ;	nArg ++ ;
 	if (IS_CONVATTR_STATUS_MASK (*pCurAttr, CAForegroundPixel)) {
@@ -1514,13 +1556,10 @@ toffTheSpotWindow_createModeshellWindow (
 
 	/*	fixed modeshell */
 	wndFocus	= wgThis->toffwin.m_wndClient ;
-	x		= wgThis->toffwin.m_caCurrent.m_Status.m_area.x ;
-	y		= wgThis->toffwin.m_caCurrent.m_Status.m_area.y ;
-	lMask	= toffTheSpotWindow_getStatusWindowAttribute (gw, &swa) ;
+	lMask		= toffTheSpotWindow_getStatusWindowAttribute (gw, &swa) ;
+	x			= wgThis->toffwin.m_caCurrent.m_Status.m_area.x ;
+	y			= wgThis->toffwin.m_caCurrent.m_Status.m_area.y ;
 
-	/*	Focus Window Žդ Modeshell 롣Ĥ terminal 
-	 *	Ƥ뤬ɤΤ ʤ overspec ʵ뤬ġ
-	 */
 	wndModeshell	= XCreateWindow (pDisplay, wndFocus, x, y, 1, 1, 0, CopyFromParent, CopyFromParent, CopyFromParent, lMask, &swa) ;
 	if (wndModeshell == None) 
 		return	False ;
