@@ -323,7 +323,7 @@ toverTheSpotWindow_onRealize (
 	/*	FocusWindow  Resize ƻ뤹롣 Preedit  CAClientArea 
 	 *	ꤷʤ Client кǤ롣*/
 	AfxRegisterWindow (pDisplay, wgThis->toverwin.m_wndFocus, ConfigureNotify, StructureNotifyMask, gw, toverTheSpotWindow_onFocusResize) ;
-
+	
 	if (!toverTheSpotWindow_createTerminalWindow (gw) ||
 		!toverTheSpotWindow_createModeshellWindow (gw)) {
 		XtDestroyWidget (gw) ;
@@ -435,6 +435,9 @@ toverTheSpotWindow_onKeyPress (
 {
 	if (!XtIsRealized (gw))
 		return ;
+#if defined (DEBUG)
+	fprintf (stderr, "toverTheSpotWindow_onKeyPress(%p)\n", gw) ;
+#endif
 	/*	Event Select: None ȡ˸ä KeyEvent ꤲǽ
 	 *	롣*/
 	XtCallCallbacks (gw, XtNkeyPressCallback, (XtPointer) pEvent) ;
@@ -830,6 +833,7 @@ toverTheSpotWindow_onSetAttributeFocusWindow (
 	register TConversionAttribute*	pCurAttr ;
 	register Window					wndNewFocus, wndOldFocus ;
 	register int					i ;
+	register XRectangle*			pArea ;
 	
 	assert (gw != NULL) ;
 	assert (pAttribute != NULL) ;
@@ -853,14 +857,7 @@ toverTheSpotWindow_onSetAttributeFocusWindow (
 
 	CONVATTR_FOCUSWINDOW (wgThis->toverwin.m_caCurrent) = wndNewFocus ;
 	wgThis->toverwin.m_wndFocus	= wndNewFocus ;
-
-	pTerminal	= &wgThis->toverwin.m_Terminal ;
-	for (i = 0 ; i < 3 ; i ++) {
-		if (pTerminal->m_wndTerminal [i] != None)
-			XReparentWindow (pDisplay, pTerminal->m_wndTerminal [i], wndNewFocus, 0, 0) ;
-	}
-	if (wgThis->toverwin.m_wndModeshell != None)
-		XReparentWindow (pDisplay, wgThis->toverwin.m_wndModeshell, wndNewFocus, 0, 0) ;
+	toverTheSpotWindow_configureFocusOffset (gw) ;
 
 	/*	Realize Ƥ֤Ǥ Focus Window ѹ CAClientArea 
 	 *	ǤȽǤ롣
@@ -868,7 +865,9 @@ toverTheSpotWindow_onSetAttributeFocusWindow (
 	pCurAttr	= &wgThis->toverwin.m_caCurrent ;
 	SET_CONVATTR_MASK (*pCurAttr, CAPreeditMask) ;
 	SET_CONVATTR_PREEDIT_MASK (*pCurAttr, CAClientArea) ;
-	XGetRectangleOfWindow (pDisplay, wndNewFocus, &CONVATTR_PREEDIT_CLIENTAREA (*pCurAttr)) ;
+	pArea		= &CONVATTR_PREEDIT_CLIENTAREA (*pCurAttr) ;
+	XGetRectangleOfWindow (pDisplay, wndNewFocus, pArea) ;
+	pArea->x	= pArea->y	= 0 ;
 
 	/*	Focus Window  Resize δƻ³롣*/
 	if (wndOldFocus != None)
@@ -942,13 +941,16 @@ toverTheSpotWindow_onSetPAttributeSpotLocation (
 		register int		nAscent, nDescent, nFontHeight ;
 		register Boolean	fLineSpacing ;
 		int					nLineSpacing ;
+		register int		nFocusX, nFocusY ;
 
 		fLineSpacing	= toverTheSpotWindow_getPreeditLineSpacing (gw, &nLineSpacing) ;
 		nDescent		= TFontSet_GetDescent (pFontSet) ;
 		nAscent			= TFontSet_GetAscent  (pFontSet) ;
 		nFontHeight		= TFontSet_GetHeight  (pFontSet) ;
+		nFocusX			= wgThis->toverwin.m_ptFocus.x ;
+		nFocusY			= wgThis->toverwin.m_ptFocus.y ;
 		y	+= (fLineSpacing && nFontHeight < nLineSpacing)? (nLineSpacing - nAscent) : nDescent ;
-		XMoveWindow (XtDisplay (gw), wgThis->toverwin.m_wndModeshell, x, y) ;
+		XMoveWindow (XtDisplay (gw), wgThis->toverwin.m_wndModeshell, nFocusX + x, nFocusY + y) ;
 	}
 	return	fMove && (nControl > 0 && nText > 0) ;
 }
@@ -1668,14 +1670,14 @@ toverTheSpotWindow_configureTerminalWindow (
 	register TOverTheSpotTerminal*				pTerminal ;
 	register TOverTheSpotTerminalTextControl*	pControl ;
 	register TOverTheSpotTerminalTextControl*	pControlHead ;
-	register Display*				pDisplay ;
-	register const Char*			pText ;
-	register int					nControl, nWindow, nMaxWindow, nClientAreaX, nClientAreaY ;
-	register int					nX, nY, nMasterClientAreaWidth, nClientAreaWidth ;
-	register TFontSet*				pFontSet ;
-	int								rnLineCount [3], nLineSpacing ;
-	TSize							sz ;
-	register Boolean					fEmpty, fLineSpacing ;
+	register Display*		pDisplay ;
+	register const Char*	pText ;
+	register int			nControl, nWindow, nMaxWindow, nClientAreaX, nClientAreaY ;
+	register int			nX, nY, nMasterClientAreaWidth, nClientAreaWidth, nFocusX, nFocusY ;
+	register TFontSet*		pFontSet ;
+	int						rnLineCount [3], nLineSpacing ;
+	TSize					sz ;
+	register Boolean		fEmpty, fLineSpacing ;
 
 	pDisplay		= XtDisplay (gw) ;
 	pTerminal		= &wgThis->toverwin.m_Terminal ;
@@ -1700,20 +1702,17 @@ toverTheSpotWindow_configureTerminalWindow (
 	if (IS_CONVATTR_MASK (wgThis->toverwin.m_caCurrent, CAPreeditMask) &&
 		IS_CONVATTR_PREEDIT_MASK (wgThis->toverwin.m_caCurrent, CAClientArea)) {
 		nMasterClientAreaWidth	= wgThis->toverwin.m_caCurrent.m_Preedit.m_area.width ;
-#if 0
 		nClientAreaX			= wgThis->toverwin.m_caCurrent.m_Preedit.m_area.x ;
 		nClientAreaY			= wgThis->toverwin.m_caCurrent.m_Preedit.m_area.y ;
-#else
-		nClientAreaX			= 0 ;
-		nClientAreaY			= 0 ;
-#endif
 	} else {
 		nClientAreaX			= 0 ;
 		nClientAreaY			= 0 ;
 		nMasterClientAreaWidth	= 65535 ;
 	}
-	nX					= wgThis->toverwin.m_ptSpot.x + nClientAreaX ;
-	nY					= wgThis->toverwin.m_ptSpot.y + nClientAreaY - TFontSet_GetAscent (pFontSet) ;
+	nFocusX		= wgThis->toverwin.m_ptFocus.x ;
+	nFocusY		= wgThis->toverwin.m_ptFocus.y ;
+	nX			= wgThis->toverwin.m_ptSpot.x ;
+	nY			= wgThis->toverwin.m_ptSpot.y - TFontSet_GetAscent (pFontSet) ;
 #if defined (DEBUG) || 0
 	fprintf (stderr, "Spot:(%d, %d) Client:(%d, %d, %d, %d), line=%d\n",
 			 wgThis->toverwin.m_ptSpot.x,
@@ -1769,7 +1768,7 @@ toverTheSpotWindow_configureTerminalWindow (
 #endif
 		if (pTerminal->m_wndTerminal [nWindow] != None) {
 			if (nWidth > 0 && nHeight > 0) {
-				XMoveResizeWindow (pDisplay, pTerminal->m_wndTerminal [nWindow], nX, nY, nWidth, nHeight) ;
+				XMoveResizeWindow (pDisplay, pTerminal->m_wndTerminal [nWindow], nFocusX + nX, nFocusY + nY, nWidth, nHeight) ;
 				XMapRaised (pDisplay, pTerminal->m_wndTerminal [nWindow]) ;
 				(srOverTheSpotTerminalExposeProc [nWindow])(gw, 0) ;
 //				XClearArea (pDisplay, pTerminal->m_wndTerminal [nWindow], 0, 0, 0, 0, True) ;
@@ -1794,7 +1793,7 @@ toverTheSpotWindow_configureTerminalWindow (
 		wgThis->toverwin.m_fFloatingModeshell) {
 
 		if (nMaxWindow <= 1) {
-			nX	=  wgThis->toverwin.m_ptSpot.x + nClientAreaX ;
+			nX	=  wgThis->toverwin.m_ptSpot.x ;
 			if (fEmpty) {
 				register int	nFontHeight	= TFontSet_GetHeight (pFontSet) ;
 				nY	+= (fLineSpacing && nFontHeight < nLineSpacing)? nLineSpacing : nFontHeight ;
@@ -1805,7 +1804,7 @@ toverTheSpotWindow_configureTerminalWindow (
 #if defined (DEBUG)
 		fprintf (stderr, "spot(x,y,ls,empty)=(%d,%d,%d,%d)\n", nX, nY, nLineSpacing, fEmpty) ;
 #endif
-		XMoveWindow (pDisplay, wgThis->toverwin.m_wndModeshell, nX, nY) ;
+		XMoveWindow (pDisplay, wgThis->toverwin.m_wndModeshell, nFocusX + nX, nFocusY + nY) ;
 	}
 	return ;
 }
@@ -1968,8 +1967,8 @@ toverTheSpotWindow_configureFocusOffset (
 		fRedraw	= True ;
 	}
 	if (wgThis->toverwin.m_ptFocus.x != nFocusX || wgThis->toverwin.m_ptFocus.y != nFocusY) {
-		wgThis->toverwin.m_ptFocus.x	= x ;
-		wgThis->toverwin.m_ptFocus.y	= y ;
+		wgThis->toverwin.m_ptFocus.x	= nFocusX ;
+		wgThis->toverwin.m_ptFocus.y	= nFocusY ;
 		fRedraw	= True ;
 	}
 	return	fRedraw ;
@@ -2139,7 +2138,7 @@ toverTheSpotWindow_createTerminalWindow (
 	XSetWindowAttributes			swa ;
 
 	lMask		= toverTheSpotWindow_getPreeditWindowAttribute (gw, &swa) ;
-	wndFocus	= wgThis->toverwin.m_wndFocus ;
+	wndFocus	= wgThis->toverwin.m_wndClient ;
 	pTerminal	= &wgThis->toverwin.m_Terminal ;
 	for (i = 0 ; i < 3 ; i ++) {
 		wndTerminal	= XCreateWindow (pDisplay, wndFocus, 0, 0, 1, 1, 0, CopyFromParent, CopyFromParent, CopyFromParent, lMask, &swa) ;
@@ -2173,16 +2172,15 @@ toverTheSpotWindow_createModeshellWindow (
 	register unsigned long	lMask ;
 	XSetWindowAttributes	swa ;
 
+	wndFocus	= wgThis->toverwin.m_wndClient ;
 	if (!IS_CONVATTR_MASK (wgThis->toverwin.m_caCurrent, CAStatusMask) ||
 		!IS_CONVATTR_STATUS_MASK (wgThis->toverwin.m_caCurrent, CAClientArea) ||
 		wgThis->toverwin.m_wndClient == wgThis->toverwin.m_wndFocus) {
 		/*	floating modeshell */
-		wndFocus	= wgThis->toverwin.m_wndFocus ;
 		wgThis->toverwin.m_fFloatingModeshell	= True ;
 		x	=	y	= 0 ;
 	} else {
 		/*	fixed modeshell */
-		wndFocus	= wgThis->toverwin.m_wndClient ;
 		wgThis->toverwin.m_fFloatingModeshell	= False ;
 		x	= wgThis->toverwin.m_caCurrent.m_Status.m_area.x ;
 		y	= wgThis->toverwin.m_caCurrent.m_Status.m_area.y ;
@@ -2190,11 +2188,6 @@ toverTheSpotWindow_createModeshellWindow (
 
 	if (wgThis->toverwin.m_fFloatingModeshell) {
 		lMask	= toverTheSpotWindow_getPreeditWindowAttribute (gw, &swa) ;
-#if 0
-		lMask	|= CWSaveUnder | CWBackingStore ;
-		swa.save_under		= True ;
-		swa.backing_store	= Always ;
-#endif
 	} else {
 		lMask	= toverTheSpotWindow_getStatusWindowAttribute (gw, &swa) ;
 	}
@@ -2237,7 +2230,7 @@ toverTheSpotWindow_createMinibufferFrame (
 	nArg	= 0 ;
 	XtSetArg (rArg [nArg], XtNmappedWhenManaged, False) ;	nArg ++ ;
 	XtSetArg (rArg [nArg], XtNclientWindow,		wgThis->toverwin.m_wndClient) ;	nArg ++ ;
-	XtSetArg (rArg [nArg], XtNfocusWindow,		wgThis->toverwin.m_wndFocus) ;	nArg ++ ;
+	XtSetArg (rArg [nArg], XtNfocusWindow,		None) ;		nArg ++ ;
 	XtSetArg (rArg [nArg], XtNinput,			fInput) ;	nArg ++ ;	
 
 	pCurAttr	= &wgThis->toverwin.m_caCurrent ;
