@@ -84,11 +84,10 @@ static	Boolean	trootWindow_autoPopupp		(Widget) ;
 static	void	trootWindow_popup			(Widget) ;
 static	void	trootWindow_popdown			(Widget) ;
 
-static	void	trootWindow_onTerminalKeyPress	(Widget, XtPointer, XEvent*, Boolean*) ;
 static	void	trootWindow_onTerminalConfigure	(Widget, XtPointer, XEvent*, Boolean*) ;
 
 static	void	trootWindow_adjustPopupWindow	(Widget) ;
-
+static	Window	trootWindow_getToplevelWindow	(Display*, Window) ;
 
 static	XtActionsRec	srRootWindowAction []	= {
 	{ "onKeyPress",		trootWindow_onKeyPress, },
@@ -192,32 +191,35 @@ trootWindow_onRealize (
 	register XtValueMask*			pValueMask,
 	register XSetWindowAttributes*	pXSWA)
 {
-	TRootWindowWidget	wgThis	= (TRootWindowWidget) gw ;
-	CoreWidgetClass		super	= (CoreWidgetClass) XtClass (gw)->core_class.superclass ;
+	TRootWindowWidget	wgThis		= (TRootWindowWidget) gw ;
+	register Display*	pDisplay	= XtDisplay (gw) ;
+	CoreWidgetClass		super		= (CoreWidgetClass) XtClass (gw)->core_class.superclass ;
 	Widget				wgPopup ;
 	Widget				wgTerminal ;
 	Arg					rArg [8] ;
 	int					nArg ;
+	register Window		wndToplevel ;
 
 	(*super->core_class.realize) (gw, pValueMask, pXSWA) ;
 
+	wndToplevel	= trootWindow_getToplevelWindow (pDisplay, wgThis->trootwin.m_wndClient) ;
+	XSetTransientForHint (pDisplay, XtWindow (gw), wndToplevel) ;
+
 	/*	Popup Widget 롣 Popup Widget Τ RootWindowStyle
 	 *	 Window ˤʤ롣
 	 */
 	nArg	= 0 ;
 	XtSetArg (rArg [nArg], XtNwidth,	gw->core.width) ;			nArg ++ ;
 	XtSetArg (rArg [nArg], XtNheight,	gw->core.height) ;			nArg ++ ;
-	XtSetArg (rArg [nArg], XtNinput,	wgThis->tframe.m_fInput) ;	nArg ++ ;
-#if 0
-	XtSetArg (rArg [nArg], XtNinput,	True) ;						nArg ++ ;
-#endif
+	XtSetArg (rArg [nArg], XtNinput,	False) ;					nArg ++ ;
+
 	/*XtSetArg (rArg [nArg], XtNmappedWhenManaged,	False) ;		nArg ++ ;*/
 	assert (nArg < (sizeof (rArg) / sizeof (rArg [0]))) ;
 	wgPopup	= XtCreatePopupShell ("RootWindowStyleFrame", transientShellWidgetClass, gw, rArg, nArg) ;
 
 	/*	Terminal Window 롣*/
 	nArg	= 0 ;
-	XtSetArg (rArg [nArg], XtNinput, True) ; nArg ++ ;
+	XtSetArg (rArg [nArg], XtNinput, False) ; nArg ++ ;
 	assert (nArg < (sizeof (rArg) / sizeof (rArg [0]))) ;
 	wgTerminal	= XtCreateManagedWidget ("RootWindowTerminal", tterminalWidgetClass, wgPopup, rArg, nArg) ;
 
@@ -227,8 +229,6 @@ trootWindow_onRealize (
 	XtRealizeWidget (wgPopup) ;
 
 	trootWindow_adjustPopupWindow (gw) ;
-
-	XtAddEventHandler (wgTerminal, KeyPressMask, False, trootWindow_onTerminalKeyPress, gw) ;
 	XtAddEventHandler (wgPopup,    StructureNotifyMask, False, trootWindow_onTerminalConfigure, gw) ;
 	return ;
 }
@@ -251,12 +251,16 @@ trootWindow_onDestroy (
 	TRootWindowWidget	wgThis	= (TRootWindowWidget) gw ;
 
 	if (wgThis->trootwin.m_wgPopup != NULL) {
-		XtDestroyWidget (wgThis->trootwin.m_wgPopup) ;
+		register Widget	wgPopup	= wgThis->trootwin.m_wgPopup ;
+
+		XtDestroyWidget (wgPopup) ;
 		wgThis->trootwin.m_wgPopup	= NULL ;
 	}
 	/*	Manage ƤΤǡפ */
 	if (wgThis->trootwin.m_wgTerminal != NULL) {
-		XtDestroyWidget (wgThis->trootwin.m_wgTerminal) ;
+		register Widget	wgTerminal	= wgThis->trootwin.m_wgTerminal ;
+
+		XtDestroyWidget (wgTerminal) ;
 		wgThis->trootwin.m_wgTerminal	= NULL ;
 	}
 	return ;
@@ -271,6 +275,9 @@ trootWindow_onKeyPress (
 {
 	if (!XtIsRealized (gw))
 		return ;
+#if defined (DEBUG)
+	fprintf (stderr, "trootWindow_onKeyPress(%p)\n", gw) ;
+#endif
 	/*	Event Select: None ȡ˸ä KeyEvent ꤲǽ
 	 *	롣*/
 	XtCallCallbacks (gw, XtNkeyPressCallback, (XtPointer) pEvent) ;
@@ -287,23 +294,6 @@ trootWindow_onKeyRelease (
 	return ;
 }
 
-/*	Terminal ؤ key Ϥ򽦤ä client ˽뤿 wrapper callback ؿ
- *	 callback  callback Ťͤơߤΰ¸٤򸺤餷ƤΤɡä
- *	ɤˤʤäΤ⤷ʤ(;_;)
- */
-void
-trootWindow_onTerminalKeyPress (
-	register Widget			gwTerinal,
-	register XtPointer		closure,
-	register XEvent*		pEvent,
-	register Boolean*		pContinueToDispatch)
-{
-	register TRootWindowWidget	wgThis		= (TRootWindowWidget) closure ;
-
-	XtCallCallbacks ((Widget) wgThis, XtNkeyPressCallback, (XtPointer) pEvent) ;
-	return ;
-}
-
 void
 trootWindow_onTerminalConfigure (
 	register Widget			gwTerminal,
@@ -492,7 +482,7 @@ trootWindow_flush (
 #if defined (DEBUG)
 			fprintf (stderr, "HaveText...\n") ;
 #endif
-			XtPopup (wgThis->trootwin.m_wgPopup, XtGrabNonexclusive) ;
+			XtPopup (wgThis->trootwin.m_wgPopup, XtGrabNone) ;
 		} else {
 #if defined (DEBUG)
 			fprintf (stderr, "No Text...\n") ;
@@ -616,8 +606,8 @@ trootWindow_popup (
 	TRootWindowWidget	wgThis	= (TRootWindowWidget) gw ;
 
 	if (!wgThis->trootwin.m_fAutoPopup ||
-		wgThis->trootwin.m_fHaveText)
-		XtPopup (wgThis->trootwin.m_wgPopup, XtGrabNonexclusive) ;
+		wgThis->trootwin.m_fHaveText) 
+		XtPopup (wgThis->trootwin.m_wgPopup, XtGrabNone) ;
 	wgThis->trootwin.m_fActive	= True ;
 	return ;
 }
@@ -635,6 +625,42 @@ trootWindow_popdown (
 	return ;
 }
 
+Window
+trootWindow_getToplevelWindow (
+	register Display*	pDisplay,
+	register Window		win)
+{
+    Atom			wm_state;
+    Atom 			type;
+    int				format;
+    unsigned long	nitems, bytesafter;
+    unsigned char*	data;
+    Window			root, parent;
+    Window*			children;
+    unsigned int	nchildren;
+
+    wm_state	= XInternAtom (pDisplay, "WM_STATE", True);
+    for (;;) {
+		type = None;
+		if (wm_state != None) {
+			data = NULL;
+			XGetWindowProperty (pDisplay, win, wm_state, 0L, 0L, False, AnyPropertyType, &type, &format, &nitems, &bytesafter, &data) ;
+			if (data != NULL)
+				XtFree((char *)data) ;
+			if (type != None)
+				break ;
+		}
+		if (!XQueryTree (pDisplay, win, &root, &parent, &children, &nchildren))
+			break ;
+		if (nchildren > 0)
+			XtFree((char *)children) ;
+		if (root == parent)
+			break ;
+		win = parent ;
+    }
+    return win;
+}
+
 /*	ƤӽФ Method
  */
 Widget
