@@ -91,6 +91,7 @@ static	Boolean	tsubWindowFrame_autoPopupp		(Widget) ;
 static	int		tsubWindowFrame_onTerminalDestroy	(void*, XEvent*) ;
 static	int		tsubWindowFrame_onTerminalConfigure	(void*, XEvent*) ;
 static	int		tsubWindowFrame_onTerminalExpose 	(void*, XEvent*) ;
+static	Window	tsubWindowFrame_getToplevelWindow	(Display*, Window) ;
 
 static	XtActionsRec	srSubWindowFrameAction []	= {
 } ;
@@ -216,6 +217,7 @@ tsubWindowFrame_onRealize (
 
 	if (!tsubWindowFrame_createTerminal (gw, wndFocus)) 
 		XtDestroyWidget (gw) ;
+
 	return ;
 }
 
@@ -270,7 +272,7 @@ tsubWindowFrame_createTerminal (
 {
 	register TSubWindowFrameWidget	wgThis = (TSubWindowFrameWidget) gw ;
 	register Display*			pDisplay ;
-	register Window				wndTerminal ;
+	register Window				wndTerminal, wndToplevel ;
 	register unsigned long		lMask ;
 	XSetWindowAttributes		xsa ;
 	register GC					gc ;
@@ -283,6 +285,9 @@ tsubWindowFrame_createTerminal (
 	wndTerminal				= XCreateWindow (pDisplay, wndFocus, gw->core.x, gw->core.y, gw->core.width, gw->core.height, 0, CopyFromParent, CopyFromParent, CopyFromParent, lMask, &xsa) ;
 	if (wndTerminal == None) 
 		return	False ;
+
+	wndToplevel		= tsubWindowFrame_getToplevelWindow (pDisplay, wgThis->tsubwin.m_wndClient) ;
+	XSetTransientForHint (pDisplay, wndTerminal, wndToplevel) ;
 	
 	/*	Focus  Resize 򸫤ơWindow  size ȤǤϤ
 	 *	ΤFocus ΥƱ礭Žդ櫓ǤϤʤΤǡ
@@ -874,9 +879,10 @@ tsubWindowFrame_onTerminalExpose (
 	wndTerminal		= wgThis->tsubwin.m_wndTerminal ;
 	pTerminal		= &wgThis->tsubwin.m_Terminal ;
 	nControl		= TVarbuffer_GetUsage  (&pTerminal->m_vbufControl) ;
-	if (nControl <= 0 || pTerminal->m_nLine <= 0)
+	if (nControl <= 0 || pTerminal->m_nLine <= 0) {
+		XClearArea (pDisplay, wndTerminal, 0, 0, wgThis->core.width, wgThis->core.height, False) ;
 		return	0 ;
-
+	}
 	pControl		= TVarbuffer_GetBuffer (&pTerminal->m_vbufControl) ;
 	pFontSet		= &wgThis->tsubwin.m_fsTerminal ;
 	gc				= wgThis->tsubwin.m_gcTerminal ;
@@ -910,10 +916,10 @@ tsubWindowFrame_onTerminalExpose (
 			TTextOut (pDisplay, wndTerminal, gc, pFontSet, x, y, pText + pControl->m_nTextPos, pControl->m_nText, &sz) ;
 			x	+= sz.cx ;
 		}
-
+		
 		/*	ιԤޤ뤫ɤΥå*/
 		if (pControl->m_fFinish) {
-			XClearArea (pDisplay, wndTerminal, x, y - nAscent, 65535, y + nDescent, False) ;
+			XClearArea (pDisplay, wndTerminal, x, y - nAscent, 65535, nAscent + nDescent, False) ;
 			nLine	-- ;
 			if (nLine <= 0)
 				break ;
@@ -925,6 +931,7 @@ tsubWindowFrame_onTerminalExpose (
 		pControl	++ ;
 		nControl	-- ;
 	}
+	XClearArea (pDisplay, wndTerminal, x, y - nAscent, 65535, nAscent + nDescent, False) ;
 	return	0 ;
 }
 
@@ -998,6 +1005,42 @@ tsubWindowFrame_onTerminalDestroy (
 	return	0 ;
 }
 
+Window
+tsubWindowFrame_getToplevelWindow (
+	register Display*	pDisplay,
+	register Window		win)
+{
+    Atom			wm_state;
+    Atom 			type;
+    int				format;
+    unsigned long	nitems, bytesafter;
+    unsigned char*	data;
+    Window			root, parent;
+    Window*			children;
+    unsigned int	nchildren;
+
+    wm_state	= XInternAtom (pDisplay, "WM_STATE", True);
+    for (;;) {
+		type = None;
+		if (wm_state != None) {
+			data = NULL;
+			XGetWindowProperty (pDisplay, win, wm_state, 0L, 0L, False, AnyPropertyType, &type, &format, &nitems, &bytesafter, &data) ;
+			if (data != NULL)
+				XtFree((char *)data) ;
+			if (type != None)
+				break ;
+		}
+		if (!XQueryTree (pDisplay, win, &root, &parent, &children, &nchildren))
+			break ;
+		if (nchildren > 0)
+			XtFree((char *)children) ;
+		if (root == parent)
+			break ;
+		win = parent ;
+    }
+    return win;
+}
+
 Widget
 TSubWindowFrame_Create (
 	Widget			wgParent,
@@ -1009,5 +1052,3 @@ TSubWindowFrame_Create (
 	return	XtCreateWidget ("TSubWindowFrame", tsubWindowFrameWidgetClass, wgParent, arglist, nArglist) ;
 }
 
-
-
