@@ -400,7 +400,7 @@ XIMInputContext_SetInputStyle (
 	register XIMStyle			imstyle)
 {
 #if defined (DEBUG) || 0
-	fprintf (stderr, "XIMInputContext_SetInputStyle (%p, %d)\n",
+	fprintf (stderr, "XIMInputContext_SetInputStyle (%p, %ld)\n",
 			 pThis, imstyle) ;
 #endif
 	pThis->m_dwInputStyle		= imstyle ;
@@ -414,7 +414,7 @@ XIMInputContext_SetClientWindow (
 	register Window				wndClient)
 {
 #if defined (DEBUG) || 0
-	fprintf (stderr, "XIMInputContext_SetClientWindow (%p, %x)\n",
+	fprintf (stderr, "XIMInputContext_SetClientWindow (%p, %lx)\n",
 			 pThis, wndClient) ;
 #endif
 	/*	client window ϰ٤Ǥʤ*/
@@ -430,7 +430,7 @@ XIMInputContext_SetFocusWindow (
 	register Window				wndFocus)
 {
 #if defined (DEBUG) || 0
-	fprintf (stderr, "XIMInputContext_SetFocusWindow (%p, %x)\n",
+	fprintf (stderr, "XIMInputContext_SetFocusWindow (%p, %lx)\n",
 			 pThis, wndFocus) ;
 #endif
 	if (!IS_CONVATTR_MASK (pThis->m_ConvAttr, CAFocusWindow) ||
@@ -2242,10 +2242,6 @@ ximInputContext_onKeyNotify (
 	struct timeval				t ;
 	struct timezone				tz ;
 
-#if defined (DEBUG)
-	fprintf (stderr, "ximInputMethod_onKeyNotify (%p, %lx)\n", pvClient, cc) ;
-#endif
-
 	wgClient	= XIMInputMethod_GetClient (pThis->m_pInputMethod) ;
 	pDisplay	= XtDisplay (wgClient) ;
 
@@ -2275,15 +2271,6 @@ ximInputContext_onKeyNotify (
 		ximInputContext_onTextNotify (pvClient, &ch, 1) ;
 		return ;
 	}
-#if defined (DEBUG) || 0
-	fprintf (stderr, "Focus(%lx), Client(%lx), Root(%lx), Child(%lx), Key(%lx)\n",
-			 IS_CONVATTR_MASK (pThis->m_ConvAttr, CAFocusWindow)? 
-			 pThis->m_ConvAttr.m_wndFocus : None,
-			 pThis->m_wndClient,
-			 wndRoot,
-			 wndChild,
-			 pThis->m_ev.xkey.window) ;
-#endif
 	gettimeofday (&t, &tz) ;
 	ev.xkey.type		= KeyPress ;
 	ev.xkey.serial		= 0 ;
