@@ -548,13 +548,16 @@ ximServer_removeClient (
 {
 	register XIMServerWidget	wgThis	= (XIMServerWidget) gw ;
 	register Widget*			pClient ;
+	register Widget*			pLast ;
 	register int				i, nClient ;
 
 	pClient	= TVarbuffer_GetBuffer (&wgThis->ximServer.m_vbufClient) ;
 	nClient	= TVarbuffer_GetUsage  (&wgThis->ximServer.m_vbufClient) ;
+	pLast	= pClient + nClient - 1 ;
 	for (i = 0 ; i < nClient ; i ++) {
 		if (*pClient == gwClient) {
-			*pClient	= *(pClient + nClient - 1) ;
+			if (pClient != pLast)
+				*pClient	= *pLast ;
 			TVarbuffer_Sub (&wgThis->ximServer.m_vbufClient, 1) ;
 			XtRemoveAllCallbacks (gwClient, XtNdestroyCallback) ;
 			return ;
