@@ -46,6 +46,9 @@ static	void	skkinputApp_onProtocolDestroy	(Widget, XtPointer, XtPointer) ;
 static	int		skkinputApp_onXIoError			(Display*) ;
 static	void	skkinputApp_onSignal			(int) ;
 static	void	skkinputApp_showUsage			(int*, char*[]) ;
+#if defined (DEBUG_CONSOLE)
+static	void	skkinputApp_testModeMainLoop	(void) ;
+#endif
 
 /*========================================================================*
  *	Хѿ
@@ -66,6 +69,9 @@ typedef struct {
 	String		m_strServerHost ;
 	int			m_nPortNum ;
 	String		m_strConfigPath ;
+#if defined (DEBUG_CONSOLE)
+	Boolean		m_fTestMode ;
+#endif
 }	SkkinputAppResRec ;
 
 static SkkinputAppResRec	theAppRes ;
@@ -92,6 +98,11 @@ static XtResource		rresSkkinputApp []	= {
 	{ "configPath",			"ConfigPath",			XtRString,
 	  sizeof (String),		offset (m_strConfigPath),
 	  XtRImmediate,			(XtPointer) ELISP_DIR, },
+#if defined (DEBUG_CONSOLE)
+	{ "testMode",			"TestMode",				XtRBoolean,
+	  sizeof (Boolean),		offset (m_fTestMode),	
+	  XtRImmediate,			(XtPointer) False, },
+#endif
 } ;
 
 #undef	offset
@@ -114,6 +125,10 @@ static XrmOptionDescRec	roptDescSkkinputApp [] = {
 	{ "-fg",		"*foreground",			XrmoptionSepArg,	NULL, },
 	{ "-bg",		"*background",			XrmoptionSepArg,	NULL, },
 	{ "-bd",		"*borderColor",			XrmoptionSepArg,	NULL, },
+#if defined (DEBUG_CONSOLE)
+	{ "-test",		".testMode",			XrmoptionNoArg,		"True", },
+	{ "+test",		".testMode",			XrmoptionNoArg,		"False", },
+#endif
 } ;
 
 int
@@ -126,27 +141,35 @@ main (int iArgc, char* rpArgv [])
 	if (! skkinputApp_initApplication (&iArgc, rpArgv))
 		return	EXIT_FAILURE ;
 
-	pOrgSigHandler		= signal (SIGHUP, SIG_IGN) ;
-	theApp.m_pLispMgr	= NULL ;
-	do {
-		theApp.m_fContinue	= False ;
-		theApp.m_fExitFlag	= False ;
+#if defined (DEBUG_CONSOLE)
+	if (theAppRes.m_fTestMode) {
+		skkinputApp_testModeMainLoop () ;
+	} else {
+#endif
+		pOrgSigHandler		= signal (SIGHUP, SIG_IGN) ;
+		theApp.m_pLispMgr	= NULL ;
+		do {
+			theApp.m_fContinue	= False ;
+			theApp.m_fExitFlag	= False ;
+			
+			if (! skkinputApp_initInstance ())
+				return	EXIT_FAILURE ;
+			
+			(void) signal (SIGHUP,  skkinputApp_onSignal) ;
+			(void) signal (SIGINT,  skkinputApp_onSignal) ;
+			(void) signal (SIGQUIT, skkinputApp_onSignal) ;
+			if (theApp.m_nProtocol > 0)
+				skkinputApp_mainLoop () ;
+			(void) signal (SIGHUP,  SIG_IGN) ;
+			(void) signal (SIGINT,  SIG_IGN) ;
+			(void) signal (SIGQUIT, SIG_IGN) ;
+			skkinputApp_exitInstance () ;
+		}	while (theApp.m_fContinue) ;
 
-		if (! skkinputApp_initInstance ())
-			return	EXIT_FAILURE ;
-
-		(void) signal (SIGHUP,  skkinputApp_onSignal) ;
-		(void) signal (SIGINT,  skkinputApp_onSignal) ;
-		(void) signal (SIGQUIT, skkinputApp_onSignal) ;
-		if (theApp.m_nProtocol > 0)
-			skkinputApp_mainLoop () ;
-		(void) signal (SIGHUP,  SIG_IGN) ;
-		(void) signal (SIGINT,  SIG_IGN) ;
-		(void) signal (SIGQUIT, SIG_IGN) ;
-		skkinputApp_exitInstance () ;
-	}	while (theApp.m_fContinue) ;
-
-	(void) signal (SIGHUP, pOrgSigHandler) ;
+		(void) signal (SIGHUP, pOrgSigHandler) ;
+#if defined (DEBUG_CONSOLE)
+	}
+#endif
 	skkinputApp_exitApplication () ;
 
 	return	EXIT_SUCCESS ; ;
@@ -411,7 +434,7 @@ skkinputApp_showUsage (
 		if (strcmp (pArgv [i], "-v") && strcmp (pArgv [i], "-version")) 
 			fprintf (stderr, "Unknown option: %s\n", pArgv [i]) ;
 	}
-	fprintf (stderr, "skkinput version 3.0_3\n") ;
+	fprintf (stderr, "skkinput version 3.0.5\n") ;
 	fprintf (stderr, "[usage]\n") ;
 	fprintf (stderr, " -v, -version:  show version\n") ;
 #if defined (SUPPORT_KINPUT)
@@ -431,3 +454,52 @@ skkinputApp_showUsage (
 	return ;
 }
 
+#if defined (DEBUG_CONSOLE)
+void
+skkinputApp_testModeMainLoop (void)
+{
+	register TLispManager*	pLispMgr ;
+	register TLispMachine*	pLM ;
+	TLispEntity*			pEntBuffer ;
+	register TLispEntity*	pEntity ;
+	char	rbuf [256] ;
+	register int			nLength ;
+
+	/*	lisp manager ν*/
+	if (TFAILED (TLispMgr_Create (&theApp.m_pLispMgr)))
+		return ;
+	pLispMgr	= theApp.m_pLispMgr ;
+	if (TFAILED (TLispMachine_Create (pLispMgr, NULL, &theApp.m_pLM)))
+		return ;
+	pLM			= theApp.m_pLM ;
+	lispMgr_CreateBuffer (pLispMgr, &pEntBuffer) ;
+	lispMachine_InsertBuffer (pLM, pEntBuffer) ;
+	lispMachineCode_SetCurrentBuffer (pLM, pEntBuffer) ;
+
+	while (! feof (stdin)) {
+		printf ("> ") ;
+		fflush (stdout) ;
+		if (fgets (rbuf, NELEMENTS (rbuf), stdin) == NULL)
+			break ;
+		nLength	= strlen (rbuf) ;
+		if (nLength > 0) {
+			while (nLength > 0 && (rbuf [nLength - 1] == '\n' || rbuf [nLength - 1] == '\r'))
+				nLength	-- ;
+			rbuf [nLength]	= '\0' ;
+		}
+		if (nLength <= 0)
+			continue ;
+
+		pEntity	= lispMgr_ParseStringA (pLispMgr, rbuf, nLength, NULL) ;
+		if (pEntity != NULL) 
+			TLispMachine_Test (pLM, pEntity) ;
+		fflush (stderr) ;
+		lispMgr_CollectGarbage (pLispMgr) ;
+		lispMgr_CollectGarbage (pLispMgr) ;
+		lispMgr_CollectGarbage (pLispMgr) ;
+		lispMgr_CollectGarbage (pLispMgr) ;
+	}
+	return ;
+}
+#endif
+
