@@ -1,8 +1,7 @@
 (use-local-map im-local-map)
 (setq lc-jp 14)
 (skk-mode)
+(make-local-hook 'post-command-hook)
 (add-hook 'post-command-hook 'im-auto-send-text t t)
-
-(local-set-key [f2] 'j-clock)
-
-
+;(princ skk-mode-hook)
+;(princ post-command-hook)
